﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Forms;
using Nedap.HandheldApi;

namespace RetailDemoApp
{
    public partial class frmMonitor : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void UpdateUIDelegate(Collection<EpcObservation> epcObservations);
        private delegate void ClearListDelegate();

        #endregion

        #region frmMonitor() - constuctor
        public frmMonitor()
        {
            InitializeComponent();

            commonMenu.MenuItems[0].MenuItems.RemoveAt(2);
            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region frmMonitor_Activated(object sender, System.EventArgs e)
        private void frmMonitor_Activated(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTags += new System.EventHandler<ReadRfidTagsEventArgs>(facade_OnReadRfidTags);
            Program.Controller.Facade.RfidMonitorStart();
        }
        #endregion

        #region frmMonitor_Deactivate(object sender, System.EventArgs e)
        private void frmMonitor_Deactivate(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTags -= new System.EventHandler<ReadRfidTagsEventArgs>(facade_OnReadRfidTags);
            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #region facade_OnReadRfidTags(object sender, DidReadRfidTagsEventArgs e)
        // When device read RFID tags display results on the form.
        private void facade_OnReadRfidTags(object sender, ReadRfidTagsEventArgs e)
        {
            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), e.EpcObservations);
        }
        #endregion

        #endregion

        #region Methods

        #region FormUpdateMethod(Collection<EpcObservation> epcObservations)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="epcObservations">EpcObservation collection.</param>
        private void FormUpdateMethod(Collection<EpcObservation> epcObservations)
        {
            lbMonitorItemsList.Items.Clear();
            for (int i = 0; i < epcObservations.Count; i++)
            {
                lbMonitorItemsList.Items.Add(string.Format("rssi {0} {1}", epcObservations[i].RSSI, epcObservations[i].EpcCode.HexString));
            }
        }
        #endregion

        #endregion
    }
}