﻿using System.Windows.Forms;

namespace RetailDemoApp
{
    public partial class frmNFCCard : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void UpdateUIDelegate(string serialNumber);

        #endregion

        #region frmNFCCard() - constructor
        public frmNFCCard()
        {
            InitializeComponent();

            commonMenu.MenuItems[0].MenuItems.RemoveAt(4);
            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region frmNFCCard_Activated(object sender, System.EventArgs e)
        private void frmNFCCard_Activated(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadNfcCard += new System.EventHandler<Nedap.HandheldApi.ReadNfcCardEventArgs>(facade_OnReadNfcCard);
            Program.Controller.Facade.NfcReadStart();
        }
        #endregion

        #region facade_OnReadNfcCard(object sender, Nedap.HandheldApi.DidReadNfcCardEventArgs e)
        // When device reads NFC card display it on the form.
        private void facade_OnReadNfcCard(object sender, Nedap.HandheldApi.ReadNfcCardEventArgs e)
        {
            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), e.CardNumber.ToString());
        }
        #endregion

        #region frmNFCCard_Deactivate(object sender, System.EventArgs e)
        private void frmNFCCard_Deactivate(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadNfcCard -= new System.EventHandler<Nedap.HandheldApi.ReadNfcCardEventArgs>(facade_OnReadNfcCard);
            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #region btnClear_Click(object sender, System.EventArgs e)
        private void btnClear_Click(object sender, System.EventArgs e)
        {
            lblSerialNumberValue.Text = string.Empty;
            Program.Controller.Facade.NfcReadStart();
        }
        #endregion

        #endregion

        #region FormUpdateMethod(string serialNumber)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="serialNumber">Serial port number.</param>
        private void FormUpdateMethod(string serialNumber)
        {
            lblSerialNumberValue.Text = serialNumber;
        }
        #endregion
    }
}