using System.Windows.Forms;
using Nedap.HandheldApi;
using Nedap.EPC.TDT;

namespace RetailDemoApp
{
    public partial class frmRetailDemo : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void UpdateUIDelegate(EpcObservation epcObservation);
        private bool highVolume = false;

        #endregion

        #region frmRetailDemo() - constructor

        public frmRetailDemo(bool isHighVolume)
        {
            InitializeComponent();
            highVolume = isHighVolume;
            if (highVolume)
            {
                commonMenu.MenuItems[0].MenuItems.RemoveAt(1);
                lblInventory.Text = "High Volume Inventory";
            }
            else
            {
                commonMenu.MenuItems[0].MenuItems.RemoveAt(0);
            }
            this.Menu = commonMenu;
            lblNumberOfItems.Text = "0 ";
        }

        #endregion

        #region Events

        #region frmRetailDemo_Activated(object sender, System.EventArgs e)
        private void frmRetailDemo_Activated(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTag += new System.EventHandler<ReadRfidTagEventArgs>(facade_OnReadRfidTag);

            if (!highVolume)
            {
                Program.Controller.Facade.RfidInventoryStart();
            }
            else
            {
                Program.Controller.Facade.RfidInventoryStart(true);
            }
        }
        #endregion

        #region facade_OnReadRfidTag(object sender, DidReadRfidTagEventArgs e)
        // When device reads RFID tags display results on the form.
        private void facade_OnReadRfidTag(object sender, ReadRfidTagEventArgs e)
        {
            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), e.EpcObservation);
        }
        #endregion

        #region frmRetailDemo_Deactivate(object sender, System.EventArgs e)
        private void frmRetailDemo_Deactivate(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTag -= new System.EventHandler<ReadRfidTagEventArgs>(facade_OnReadRfidTag);

            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #region btnClear_Click(object sender, System.EventArgs e)
        private void btnClear_Click(object sender, System.EventArgs e)
        {
            if (lbInventoryItemsList.Items.Count > 0)
            {
                lbInventoryItemsList.Items.Clear();

            }
            lblNumberOfItems.Text = "0 ";
            Program.Controller.Facade.RfidInventoryReset();
        }
        #endregion

        #endregion

        #region Methods

        #region FormUpdateMethod(EpcObservation epcObservation)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="epcObservation">EpcObservation.</param>
        private void FormUpdateMethod(EpcObservation epcObservation)
        {
            try
            {
                lbInventoryItemsList.Items.Add(epcObservation.EpcCode.PureIdentityURI);
            }
            catch (TDTTranslationException)
            {
                lbInventoryItemsList.Items.Add(epcObservation.EpcCode.HexString);
            }
            lblNumberOfItems.Text = lbInventoryItemsList.Items.Count.ToString();
        }
        #endregion

        #endregion
    }
}