﻿using System;
using System.Windows.Forms;
using System.Threading;
using Nedap.HandheldApi.CommunicationPort;

namespace RetailDemoApp
{
    public partial class frmWaitScreen : Form
    {
        private CommonMenu commonMenu = new CommonMenu();

        #region frmWaitScreen() - constructor
        public frmWaitScreen()
        {
            InitializeComponent();

            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region frmWaitScreen_Activated(object sender, EventArgs e)
        private void frmWaitScreen_Activated(object sender, EventArgs e)
        {
            Program.Controller.Session.OnSessionHandheldConnect += new EventHandler<EventArgs>(Session_OnSessionHandheldConnect);
            if (Program.Controller.Port is BluetoothCommunicationPort)
            {
                ((BluetoothCommunicationPort)Program.Controller.Port).OnMultipleHandheldsFound += new EventHandler<MultipleHandheldsFoundEventArgs>(frmWaitScreen_OnMultipleHandheldsFound);
            }
            DisableMenuItems();

            lblMessage.Text = "Discovering and connecting to !d hand.\r\nPlease wait...";
        }
        #endregion

        #region Session_OnSessionHandheldConnect(object sender, EventArgs e)
        private void Session_OnSessionHandheldConnect(object sender, EventArgs e)
        {
            this.BeginInvoke(new EventHandler(OnConnectAsync));
        }
        #endregion

        #region  frmWaitScreen_OnMultipleHandheldsFound(object sender, MultipleHandheldsFoundEventArgs e)
        void frmWaitScreen_OnMultipleHandheldsFound(object sender, MultipleHandheldsFoundEventArgs e)
        {
            ChooseReaderDialog chooseReaderDialog = new ChooseReaderDialog(e.Readers.RearedNames);
            DialogResult result = chooseReaderDialog.ShowDialog();
            e.Readers.selectedReaderIndex = (result == DialogResult.OK) ? chooseReaderDialog.lbxDiscoveredReaders.SelectedIndex : 0;
        }
        #endregion

        #region frmWaitScreen_Deactivate(object sender, EventArgs e)
        private void frmWaitScreen_Deactivate(object sender, EventArgs e)
        {
            Program.Controller.Session.OnSessionHandheldConnect -= new EventHandler<EventArgs>(Session_OnSessionHandheldConnect);
            if (Program.Controller.Port is BluetoothCommunicationPort)
            {
                ((BluetoothCommunicationPort)Program.Controller.Port).OnMultipleHandheldsFound -= new EventHandler<MultipleHandheldsFoundEventArgs>(frmWaitScreen_OnMultipleHandheldsFound);
            }
        }
        #endregion

        #endregion

        #region Methods

        private void OnConnectAsync(object sender, EventArgs e)
        {
            EnableMenuItems();
            MessageBox.Show("!d hand is paired and connected to the phone!", "RetailDemoApp", MessageBoxButtons.OK, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1);
            Program.Manager.ShowForm(Program.Manager.PreviousForm);
        }

        private void EnableMenuItems()
        {
            foreach (MenuItem item in commonMenu.MenuItems[0].MenuItems)
            {
                item.Enabled = true;
            }
        }

        private void DisableMenuItems()
        {
            foreach (MenuItem item in commonMenu.MenuItems[0].MenuItems)
            {
                if (item.Text != "Exit")
                    item.Enabled = false;
            }
        }

        #endregion
    }
}