﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using InTheHand.Net.Bluetooth;
using System.Diagnostics;
using InTheHand.Net.Sockets;
using InTheHand.Net;
using System.IO;
using System.Threading;

namespace _32FeetProof
{
    public partial class Form1 : Form
    {
        private delegate void UpdateDevicesDelegate(List<string> devices);
        private delegate void UpdateStatusLabelDelegate(string deviceStatusMessage);
        private delegate void UpdateButtonStateDelegate();

        private Thread readerThread;
        private bool readerThreadRunning;
        private int buffSize = 0xff;
        private byte[] readBuffer = new byte[0xFF];
        private int currentPosition = 0;

        private BluetoothClient btClient;
        private BluetoothAddress handheldReaderAddress;
        private string handheldReaderName;
        private Stream deviceStream;


        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Activated(object sender, EventArgs e)
        {
            StartDeviceDiscovery();
        }

        private void StartDeviceDiscovery()
        {
            BluetoothComponent bco = new BluetoothComponent();
            bco.DiscoverDevicesProgress += HandleDiscoDevicesProgress;
            bco.DiscoverDevicesComplete += HandleDiscoDevicesComplete;
            bco.DiscoverDevicesAsync(10, true, true, true, false, null);
        }

        private void HandleDiscoDevicesProgress(object sender, DiscoverDevicesEventArgs e)
        {
            List<string> devices = new List<string>();
            foreach (BluetoothDeviceInfo device in e.Devices)
            {
                devices.Add(device.DeviceName);
            }
            BeginInvoke(new UpdateDevicesDelegate(UpdateDevices), devices);
        }

        private void UpdateDevices(List<string> devices)
        {
            lbDiscoveredDevices.Items.Clear();
            foreach (string device in devices)
            {
                lbDiscoveredDevices.Items.Add(device);
            }
            Invalidate();
        }

        private void HandleDiscoDevicesComplete(object sender, DiscoverDevicesEventArgs e)
        {
            List<string> devices = new List<string>();
            foreach (BluetoothDeviceInfo device in e.Devices)
            {
                devices.Add(device.DeviceName);
                if (device.DeviceName.IndexOf("!D Hand", StringComparison.OrdinalIgnoreCase) >= 0)
                {
                    handheldReaderAddress = device.DeviceAddress;
                    handheldReaderName = device.DeviceName;
                    btnStartConnect.Enabled = true;
                }
            }
            BeginInvoke(new UpdateDevicesDelegate(UpdateDevices), devices);
            BeginInvoke(new UpdateStatusLabelDelegate(UpdateStatusLabel), e.Devices.Length.ToString() + " devices discovered.");
        }

        private void UpdateStatusLabel(string deviceStatusMessage)
        {
            lblDiscoveryProgressLabel.Text = deviceStatusMessage;
            Invalidate();
        }

        private void UpdateButtonState()
        {
            btnCheckBattery.Enabled = true;
            btnStartConnect.Enabled = false;
            Invalidate();
        }

        private void btnStartConnect_Click(object sender, EventArgs e)
        {
            BeginInvoke(new UpdateStatusLabelDelegate(UpdateStatusLabel), "Connecting...");
            Thread connectingThread = new Thread(new ThreadStart(ConnectingThreadMethod));
            connectingThread.Start();
        }
        private void ConnectingThreadMethod()
        {
            if (BluetoothSecurity.PairRequest(handheldReaderAddress, "0000"))
            {
                BluetoothEndPoint ep = new BluetoothEndPoint(handheldReaderAddress, BluetoothService.SerialPort);
                btClient = new BluetoothClient();
                btClient.Connect(ep);
                deviceStream = btClient.GetStream();
                BeginInvoke(new UpdateButtonStateDelegate(UpdateButtonState));
                BeginInvoke(new UpdateStatusLabelDelegate(UpdateStatusLabel), "Connected to " + handheldReaderName);
                readerThread = new Thread(new ThreadStart(ReaderThreadMethod));
                readerThreadRunning = true;
                readerThread.Start();
            }
        }

        private void btnCheckBattery_Click(object sender, EventArgs e)
        {
            deviceStream.Write(new byte[] { 0x06, 0x03, 0x01, 0xFC }, 0, 4);
            deviceStream.Flush();
        }

        private void ReaderThreadMethod()
        {
            try
            {
                while (readerThreadRunning)
                {
                    int bytesRead = deviceStream.Read(readBuffer, currentPosition, buffSize - currentPosition);
                    currentPosition += bytesRead;
                    TryParse(currentPosition);
                }
            }
            catch (Exception)
            {
            }
        }

        private void TryParse(int dataLength)
        {
            int j = 0;
            while (j < dataLength)
            {
                if (readBuffer[j] == 0x06) break;
                j++;
            }
            TrimBuffer(j, dataLength);

            while (readBuffer[0] == 0x06)
            {
                if (readBuffer[1] == 0 || readBuffer[1] >= dataLength)
                    return;
                switch (readBuffer[2])
                {
                    case 0x81://batery
                        BeginInvoke(new UpdateStatusLabelDelegate(UpdateStatusLabel), "Battery level: " + readBuffer[3] + "%.");
                        break;
                    case 0xC1://button pushed
                        BeginInvoke(new UpdateStatusLabelDelegate(UpdateStatusLabel), "Button pushed!");
                        break;
                }
                TrimBuffer(readBuffer[1] + 1, dataLength);
                currentPosition = 0;
            }
        }

        private void TrimBuffer(int removeCount, int dataLength)
        {
            if (removeCount > 0)
            {
                byte[] tmpBuff = readBuffer;
                readBuffer = new byte[buffSize];
                if (removeCount == dataLength)
                    return;
                Buffer.BlockCopy(tmpBuff, removeCount, readBuffer, 0, dataLength - removeCount);
            }
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            if (handheldReaderAddress != null)
            {
                BluetoothSecurity.RemoveDevice(handheldReaderAddress);
            }
            if (btClient != null)
            {
                btClient.Dispose();
            }
            readerThreadRunning = false;
            if (readerThread != null)
            {
                readerThread.Join(1000);
            }
            Close();
        }
    }
}