using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;

namespace COMportTest
{
    public partial class Form1 : Form
    {
        private SerialPort serialPort;
        private bool listenerActive;

        public Form1()
        {
            InitializeComponent();
            serialPort = new SerialPort();
        }

        private void enumeratePorts_Click(object sender, EventArgs e)
        {
            string[] ports = SerialPort.GetPortNames();
            foreach (string port in ports)
            {
                availablePorts.Items.Add(port);
            }
            availablePorts.Invalidate();
        }

        private void openPort_Click(object sender, EventArgs e)
        {
            ClosePort();
            serialPort.PortName = (string)availablePorts.SelectedItem;
            serialPort.BaudRate = 9600;
            serialPort.Parity = Parity.None;
            serialPort.DataBits = 8;
            serialPort.StopBits = StopBits.One;
            serialPort.Open();
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            if (serialPort != null && serialPort.IsOpen)
            {
                serialPort.Close();
                serialPort.Dispose();
            }
        }

        private void ClosePort()
        {
            if (serialPort != null && serialPort.IsOpen)
            {
                serialPort.Close();
            }
        }

        private void sendMessage_Click(object sender, EventArgs e)
        {
            serialPort.Write(message.Text);
            communication.Text += "\r\nS: " + message.Text;
            Invalidate();
        }

        private void DataRecievedHandler(object sender, SerialDataReceivedEventArgs e)
        {
            if (serialPort != null && serialPort.IsOpen)
            {
                Invoke(new ThreadStart(new MessageReceivedDelegate(MessageReceived)));
            }
        }
        private delegate void MessageReceivedDelegate();
        private void MessageReceived()
        {
            communication.Text += "\r\nR: " + serialPort.ReadExisting();
            Invalidate();
        }

        private void StartListen_Click(object sender, EventArgs e)
        {
            if (serialPort != null && serialPort.IsOpen && !listenerActive)
            {
                serialPort.DataReceived += new SerialDataReceivedEventHandler(DataRecievedHandler);
                listenerActive = true;
            }
        }

        private void Exit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}