﻿using System;

using System.Collections.Generic;
using System.Text;

namespace Nedap.EPC.TDT
{
    /// <summary>
    /// Class that contains methods for translating an electronic product code (EPC)
    /// from one level of representation to another.
    /// </summary>
    public class EpcConvert
    {
        /// <summary>
        /// ConvertToHexStringFromBinaryString method converts a binary string
        /// to a hexadecimal string.
        /// </summary>
        /// <param name="binary">Binary string</param>
        /// <returns>Hexadecimal string</returns>
        public static string ConvertToHexStringFromBinaryString(string binary)
        {
            // if length is not multiple of four, pad zeros
            int length = binary.Length;
            int mod = length % 4;
            int pad = 0;
            if (mod > 0)
            {
                pad = 4 - mod;
            }
            string binaryPadded = binary.PadLeft(length + pad, '0');
            int hexChars = binaryPadded.Length / 4;
            StringBuilder hexString = new StringBuilder();
            for (int i = 0; i < hexChars; i++)
            {
                if (binaryPadded.StartsWith("0000")) hexString.Append("0");
                if (binaryPadded.StartsWith("0001")) hexString.Append("1");
                if (binaryPadded.StartsWith("0010")) hexString.Append("2");
                if (binaryPadded.StartsWith("0011")) hexString.Append("3");
                if (binaryPadded.StartsWith("0100")) hexString.Append("4");
                if (binaryPadded.StartsWith("0101")) hexString.Append("5");
                if (binaryPadded.StartsWith("0110")) hexString.Append("6");
                if (binaryPadded.StartsWith("0111")) hexString.Append("7");
                if (binaryPadded.StartsWith("1000")) hexString.Append("8");
                if (binaryPadded.StartsWith("1001")) hexString.Append("9");
                if (binaryPadded.StartsWith("1010")) hexString.Append("A");
                if (binaryPadded.StartsWith("1011")) hexString.Append("B");
                if (binaryPadded.StartsWith("1100")) hexString.Append("C");
                if (binaryPadded.StartsWith("1101")) hexString.Append("D");
                if (binaryPadded.StartsWith("1110")) hexString.Append("E");
                if (binaryPadded.StartsWith("1111")) hexString.Append("F");

                binaryPadded = binaryPadded.Remove(0, 4);
            }
            return hexString.ToString();
        }
        /// <summary>
        /// ConvertToBinaryStringFromHexString method converts a hexadecimal string
        /// to a binary string.
        /// </summary>
        /// <param name="hex">Hexadecimal string</param>
        /// <returns>Binary string</returns>
        public static string ConvertToBinaryStringFromHexString(string hex)
        {
            StringBuilder binary = new StringBuilder();
            int length = hex.Length;
            for (int i = 0; i < length; i++)
            {
                if (hex.StartsWith("0")) binary.Append("0000");
                if (hex.StartsWith("1")) binary.Append("0001");
                if (hex.StartsWith("2")) binary.Append("0010");
                if (hex.StartsWith("3")) binary.Append("0011");
                if (hex.StartsWith("4")) binary.Append("0100");
                if (hex.StartsWith("5")) binary.Append("0101");
                if (hex.StartsWith("6")) binary.Append("0110");
                if (hex.StartsWith("7")) binary.Append("0111");
                if (hex.StartsWith("8")) binary.Append("1000");
                if (hex.StartsWith("9")) binary.Append("1001");
                if (hex.StartsWith("A")) binary.Append("1010");
                if (hex.StartsWith("B")) binary.Append("1011");
                if (hex.StartsWith("C")) binary.Append("1100");
                if (hex.StartsWith("D")) binary.Append("1101");
                if (hex.StartsWith("E")) binary.Append("1110");
                if (hex.StartsWith("F")) binary.Append("1111");

                hex = hex.Remove(0, 1);
            }
            return binary.ToString();
        }
        /// <summary>
        /// ConvertToBinaryStringFromData method converts an array of bytes
        /// to a binary string.
        /// </summary>
        /// <param name="data">Array of bytes</param>
        /// <returns>Binary string</returns>

        public static string ConvertToBinaryStringFromData(byte[] data)
        {
            StringBuilder binary = new StringBuilder();
            for (int i = 0; i < data.Length; i++)
            {
                for (int z = 128; z > 0; z >>= 1)
                {
                    binary.Append(((data[i] & z) == z) ? "1" : "0");
                }
            }
            return binary.ToString();
        }
        /// <summary>
        /// ConvertToDataFromHexString method converts a hexadecimal string
        /// to an array of bytes.
        /// </summary>
        /// <param name="hex">Hexadecimal string</param>
        /// <returns>Array of bytes</returns>
        public static byte[] ConvertToDataFromHexString(string hex)
        {
            List<byte> data = new List<byte>();
            if (hex.Length % 2 == 1)
            {
                hex = "0" + hex;
            }
            for (int i = 0; i < hex.Length; i += 2)
            {
                data.Add(Convert.ToByte(hex.Substring(i, 2), 16));
            }
            return data.ToArray();
        }

        /// <summary>
        /// ConvertToHexStringFromData method converts an array of bytes
        /// to a hexadecimal string.
        /// </summary>
        /// <param name="data">Array of bytes</param>
        /// <returns>Hexadecimal string</returns>
        public static string ConvertToHexStringFromData(byte[] data)
        {
            StringBuilder hexOutput = new StringBuilder();
            foreach (byte word in data)
            {
                hexOutput.Append(String.Format("{0:X2}", word));
            }
            return hexOutput.ToString();
        }
    }
}
