﻿using System;

using System.Collections.Generic;
using System.Text;
using InTheHand.Net;

namespace Nedap.HandheldApi.CommunicationPort
{
    /// <summary>
    /// Structure that holds discovered readers and selected handheld reader.
    /// </summary>
    public struct DiscoveredReaders
    {
        /// <summary>
        /// Index of selected hadheld reader that should be set by event handler.
        /// </summary>
        public int selectedReaderIndex;

        /// <summary>
        /// Dictioanry that contains found readers and it
        /// </summary>
        public List<string> RearedNames { get; set; }
    }
    
    /// <summary>
    /// These event arguments carries informationa about discovered handheld readers
    /// in case that there is more than one handheld reader present. Handler of this
    /// event should set SelectedDevice index in order to select one of the readers
    /// for connecting.
    /// </summary>
    public class MultipleHandheldsFoundEventArgs : EventArgs
    {
        /// <summary>
        /// Property that holds discovered handheld readers and the one that should be selected (used for conneciton).
        /// </summary>
        public DiscoveredReaders Readers;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="readers"></param>
        public MultipleHandheldsFoundEventArgs(DiscoveredReaders readers)
        {
            Readers = readers;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public MultipleHandheldsFoundEventArgs()
        {
            Readers = new DiscoveredReaders();
        }
    }
}
