﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Nedap.HandheldApi.CommunicationQueue
{
    /// <summary>
    /// Interface to communication port.
    /// </summary>
    public interface ICommunicationPort : IDisposable
    {
        /// <summary>
        /// Connect to port
        /// </summary>
        /// <returns>True if success</returns>
        bool Connect();

        /// <summary>
        /// Disconnect from port
        /// </summary>
        /// <returns>True if success</returns>
        bool Disconnect();
        
        /// <summary>
        /// Defines if port is connected
        /// </summary>
        bool Connected { get; }

        /// <summary>
        /// Writes data to port
        /// </summary>
        /// <param name="buffer">Data buffer</param>
        /// <param name="offset">Position where to start writing</param>
        /// <param name="length">Number of bytes to write</param>
        /// <returns></returns>
        int WriteData(byte[] buffer, int offset, int length);

        /// <summary>
        /// Reads data from port, if no data is available it should return with 0 result.
        /// </summary>
        /// <param name="buffer">Buffer to read into</param>
        /// <param name="offset">Position where to start writing</param>
        /// <param name="length">Maximum number of bytes to read</param>
        /// <returns>Actual read bytes, which is possible to be 0 if no data is available</returns>
        int ReadData(byte[] buffer, int offset, int length);

        /// <summary>
        /// Event thrown when data is available to read.
        /// </summary>
        event EventHandler OnDataReceived;
    }
}
