﻿using System;
using System.ComponentModel;

namespace Nedap.HandheldApi
{
    /// <summary>
    /// Interface to connection state.
    /// </summary>
    internal interface IConnectionState : IDisposable
    {
        /// <summary>
        /// Checking interval (in miliseconds)
        /// </summary>
        int CheckInterval { get; set; }

        /// <summary>
        /// Maximum time that can pass without device answer before sending command to device (in miliseconds)
        /// </summary>
        int MaxWithNoAnswer { get; set; }

        /// <summary>
        /// Connection status.
        /// </summary>
        /// <returns>True if connected</returns>
        bool CheckConnection();

        /// <summary>
        /// Event thrown when connection state changes.
        /// </summary>
        event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Event thrown when device connects.
        /// </summary>
        event EventHandler<EventArgs> DeviceConnected;

        /// <summary>
        /// Event thrown when device disconnects.
        /// </summary>
        event EventHandler<EventArgs> DeviceDisconnected;
    }
}