﻿
namespace Nedap.HandheldApi
{
    /// <summary>
    /// Region where the handheld will be used. Please note that not all regions might be supported by the current hardware.
    /// </summary>
    public enum RfidRegion
    {
        Undefined		= 0,
        Europe		    = 1,
        NorthAmerica	= 2,
        Korea			= 4,
        China			= 8
    }

    /// <summary>
    /// RFID frequency values supported by the handheld.
    /// </summary>
    public enum RfidFrequency
    {
        Europe865700 = 865700,
        Europe866300 = 866300,
        Europe866900 = 866900,
        Europe867500 = 867500
    }

    /// <summary>
    /// Barcode types that will be read by the barcode reader.
    /// </summary>
    
    public enum BarcodeType
    {
        All				= 0,
        Ean8			= 1,
        Ean13			= 2,
        Upca			= 3,
        Upce			= 4,
        Ean128			= 5,
        Codabar			= 6,
        Code11			= 7,
        Code39			= 8,
        Code93			= 9,
        EanBookland		= 10,
        Discrete2Of5	= 11,
        Interleaved2Of5	= 12,
        Code128			= 13,
        Msi				= 14,
        GS1Databar		= 15,
        Unknown			= 0xFF
    }

    /// <summary>
    /// RFID memory bank values supported by the handheld.
    /// </summary>    
    public enum RfidEpcMemoryBank
    {
        Reserved	= 0,
        Epc		    = 1,
        Tid		    = 2,
        UserMemory	= 3
    }

    /// <summary>
    /// RFID EPC session values supported by the handheld.
    /// </summary>
    public enum RfidEpcSession
    {
        Session0 = 0,
        Session1 = 1,
        Session2 = 2,
        Session3 = 3
    }

    /// <summary>
    /// RFID EPC target values supported by the handheld.
    /// </summary>
    public enum RfidEpcTarget
    {
        TargetA = 0,
        TargetB = 1
    }

    /// <summary>
    /// RFID EPC m values supported by the handheld.
    /// </summary>    
    public enum RfidEpcMValue
    {
        Value0 = 0,
        Value2 = 1,
        Value4 = 2,
        Value8 = 3
    }

    /// <summary>
    /// Tag manufacturer identification. Available on the internet at http://www.epcglobalinc.org/standards/mdid/.
    /// </summary>    
    public enum RfidEpcMdid
    {
        Impinj				= 0x001,
        TexasInstruments     = 0x002,
        AlienTechnology      = 0x003,
        Intelleflex          = 0x004,
        Atmel                = 0x005,
        Nxp                  = 0x006,
        STMicroelectronics   = 0x007,
        EPMicroelectronics   = 0x008,
        Motorola             = 0x009,
        SentechSndBhd        = 0x00A,
        EMMicroelectronics   = 0x00B,
        RenesasTechnology    = 0x00C,
        Mstar                = 0x00D,
        TycoInternational    = 0x00E,
        QuanrayElectronics   = 0x00F,
        Fujitsu              = 0x010,
        Lsis                 = 0x011,
        CaenRfid             = 0x012,
        Peg                  = 0x013
    }

    /// <summary>
    /// Tune values supported by the handheld.
    /// </summary>
    public enum UITune
    {
        Short	= 0,
        Success	= 1,
        Failure	= 2,
    }
}
