﻿using System;
using Nedap.EPC.TDT;

namespace Nedap.HandheldApi
{
    public class EpcObservation : Object
    {
        /// <summary>
        /// EPC code
        /// </summary>
        public EpcCode EpcCode { get; set; }
        /// <summary>
        /// Time stamp
        /// </summary>
        public DateTime TimeStamp { get; set; }
        /// <summary>
        /// RSSI
        /// </summary>
        public int RSSI { get; set; }
        /// <summary>
        /// Antenna
        /// </summary>
        public int Antenna { get; set; }
        /// <summary>
        /// Read count in the current read cycle. 
        /// Since most readcycles will be smaller than one second, only in Session 0 this value will be more than one.
        /// </summary>
        public int ReadCount { get; set; }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="epc"></param>
        /// <param name="timeStamp"></param>
        /// <param name="rssi"></param>
        /// <param name="readCount"></param>
        public EpcObservation(EpcCode epc, DateTime timeStamp, int rssi, int readCount)
        {
            this.EpcCode = epc;
            TimeStamp = timeStamp;
            RSSI = rssi;
            ReadCount = readCount;
        }

        /// <summary>
        /// Equals
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public override bool Equals(object obj)
        {
            if (obj == null || this.GetType() != obj.GetType())
            {
                return false;
            }
            EpcObservation nhepcObservation = (EpcObservation)obj;
            return IsEPCCodeEqual(nhepcObservation);
        }

        /// <summary>
        /// Hash code
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return EpcCode.GetHashCode();
        }

        /// <summary>
        /// Compares EPC code of given observation with EPC code of this observation.
        /// </summary>
        /// <param name="nhepcObservation"></param>
        /// <returns></returns>
        private bool IsEPCCodeEqual(EpcObservation nhepcObservation)
        {
            return this.EpcCode.Equals(nhepcObservation.EpcCode);
        }

    }
}