using System;
using System.Collections;
using Nedap.HandheldApi;

[Serializable]
internal sealed class PresentationContext : statemap.FSMContext
{
#if DEBUG
    // Create a logger for use in this class
    private static readonly log4net.ILog log = log4net.LogManager.GetLogger(typeof(PresentationContext));
#endif
    //---------------------------------------------------------------
    // Properties.
    //



    #region Events

    public event EventHandler StateChanged;

    #endregion

    public PresentationState State
    {
        get
        {
            if (_state == null)
            {
                throw (
                    new statemap.StateUndefinedException());
            }

            return ((PresentationState)_state);
        }
        set
        {
#if DEBUG
            log.Debug("NEW STATE: " + ((value != null) ? value.Name : "NULL"));
#endif
            _state = value;
            if (StateChanged != null)
            {
                StateChanged(this, EventArgs.Empty);
            }
        }
    }

    internal Presentation Owner
    {
        get
        {
            return (_owner);
        }
        set
        {
            _owner = value;
        }
    }

    //---------------------------------------------------------------
    // Member methods.
    //

    internal PresentationContext(Presentation owner) :
        base()
    {
        _owner = owner;
        _state = MainMap.Disconnected;
        MainMap.Disconnected.Entry(this);
    }

    public void barcodeReadFail()
    {
        _transition = "barcodeReadFail";
        State.barcodeReadFail(this);
        _transition = "";
        return;
    }

    public void barcodeReadStart()
    {
        _transition = "barcodeReadStart";
        State.barcodeReadStart(this);
        _transition = "";
        return;
    }

    public void barcodeReadSuccess()
    {
        _transition = "barcodeReadSuccess";
        State.barcodeReadSuccess(this);
        _transition = "";
        return;
    }

    public void connected()
    {
        _transition = "connected";
        State.connected(this);
        _transition = "";
        return;
    }

    public void disconnected()
    {
        _transition = "disconnected";
        State.disconnected(this);
        _transition = "";
        return;
    }

    public void handheldStop()
    {
        _transition = "handheldStop";
        State.handheldStop(this);
        _transition = "";
        return;
    }

    public void nfcCardFound()
    {
        _transition = "nfcCardFound";
        State.nfcCardFound(this);
        _transition = "";
        return;
    }

    public void nfcNoCardFound()
    {
        _transition = "nfcNoCardFound";
        State.nfcNoCardFound(this);
        _transition = "";
        return;
    }

    public void nfcReadErrorTimer()
    {
        _transition = "nfcReadErrorTimer";
        State.nfcReadErrorTimer(this);
        _transition = "";
        return;
    }

    public void nfcReadRetry()
    {
        _transition = "nfcReadRetry";
        State.nfcReadRetry(this);
        _transition = "";
        return;
    }

    public void nfcReadStart()
    {
        _transition = "nfcReadStart";
        State.nfcReadStart(this);
        _transition = "";
        return;
    }

    public void rfidActionFail()
    {
        _transition = "rfidActionFail";
        State.rfidActionFail(this);
        _transition = "";
        return;
    }

    public void rfidActionFailMoreTags()
    {
        _transition = "rfidActionFailMoreTags";
        State.rfidActionFailMoreTags(this);
        _transition = "";
        return;
    }

    public void rfidActionSuccess()
    {
        _transition = "rfidActionSuccess";
        State.rfidActionSuccess(this);
        _transition = "";
        return;
    }

    public void rfidActionSuccessMoreTags()
    {
        _transition = "rfidActionSuccessMoreTags";
        State.rfidActionSuccessMoreTags(this);
        _transition = "";
        return;
    }

    public void rfidInventoryContinue()
    {
        _transition = "rfidInventoryContinue";
        State.rfidInventoryContinue(this);
        _transition = "";
        return;
    }

    public void rfidInventoryIdleTimer()
    {
        _transition = "rfidInventoryIdleTimer";
        State.rfidInventoryIdleTimer(this);
        _transition = "";
        return;
    }
    public void rfidInventoryLowPowerStart()
    {
        _transition = "rfidInventoryLowPowerStart";
        State.rfidInventoryLowPowerStart(this);
        _transition = "";
        return;
    }

    public void rfidInventoryPause()
    {
        _transition = "rfidInventoryPause";
        State.rfidInventoryPause(this);
        _transition = "";
        return;
    }

    public void rfidInventoryReadErrorTimer()
    {
        _transition = "rfidInventoryReadErrorTimer";
        State.rfidInventoryReadErrorTimer(this);
        _transition = "";
        return;
    }

    public void rfidInventoryStart(bool highVolume)
    {
        _transition = "rfidInventoryStart";
        State.rfidInventoryStart(this, highVolume);
        _transition = "";
        return;
    }

    public void rfidMonitorContinue()
    {
        _transition = "rfidMonitorContinue";
        State.rfidMonitorContinue(this);
        _transition = "";
        return;
    }

    public void rfidMonitorIdleTimer()
    {
        _transition = "rfidMonitorIdleTimer";
        State.rfidMonitorIdleTimer(this);
        _transition = "";
        return;
    }

    public void rfidMonitorPause()
    {
        _transition = "rfidMonitorPause";
        State.rfidMonitorPause(this);
        _transition = "";
        return;
    }

    public void rfidMonitorReadErrorTimer()
    {
        _transition = "rfidMonitorReadErrorTimer";
        State.rfidMonitorReadErrorTimer(this);
        _transition = "";
        return;
    }

    public void rfidMonitorStart()
    {
        _transition = "rfidMonitorStart";
        State.rfidMonitorStart(this);
        _transition = "";
        return;
    }

    public void rfidReadDataStart()
    {
        _transition = "rfidReadDataStart";
        State.rfidReadDataStart(this);
        _transition = "";
        return;
    }

    public void rfidReadDetailedTagInformationStart()
    {
        _transition = "rfidReadDetailedTagInformationStart";
        State.rfidReadDetailedTagInformationStart(this);
        _transition = "";
        return;
    }

    public void rfidReadFinished()
    {
        _transition = "rfidReadFinished";
        State.rfidReadFinished(this);
        _transition = "";
        return;
    }

    public void rfidReadFinishedMoreThanZero()
    {
        _transition = "rfidReadFinishedMoreThanZero";
        State.rfidReadFinishedMoreThanZero(this);
        _transition = "";
        return;
    }

    public void rfidReadFinishedZero()
    {
        _transition = "rfidReadFinishedZero";
        State.rfidReadFinishedZero(this);
        _transition = "";
        return;
    }

    public void rfidSingleReadReadErrorTimer()
    {
        _transition = "rfidSingleReadReadErrorTimer";
        State.rfidSingleReadReadErrorTimer(this);
        _transition = "";
        return;
    }

    public void rfidSingleReadStart()
    {
        _transition = "rfidSingleReadStart";
        State.rfidSingleReadStart(this);
        _transition = "";
        return;
    }
    public void rfidWriteEPCNoTagsInField()
    {
        _transition = "rfidWriteEPCNoTagsInField";
        State.rfidWriteEPCNoTagsInField(this);
        _transition = "";
        return;
    }

    public void rfidWriteEPCStart()
    {
        _transition = "rfidWriteEPCStart";
        State.rfidWriteEPCStart(this);
        _transition = "";
        return;
    }

    public void rfidWriteEPCTooMuchTagsInField()
    {
        _transition = "rfidWriteEPCTooMuchTagsInField";
        State.rfidWriteEPCTooMuchTagsInField(this);
        _transition = "";
        return;
    }

    public void rfidWriteEPCWriteSuccessful()
    {
        _transition = "rfidWriteEPCWriteSuccessful";
        State.rfidWriteEPCWriteSuccessful(this);
        _transition = "";
        return;
    }

    public void rfidWriteEPCWriteToTagFailed()
    {
        _transition = "rfidWriteEPCWriteToTagFailed";
        State.rfidWriteEPCWriteToTagFailed(this);
        _transition = "";
        return;
    }
    public void uiButtonPushed()
    {
        _transition = "uiButtonPushed";
        State.uiButtonPushed(this);
        _transition = "";
        return;
    }

    //---------------------------------------------------------------
    // Member data.
    //

    [NonSerialized]
    private Presentation _owner;

    //---------------------------------------------------------------
    // Inner classes.
    //

    public abstract class PresentationState : statemap.State
    {
        //-----------------------------------------------------------
        // Member methods.
        //

        internal PresentationState(string name, int id) :
            base(name, id)
        { }

        protected internal virtual void Entry(PresentationContext context)
        { }

        protected internal virtual void Exit(PresentationContext context)
        { }

        protected internal virtual void barcodeReadFail(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void barcodeReadStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void barcodeReadSuccess(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void connected(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void disconnected(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void handheldStop(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void nfcCardFound(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void nfcNoCardFound(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void nfcReadErrorTimer(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void nfcReadRetry(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void nfcReadStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidActionFail(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidActionFailMoreTags(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidActionSuccess(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidActionSuccessMoreTags(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidInventoryContinue(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidInventoryIdleTimer(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidInventoryLowPowerStart(PresentationContext context)
        {
            Default(context);
        }
        protected internal virtual void rfidInventoryPause(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidInventoryReadErrorTimer(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidInventoryStart(PresentationContext context, bool highVolume)
        {
            Default(context);
        }

        protected internal virtual void rfidMonitorContinue(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidMonitorIdleTimer(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidMonitorPause(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidMonitorReadErrorTimer(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidMonitorStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidReadDataStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidReadDetailedTagInformationStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidReadFinished(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidReadFinishedMoreThanZero(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidReadFinishedZero(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidSingleReadReadErrorTimer(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidSingleReadStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidWriteEPCNoTagsInField(PresentationContext context)
        {
            Default(context);
        }
        protected internal virtual void rfidWriteEPCStart(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidWriteEPCTooMuchTagsInField(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidWriteEPCWriteSuccessful(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void rfidWriteEPCWriteToTagFailed(PresentationContext context)
        {
            Default(context);
        }
        protected internal virtual void uiButtonPushed(PresentationContext context)
        {
            Default(context);
        }

        protected internal virtual void Default(PresentationContext context)
        {
            throw (
                new statemap.TransitionUndefinedException(
                    "State: " +
                    context.State.Name +
                    ", Transition: " +
                    context.GetTransition()));
        }
    }

    internal abstract class MainMap
    {
        //-----------------------------------------------------------
        // Member methods.
        //

        //-----------------------------------------------------------
        // Member data.
        //

        //-------------------------------------------------------
        // Statics.
        //
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_Disconnected Disconnected =
            new MainMap_Default.MainMap_Disconnected("MainMap.Disconnected", 0);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_Connected Connected =
            new MainMap_Default.MainMap_Connected("MainMap.Connected", 1);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_NFCRead NFCRead =
            new MainMap_Default.MainMap_NFCRead("MainMap.NFCRead", 2);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_NFCReadIdle NFCReadIdle =
            new MainMap_Default.MainMap_NFCReadIdle("MainMap.NFCReadIdle", 3);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDSingleReadWait RFIDSingleReadWait =
            new MainMap_Default.MainMap_RFIDSingleReadWait("MainMap.RFIDSingleReadWait", 4);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDSingleReadRead RFIDSingleReadRead =
            new MainMap_Default.MainMap_RFIDSingleReadRead("MainMap.RFIDSingleReadRead", 5);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDInventoryWait RFIDInventoryWait =
            new MainMap_Default.MainMap_RFIDInventoryWait("MainMap.RFIDInventoryWait", 6);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDInventoryRead RFIDInventoryRead =
            new MainMap_Default.MainMap_RFIDInventoryRead("MainMap.RFIDInventoryRead", 7);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDInventoryIdle RFIDInventoryIdle =
            new MainMap_Default.MainMap_RFIDInventoryIdle("MainMap.RFIDInventoryIdle", 8);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDMonitorWait RFIDMonitorWait =
            new MainMap_Default.MainMap_RFIDMonitorWait("MainMap.RFIDMonitorWait", 9);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDMonitorRead RFIDMonitorRead =
            new MainMap_Default.MainMap_RFIDMonitorRead("MainMap.RFIDMonitorRead", 10);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDMonitorIdle RFIDMonitorIdle =
            new MainMap_Default.MainMap_RFIDMonitorIdle("MainMap.RFIDMonitorIdle", 11);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDWriteEPCWait RFIDWriteEPCWait =
            new MainMap_Default.MainMap_RFIDWriteEPCWait("MainMap.RFIDWriteEPCWait", 12);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDWriteEPCWrite RFIDWriteEPCWrite =
            new MainMap_Default.MainMap_RFIDWriteEPCWrite("MainMap.RFIDWriteEPCWrite", 13);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDReadDetailedTagInformationWait RFIDReadDetailedTagInformationWait =
            new MainMap_Default.MainMap_RFIDReadDetailedTagInformationWait("MainMap.RFIDReadDetailedTagInformationWait", 14);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_RFIDReadDetailedTagInformationStart RFIDReadDetailedTagInformationStart =
            new MainMap_Default.MainMap_RFIDReadDetailedTagInformationStart("MainMap.RFIDReadDetailedTagInformationStart", 15);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_BarcodeReadWait BarcodeReadWait =
            new MainMap_Default.MainMap_BarcodeReadWait("MainMap.BarcodeReadWait", 16);
        [NonSerialized]
        internal static readonly MainMap_Default.MainMap_BarcodeReadStart BarcodeReadStart =
            new MainMap_Default.MainMap_BarcodeReadStart("MainMap.BarcodeReadStart", 17);
        [NonSerialized]
        private static readonly MainMap_Default Default =
            new MainMap_Default("MainMap.Default", -1);

    }

    internal class MainMap_Default :
        PresentationState
    {
        //-----------------------------------------------------------
        // Member methods.
        //

        internal MainMap_Default(string name, int id) :
            base(name, id)
        { }

        protected internal override void disconnected(PresentationContext context)
        {

            Presentation ctxt = context.Owner;


            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent("disconnected");
                ctxt.ActionHandheldDisconnected();
            }
            finally
            {
                context.State = MainMap.Disconnected;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void handheldStop(PresentationContext context)
        {

            context.State.Exit(context);
            context.State = MainMap.Connected;
            context.State.Entry(context);
            return;
        }

        protected internal override void nfcReadStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;


            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogTimedEvent("nfcReadStart");
                ctxt.ActionUILedSetBlinking(true);
            }
            finally
            {
                context.State = MainMap.NFCRead;
                context.State.Entry(context);
            }

            return;
        }
        protected internal override void rfidInventoryStart(PresentationContext context, bool highVolume)
        {

            Presentation ctxt = context.Owner;
            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent((highVolume) ? "rfidInventoryHighVolumeStart" : "rfidInventoryStart");
                ctxt.ActionRfidSetTimeout(400);
                ctxt.ActionRfidSetMValue(RfidEpcMValue.Value4);
                ctxt.ActionRfidSetSession((highVolume) ? RfidEpcSession.Session2 : RfidEpcSession.Session1);
                ctxt.ActionRfidSetTarget(RfidEpcTarget.TargetA, false);
                ctxt.ActionRfidSetQValue(0, true);
                ctxt.ActionRfidSetReadPower(23);
            }
            finally
            {
                context.State = MainMap.RFIDInventoryWait;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void rfidInventoryLowPowerStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;


            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent(@"rfidInventoryLowPowerStart");
                ctxt.ActionRfidSetTimeout(400);
                ctxt.ActionRfidSetMValue(RfidEpcMValue.Value4);
                ctxt.ActionRfidSetSession(RfidEpcSession.Session1);
                ctxt.ActionRfidSetTarget(RfidEpcTarget.TargetA, false);
                ctxt.ActionRfidSetQValue(0, true);
                ctxt.ActionRfidSetReadPower(20);
            }
            finally
            {
                context.State = MainMap.RFIDInventoryWait;
                context.State.Entry(context);
            }

            return;
        }
        protected internal override void rfidMonitorStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;
            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent("rfidMonitorStart");
                ctxt.ActionRfidSetTimeout(300);
                ctxt.ActionRfidSetMValue(RfidEpcMValue.Value4);
                ctxt.ActionRfidSetSession(RfidEpcSession.Session0);
                ctxt.ActionRfidSetTarget(RfidEpcTarget.TargetA, true);
                ctxt.ActionRfidSetReadPower(23);
            }
            finally
            {
                context.State = MainMap.RFIDMonitorWait;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void rfidSingleReadStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;
            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent("rfidSingleReadStart");
                ctxt.ActionRfidSetTimeout(200);
                ctxt.ActionRfidSetMValue(RfidEpcMValue.Value4);
                ctxt.ActionRfidSetSession(RfidEpcSession.Session0);
                ctxt.ActionRfidSetTarget(RfidEpcTarget.TargetA, false);
                ctxt.ActionRfidSetReadPower(10);
            }
            finally
            {
                context.State = MainMap.RFIDSingleReadWait;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void rfidWriteEPCStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;
            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent("rfidWriteEPCStart");
                ctxt.ActionRfidSetTimeout(150);
                ctxt.ActionRfidSetMValue(RfidEpcMValue.Value8);
                ctxt.ActionRfidSetSession(RfidEpcSession.Session0);
                ctxt.ActionRfidSetTarget(RfidEpcTarget.TargetA, true);
            }
            finally
            {
                context.State = MainMap.RFIDWriteEPCWait;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void rfidReadDetailedTagInformationStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;


            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent("rfidReadDetailedTagInformationStart");
                ctxt.ActionRfidSetTimeout(100);
                ctxt.ActionRfidSetMValue(RfidEpcMValue.Value4);
                ctxt.ActionRfidSetSession(RfidEpcSession.Session0);
                ctxt.ActionRfidSetTarget(RfidEpcTarget.TargetA, true);
            }
            finally
            {
                context.State = MainMap.RFIDReadDetailedTagInformationWait;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void barcodeReadStart(PresentationContext context)
        {

            Presentation ctxt = context.Owner;
            context.State.Exit(context);
            context.ClearState();

            try
            {
                ctxt.ActionLogEvent("barcodeReadStart");
            }
            finally
            {
                context.State = MainMap.BarcodeReadWait;
                context.State.Entry(context);
            }

            return;
        }

        protected internal override void uiButtonPushed(PresentationContext context)
        {

            Presentation ctxt = context.Owner;

            PresentationState endState = context.State;

            context.ClearState();

            try
            {
                ctxt.DoPushButton();
                ctxt.ActionLogEvent("uiButtonPushed");
            }
            finally
            {
                context.State = endState;
            }

            return;
        }

        protected internal override void rfidReadFinishedMoreThanZero(PresentationContext context)
        {

            return;
        }

        protected internal override void rfidReadFinishedZero(PresentationContext context)
        {

            return;
        }

        protected internal override void Default(PresentationContext context)
        {

            return;
        }

        //-----------------------------------------------------------
        // Inner classes.
        //

        internal class MainMap_Disconnected :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_Disconnected(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;
                ctxt.ActionLogEndCurrentTimedEvent();
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void connected(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEvent("connected");
                    ctxt.ActionHandheldConnected();
                    ctxt.ActionHandheldConfigure();
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void disconnected(PresentationContext context)
            {

                return;
            }

            protected internal override void nfcReadStart(PresentationContext context)
            {
                return;
            }

            protected internal override void rfidInventoryStart(PresentationContext context, bool highVolume)
            {
                return;
            }

            protected internal override void rfidMonitorStart(PresentationContext context)
            {
                return;
            }

            protected internal override void rfidReadDataStart(PresentationContext context)
            {
                return;
            }

            protected internal override void rfidSingleReadStart(PresentationContext context)
            {
                return;
            }

            protected internal override void rfidWriteEPCStart(PresentationContext context)
            {
                return;
            }
        }

        internal class MainMap_Connected :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_Connected(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;
                ctxt.ActionLogEndCurrentTimedEvent();
                ctxt.ActionUILedSetBlinking(false);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {
                return;
            }

            protected internal override void disconnected(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEvent("disconnected");
                    ctxt.ActionHandheldDisconnected();
                }
                finally
                {
                    context.State = MainMap.Disconnected;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_NFCRead :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_NFCRead(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StartTimer(Presentation.PresentationTimerMethod.NfcReadErrorTimer, 2000);
                ctxt.ActionNfcTryToReadCard();
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.NfcReadErrorTimer);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void nfcCardFound(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEvent("nfcCardFound");
                    ctxt.ActionUITune(UITune.Success);
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void nfcNoCardFound(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.NFCReadIdle;
                context.State.Entry(context);
                return;
            }

            protected internal override void nfcReadErrorTimer(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogError("nfcReadErrorTimer");
                }
                finally
                {
                    context.State = MainMap.NFCRead;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_NFCReadIdle :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_NFCReadIdle(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StartTimer(Presentation.PresentationTimerMethod.NfcReadRetry, 500);
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.NfcReadRetry);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void nfcReadRetry(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.NFCRead;
                context.State.Entry(context);
                return;
            }
        }

        internal class MainMap_RFIDSingleReadWait :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDSingleReadWait(string name, int id) :
                base(name, id)
            { }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDSingleReadRead;
                context.State.Entry(context);
                return;
            }
        }

        internal class MainMap_RFIDSingleReadRead :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDSingleReadRead(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.ActionRfidReadEpc();
                ctxt.StartTimer(Presentation.PresentationTimerMethod.RfidSingleReadReadErrorTimer, 2000);
                ctxt.ActionLogEvent("rfidSingleRead");
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.RfidSingleReadReadErrorTimer);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidReadFinishedMoreThanZero(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Success);
                    ctxt.ActionLogEvent("rfidSingleReadMoreThanZero");
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidReadFinishedZero(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                    ctxt.ActionLogEvent("rfidSingleReadZero");
                }
                finally
                {
                    context.State = MainMap.RFIDSingleReadWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidSingleReadReadErrorTimer(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                    ctxt.ActionLogEvent("rfidSingleReadReadErrorTimer");
                }
                finally
                {
                    context.State = MainMap.RFIDSingleReadWait;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDInventoryWait :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDInventoryWait(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.ActionUILedSetBlinking(false);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidInventoryContinue(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUILedSetBlinking(true);
                    ctxt.ActionLogTimedEvent("rfidInventory");
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryRead;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUILedSetBlinking(true);
                    ctxt.DoContinue();
                    ctxt.ActionLogTimedEvent("rfidInventory");
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryRead;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDInventoryRead :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDInventoryRead(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StartTimer(Presentation.PresentationTimerMethod.RfidInventoryReadErrorTimer, 5000);
                ctxt.ActionRfidReadEpc();
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.RfidInventoryReadErrorTimer);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidInventoryPause(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidInventory");
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidInventoryReadErrorTimer(PresentationContext context)
            {

                Presentation ctxt = context.Owner;

                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogError("rfidInventoryReadErrorTimer");
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryRead;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidReadFinished(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDInventoryIdle;
                context.State.Entry(context);
                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidInventory");
                    ctxt.DoPause();
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryWait;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDInventoryIdle :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDInventoryIdle(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;
                ctxt.ActionRfidSwitchSession();
                ctxt.StartTimer(Presentation.PresentationTimerMethod.RfidInventoryIdleTimer, ctxt.RFIDOffTime);
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.RfidInventoryIdleTimer);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidInventoryIdleTimer(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDInventoryRead;
                context.State.Entry(context);
                return;
            }

            protected internal override void rfidInventoryPause(PresentationContext context)
            {
                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidInventory");
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidInventory");
                    ctxt.DoPause();
                }
                finally
                {
                    context.State = MainMap.RFIDInventoryWait;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDMonitorWait :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDMonitorWait(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.ActionUILedSetBlinking(false);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidMonitorContinue(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUILedSetBlinking(true);
                    ctxt.ActionLogTimedEvent("rfidMonitor");
                }
                finally
                {
                    context.State = MainMap.RFIDMonitorRead;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUILedSetBlinking(true);
                    ctxt.DoContinue();
                    ctxt.ActionLogTimedEvent("rfidMonitor");
                }
                finally
                {
                    context.State = MainMap.RFIDMonitorRead;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDMonitorRead :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDMonitorRead(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StartTimer(Presentation.PresentationTimerMethod.RfidMonitorReadErrorTimer, 5000);
                ctxt.ActionRfidReadEpc();
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.RfidMonitorReadErrorTimer);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidMonitorPause(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidMonitor");
                }
                finally
                {
                    context.State = MainMap.RFIDMonitorWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidMonitorReadErrorTimer(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDMonitorRead;
                context.State.Entry(context);
                return;
            }

            protected internal override void rfidReadFinished(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDMonitorIdle;
                context.State.Entry(context);
                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidMonitor");
                    ctxt.DoPause();
                }
                finally
                {
                    context.State = MainMap.RFIDMonitorWait;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDMonitorIdle :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDMonitorIdle(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StartTimer(Presentation.PresentationTimerMethod.RfidMonitorIdleTimer, ctxt.RFIDOffTime);
                return;
            }

            protected internal override void Exit(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.StopTimer(Presentation.PresentationTimerMethod.RfidMonitorIdleTimer);
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidMonitorIdleTimer(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDMonitorRead;
                context.State.Entry(context);
                return;
            }

            protected internal override void rfidMonitorPause(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidMonitor");
                }
                finally
                {
                    context.State = MainMap.RFIDMonitorWait;
                    context.State.Entry(context);
                }

                return;
            }
            protected internal override void uiButtonPushed(PresentationContext context)
            {
                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEndTimedEvent("rfidMonitor");
                    ctxt.DoPause();
                }
                finally
                {
                    context.State = MainMap.RFIDMonitorWait;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDWriteEPCWait :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDWriteEPCWait(string name, int id) :
                base(name, id)
            { }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDWriteEPCWrite;
                context.State.Entry(context);
                return;
            }
        }

        internal class MainMap_RFIDWriteEPCWrite :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDWriteEPCWrite(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.ActionRfidWriteEpc();
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidWriteEPCNoTagsInField(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                    ctxt.DoFailWriteEpcZeroRead();
                    ctxt.ActionLogEvent("rfidWriteEPCFailedNoTagsInField");
                    ctxt.ActionRFIDResetSelect();
                }
                finally
                {
                    context.State = MainMap.RFIDWriteEPCWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidWriteEPCTooMuchTagsInField(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                    ctxt.DoFailWriteEpcMultipleRead();
                    ctxt.ActionLogEvent("rfidWriteEPCFailedTooMuchTagsInField");
                    ctxt.ActionRFIDResetSelect();
                }
                finally
                {
                    context.State = MainMap.RFIDWriteEPCWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidWriteEPCWriteSuccessful(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Success);
                    ctxt.DoSucceedWriteEPC();
                    ctxt.ActionLogEvent("rfidWriteEPCSucceeded");
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidWriteEPCWriteToTagFailed(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                    ctxt.DoFailWriteEpcWriteFailed();
                    ctxt.ActionLogEvent("rfidWriteEPCFailedWriteFailed");
                    ctxt.ActionRFIDResetSelect();
                }
                finally
                {
                    context.State = MainMap.RFIDWriteEPCWait;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_RFIDReadDetailedTagInformationWait :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDReadDetailedTagInformationWait(string name, int id) :
                base(name, id)
            { }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                context.State.Exit(context);
                context.State = MainMap.RFIDReadDetailedTagInformationStart;
                context.State.Entry(context);
                return;
            }
        }

        internal class MainMap_RFIDReadDetailedTagInformationStart :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_RFIDReadDetailedTagInformationStart(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.ActionRfidReadDetailedTagInformation(10);
                ctxt.ActionLogEvent("rfidReadDetailedTagInformation");
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void rfidActionFail(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                }
                finally
                {
                    context.State = MainMap.RFIDReadDetailedTagInformationWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidActionFailMoreTags(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Failure);
                }
                finally
                {
                    context.State = MainMap.RFIDReadDetailedTagInformationWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidActionSuccess(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Success);
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void rfidActionSuccessMoreTags(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionUITune(UITune.Success);
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_BarcodeReadWait :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_BarcodeReadWait(string name, int id) :
                base(name, id)
            { }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void uiButtonPushed(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEvent("uiButtonPushed");
                }
                finally
                {
                    context.State = MainMap.BarcodeReadStart;
                    context.State.Entry(context);
                }

                return;
            }
        }

        internal class MainMap_BarcodeReadStart :
            MainMap_Default
        {
            //-------------------------------------------------------
            // Member methods.
            //

            internal MainMap_BarcodeReadStart(string name, int id) :
                base(name, id)
            { }

            protected internal override void Entry(PresentationContext context)
            {
                Presentation ctxt = context.Owner;

                ctxt.ActionUILedSetBlinking(true);
                ctxt.ActionBarcodeRead();
                ctxt.DoContinue();
                return;
            }

            protected internal override void Default(PresentationContext context)
            {

                return;
            }

            protected internal override void barcodeReadFail(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEvent("barcodeReadFail");
                    ctxt.DoPause();
                    ctxt.ActionUITune(UITune.Failure);
                    ctxt.ActionUILedSetBlinking(false);
                }
                finally
                {
                    context.State = MainMap.BarcodeReadWait;
                    context.State.Entry(context);
                }

                return;
            }

            protected internal override void barcodeReadSuccess(PresentationContext context)
            {

                Presentation ctxt = context.Owner;


                context.State.Exit(context);
                context.ClearState();

                try
                {
                    ctxt.ActionLogEvent("barcodeReadSuccess");
                    ctxt.DoPause();
                    ctxt.ActionUITune(UITune.Success);
                }
                finally
                {
                    context.State = MainMap.Connected;
                    context.State.Entry(context);
                }

                return;
            }
        }
    }
}

