﻿using System;

using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace Nedap.HandheldApi
{
    /// <summary>
    /// Represents custom timer class.
    /// Synchronisation when dispose is executed is implemented.
    /// </summary>
    internal class SyncTimer : IDisposable
    {
        #region Fields

        private Object lockObject;
        private bool disposed;
        private Timer timer;

        #endregion

        #region Constructor/destructor

        /// <summary>
        /// Constructor, when called timer object and lockObject objects are created.
        /// </summary>
        /// <param name="callback"></param>
        /// <param name="state"></param>
        /// <param name="dueTime"></param>
        /// <param name="period"></param>
        public SyncTimer(TimerCallback callback, object state, long dueTime, long period)
        {
            lockObject = new Object();
            timer = new Timer(callback, state, dueTime, period);
        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~SyncTimer()
        {
            Dispose();
        }

        #endregion

        /// <summary>
        /// Use to change when timer event will be trigerred. 
        /// Changes dueTime and period properties of the timer.
        /// </summary>
        /// <param name="dueTime"></param>
        /// <param name="period"></param>
        public void Change(long dueTime, long period)
        {
            timer.Change(dueTime, period);
        }


        #region IDisposable Members

        public void Dispose()
        {
            lock (lockObject)
            {
                if (!disposed)    
                {
                    disposed = true;
                    timer.Dispose();
                }
            }
        }

        #endregion
    }
}
