﻿using System;
using System.Text;
using Nedap.EPC.TDT;

namespace Nedap.HandheldApi
{
    /// <summary>
    /// Select defines EPC specific parameters
    /// </summary>
    public class Select : Object
    {
        /// <summary>
        /// Memory bank used in select
        /// </summary>
        public RfidEpcMemoryBank MemoryBank { get; set; }
        /// <summary>
        /// Address for start of select in bits
        /// </summary>
        public uint AddressInBits { get; set; }
        /// <summary>
        /// Length of select in bits
        /// </summary>
        public uint DataLengthInBits { get; set; }
        /// <summary>
        /// Actual select data
        /// </summary>
        public byte[] Data { get; set; }
        /// <summary>
        /// Select is inverted or not
        /// </summary>
        public bool Invert { get; set; }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="epc"></param>
        public Select(EpcCode epc)
        {
            MemoryBank = RfidEpcMemoryBank.Epc;
            AddressInBits = 32;
            DataLengthInBits = epc.LengthInBits;
            Data = epc.BinaryData;
            Invert = false;
        }

        /// <summary>
        /// ToString
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return String.Format(
                "Select on memoryBank: {0}, addressInBits: {1}, dataLengthInBits: {2}, data: {3}",
                MemoryBank.ToString(),
                AddressInBits,
                DataLengthInBits,
                Encoding.UTF8.GetString(Data, 0, (int)DataLengthInBits));
        }
    }
}
