﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Nedap.HandheldApi;
using Nedap.EPC.TDT;

namespace NedapTestCases.HighLevelApiTests
{
    internal class MockSession : ISession
    {
        #region Public properties

        public bool BarcodeHasReaderHappened { get; set; }
        public bool BarcodeTryDecodeHappened { get; set; }
        public bool UILedSetStatusLedBlinkingHappened { get; set; }
        public bool UILedSetStatusLedBlinkingValue { get; set; }
        public bool UILedSetStatusLedBlinkingValueChanged { get; set; }
        public bool RfidSetRegionHappened { get; set; }
        public RfidRegion RfidSetRegionValue { get; set; }
        public bool NfcTryToReadCardHappened { get; set; }
        public bool UIVibrateHappened { get; set; }
        public bool HandheldGetBatteryPercentageHappened { get; set; }
        public bool RfidSetMValueHappened { get; set; }
        public RfidEpcMValue RfidSetMValueValue { get; set; }
        public bool RfidSetSessionHappened { get; set; }
        public RfidEpcSession RfidSetSessionValue { get; set; }
        public bool RfidSetTargetHappened { get; set; }
        public RfidEpcTarget RfidSetTargetValue { get; set; }
        public bool RfidSetTargetToggle { get; set; }
        public bool RfidSetQValueHappened { get; set; }
        public int RfidSetQValueValue { get; set; }
        public bool RfidSetQValueDynamic { get; set; }
        public bool RfidReadEpcsWithTimeoutHappened { get; set; }
        public int RfidReadEpcsWithTimeoutTimeout { get; set; }
        public int RfidReadEpcsWithTimeoutPower { get; set; }
        public Select RfidReadEpcsWithTimeoutSelect { get; set; }
        public bool RfidWriteEpcHappened { get; set; }
        public EpcCode RfidWriteEpcCode { get; set; }
        public int RfidWriteEpcTimeout { get; set; }
        public int RfidWriteEpcPower { get; set; }
        public Select RfidWriteEpcSelect { get; set; }


        public Dictionary<BarcodeType, bool> BarcodeSetStatusForTypeValues { get; set; }
        public bool UIPlayTuneHappened { get; set; }
        public UITune UIPlayTuneValue { get; set; }
        public bool UIVibrateTuneHappened { get; set; }
        public UITune UIVibrateTuneValue { get; set; }
        public bool HandheldIsConnectedHappened { get; set; }
        public bool HandheldDisconnectHappened { get; set; }
        public bool LogHappened { get; set; }
        public string LogMessage { get; set; }
        public bool RfidReadDataHappened { get; set; }
        public RfidEpcMemoryBank RfidReadDataMemoryBank { get; set; }
        public int RfidReadDataAddressInWords { get; set; }
        public int RfidReadDataLengthInWods { get; set; }
        public int RfidReadDataTimeout { get; set; }
        public int RfidReadDataPower { get; set; }
        public Select RfidReadDataSelect { get; set; }
        public Password RfidReadDataPassword { get; set; }
        public bool RfidFinishReadCycleHappened { get; set; }

        #endregion

        public MockSession()
        {
            BarcodeHasReaderHappened = false;
            UILedSetStatusLedBlinkingHappened = false;
            RfidSetRegionHappened = false;
            NfcTryToReadCardHappened = false;
            UIVibrateHappened = false;
            HandheldGetBatteryPercentageHappened = false;
            RfidSetSessionHappened = false;
            RfidSetTargetHappened = false;
            RfidSetQValueHappened = false;
            RfidReadEpcsWithTimeoutHappened = false;
            BarcodeSetStatusForTypeValues = new Dictionary<BarcodeType, bool>();
            UIPlayTuneHappened = false;
            UIVibrateTuneHappened = false;
            HandheldIsConnectedHappened = false;
            HandheldDisconnectHappened = false;
        }

        #region Events

        public event EventHandler<EventArgs> OnSessionBarcodeNotRead;

        public event EventHandler<RfidReadABarcodeEventArgs> OnSessionBarcodeRead;

        public event EventHandler<EventArgs> OnSessionHandheldConnect;

        public event EventHandler<EventArgs> OnSessionHandheldDisconnect;

        public event EventHandler<HandheldReceiveBatteryPercentageEventArgs> OnSessionHandheldReceiveBatteryPercentage;

        public event EventHandler<EventArgs> OnSessionHandheldReceiveChargingStatus;

        public event EventHandler<EventArgs> OnSessionHandheldReceiveEmptyBatteryStatus;

        public event EventHandler<HandheldReceiveErrorEventArgs> OnSessionHandheldReceiveError;

        public event EventHandler<EventArgs> OnSessionHandheldReceiveLowBatteryWarning;

        public event EventHandler<EventArgs> OnSessionNfcCardNotFound;

        public event EventHandler<RfidReadCardEventArgs> OnSessionNfcCardRead;

        public event EventHandler<RfidFailButTagsAvailableEventArgs> OnSessionRfidFailButTagsAvailable;

        public event EventHandler<EventArgs> OnSessionRfidFinishReadCycle;

        public event EventHandler<RfidReadDataEventArgs> OnSessionRfidReadData;

        public event EventHandler<RfidReadTagEventArgs> OnSessionRfidReadTag;

        public event EventHandler<RfidReceiveSupportedRegionsEventArgs> OnSessionRfidReceiveSupportedRegions;

        public event EventHandler<RfidWriteDataMoreTagsAvailableEventArgs> OnSessionRfidWriteDataMoreTagsAvailable;

        public event EventHandler<EventArgs> OnSessionUIPushButton;

        public event EventHandler<EventArgs> OnSessionUIShake;

        #endregion

        #region ISession Members

        public bool BarcodeHasReader()
        {
            BarcodeHasReaderHappened = true;
            return true;
        }

        public void BarcodeSetStatusForType(BarcodeType barcodeType, bool enabled)
        {
            if (BarcodeSetStatusForTypeValues.ContainsKey(barcodeType))
            {
                BarcodeSetStatusForTypeValues[barcodeType] = enabled;
            }
            else
            {
                BarcodeSetStatusForTypeValues.Add(barcodeType, enabled);
            }
        }

        public void BarcodeTryDecode(int duration)
        {
            BarcodeTryDecodeHappened = true;
        }

        public void Dispose()
        {
        }

        public void HandheldDisconnect()
        {
            HandheldDisconnectHappened = true;
        }


        public DeviceInfo HandheldGetInformation()
        {
            return new DeviceInfo("0", "0", "0.0.0", "0.0.0");
        }

        public void HandheldGetBatteryPercentage()
        {
            HandheldGetBatteryPercentageHappened = true;
        }

        public bool HandheldIsConnected()
        {
            HandheldIsConnectedHappened = true;
            return true;
        }

        public void Log(string logLine)
        {
            LogHappened = true;
            LogMessage = logLine;
        }

        public void NfcTryToReadCard()
        {
            NfcTryToReadCardHappened = true;
        }

        public void RfidGetSupportedRegions()
        {
            throw new NotImplementedException();
        }

        public void RfidFinishReadCycle()
        {
            RfidFinishReadCycleHappened = true;
        }

        public void RfidReadData(RfidEpcMemoryBank memoryBank, int addressInWords, int lengthInWords, int timeout, int power, Select select, Password password)
        {
            RfidReadDataHappened = true;
            RfidReadDataMemoryBank = memoryBank;
            RfidReadDataAddressInWords = addressInWords;
            RfidReadDataLengthInWods = lengthInWords;
            RfidReadDataTimeout = timeout;
            RfidReadDataPower = power;
            RfidReadDataSelect = select;
            RfidReadDataPassword = password;
        }

        public void RfidReadEpcs(int timeout, int power, Select select)
        {
            RfidReadEpcsWithTimeoutHappened = true;
            RfidReadEpcsWithTimeoutTimeout = timeout;
            RfidReadEpcsWithTimeoutPower = power;
            RfidReadEpcsWithTimeoutSelect = select;
        }

        public void RfidSetCWSignal(bool enabled, int antenna)
        {
            throw new NotImplementedException();
        }

        public void RfidSetFrequencyHopTable(List<RfidFrequency> frequencies)
        {
            throw new NotImplementedException();
        }

        public void RfidSetMValue(RfidEpcMValue mValue)
        {
            RfidSetMValueHappened = true;
            RfidSetMValueValue = mValue;
        }

        public void RfidSetQValue(int qValue, bool dynamic)
        {
            RfidSetQValueHappened = true;
            RfidSetQValueValue = qValue;
            RfidSetQValueDynamic = dynamic;
        }

        public void RfidSetRegion(RfidRegion region)
        {
            RfidSetRegionHappened = true;
            RfidSetRegionValue = region;
        }

        public void RfidSetSession(RfidEpcSession session)
        {
            RfidSetSessionHappened = true;
            RfidSetSessionValue = session;
        }

        public void RfidSetTarget(RfidEpcTarget target, bool toggle)
        {
            RfidSetTargetHappened = true;
            RfidSetTargetValue = target;
            RfidSetTargetToggle = toggle;
        }

        public void RfidWriteData(byte[] dataToWrite, RfidEpcMemoryBank memoryBank, int addressInWords, int timeout, int power, Select select, Password password)
        {
            throw new NotImplementedException();
        }

        public void RfidWriteEpc(EpcCode epc)
        {
            RfidWriteEpcHappened = true;
            RfidWriteEpcCode = epc;
        }

        public void UILedSetBlinking(bool blinking)
        {
            UILedSetStatusLedBlinkingHappened = true;
            if (UILedSetStatusLedBlinkingValue != blinking)
            {
                UILedSetStatusLedBlinkingValueChanged = true;
            }
            UILedSetStatusLedBlinkingValue = blinking;
        }

        public void UIPlay(UITune tune)
        {
            UIPlayTuneHappened = true;
            UIPlayTuneValue = tune;
        }

        public void UIVibrate()
        {
            UIVibrateHappened = true;
        }

        public void UIVibrate(int timeout)
        {
            throw new NotImplementedException();
        }

        public void UIVibrateTune(UITune tune)
        {
            UIVibrateTuneHappened = true;
            UIVibrateTuneValue = tune;
        }

        #endregion

        #region Event triggers

        public void DoOnSessionBarcodeNotReadABarcode()
        {
            if (OnSessionBarcodeNotRead != null)
            {
                OnSessionBarcodeNotRead(this, new EventArgs());
            }
        }

        public void DoOnSessionBarcodeReadABarcode(string barcode, BarcodeType barcodeType)
        {
            if (OnSessionBarcodeRead != null)
            {
                OnSessionBarcodeRead(this, new RfidReadABarcodeEventArgs(barcode, barcodeType));
            }
        }

        public void DoOnSessionHandheldConnect()
        {
            if (OnSessionHandheldConnect != null)
            {
                OnSessionHandheldConnect(this, new EventArgs());
            }
        }

        public void DoOnSessionHandheldDisconnect()
        {
            if (OnSessionHandheldDisconnect != null)
            {
                OnSessionHandheldDisconnect(this, new EventArgs());
            }
        }

        public void DoOnSessionHandheldReceiveBatteryPercentage(int percentage)
        {
            if (OnSessionHandheldReceiveBatteryPercentage != null)
            {
                OnSessionHandheldReceiveBatteryPercentage(this, new HandheldReceiveBatteryPercentageEventArgs(percentage));
            }
        }

        public void DoOnSessionHandheldReceiveChargingStatus()
        {
            if (OnSessionHandheldReceiveChargingStatus != null)
            {
                OnSessionHandheldReceiveChargingStatus(this, new EventArgs());
            }
        }

        public void DoOnSessionHandheldReceiveLowBatteryWarning()
        {
            if (OnSessionHandheldReceiveLowBatteryWarning != null)
            {
                OnSessionHandheldReceiveLowBatteryWarning(this, new EventArgs());
            }
        }

        public void DoOnSessionHandheldReceiveEmptyBatteryStatus()
        {
            if (OnSessionHandheldReceiveEmptyBatteryStatus != null)
            {
                OnSessionHandheldReceiveEmptyBatteryStatus(this, new EventArgs());
            }
        }

        public void DoOnSessionHandheldReceiveError(string errorMessage)
        {
            if (OnSessionHandheldReceiveError != null)
            {
                OnSessionHandheldReceiveError(this, new HandheldReceiveErrorEventArgs(errorMessage));
            }
        }

        public void DoOnSessionNfcNotFoundACard()
        {
            if (OnSessionNfcCardNotFound != null)
            {
                OnSessionNfcCardNotFound(this, new EventArgs());
            }
        }

        public void DoOnSessionNfcReadCard(int cardNumber)
        {
            if (OnSessionNfcCardRead != null)
            {
                OnSessionNfcCardRead(this, new RfidReadCardEventArgs(cardNumber));
            }
        }

        public void DoOnSessionRfidFailButTagsAvailable(bool available)
        {
            if (OnSessionRfidFailButTagsAvailable != null)
            {
                OnSessionRfidFailButTagsAvailable(this, new RfidFailButTagsAvailableEventArgs(available));
            }
        }

        public void DoOnSessionRfidFinishReadCycle()
        {
            if (OnSessionRfidFinishReadCycle != null)
            {
                OnSessionRfidFinishReadCycle(this, new EventArgs());
            }
        }

        public void DoOnSessionRfidReadData(byte[] data, bool moreTagsAvailable)
        {
            if (OnSessionRfidReadData != null)
            {
                OnSessionRfidReadData(this, new RfidReadDataEventArgs(data, moreTagsAvailable));
            }
        }

        public void DoOnSessionRfidReadTag(EpcObservation epcObservation)
        {
            if (OnSessionRfidReadTag != null)
            {
                OnSessionRfidReadTag(this, new RfidReadTagEventArgs(epcObservation));
            }
        }

        public void DoOnSessionRfidReceiveSupportedRegions(int regionFlags)
        {
            if (OnSessionRfidReceiveSupportedRegions != null)
            {
                OnSessionRfidReceiveSupportedRegions(this, new RfidReceiveSupportedRegionsEventArgs(regionFlags));
            }
        }

        public void DoOnSessionRfidWriteDataMoreTagsAvailable(bool available)
        {
            if (OnSessionRfidWriteDataMoreTagsAvailable != null)
            {
                OnSessionRfidWriteDataMoreTagsAvailable(this, new RfidWriteDataMoreTagsAvailableEventArgs(available));
            }
        }

        public void DoOnSessionUIPushButton()
        {
            if (OnSessionUIPushButton != null)
            {
                OnSessionUIPushButton(this, new EventArgs());
            }
        }

        public void DoOnSessionUIShake()
        {
            if (OnSessionUIShake != null)
            {
                OnSessionUIShake(this, new EventArgs());
            }
        }

        public event EventHandler<EventArgs> OnSessionRfidWriteNoTagsInField;

        public event EventHandler<EventArgs> OnSessionRfidWriteTooMuchTagsInField;

        public event EventHandler<EventArgs> OnSessionRfidWriteToTagFailed;

        public event EventHandler<EventArgs> OnSessionRfidWriteSuccess;

        #endregion
    }
}
