﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Nedap.HandheldApi;
using System.Threading;
using Nedap.EPC.TDT;
using System.Collections.ObjectModel;

namespace NedapTestCases.HighLevelApiTests
{
    /// <summary>
    /// Summary description for PresentationTestCases
    /// </summary>
    [TestClass]
    public class PresentationTestCases
    {
        #region Constants
        private const int CARD_NUMBER = 1;

        private const RfidEpcMValue INVENTORY_START_M_VALUE = RfidEpcMValue.Value4;
        private const RfidEpcSession INVENTORY_START_SESSION = RfidEpcSession.Session1;
        private const RfidEpcSession INVENTORY_HIGH_VOLUME_SESSION2 = RfidEpcSession.Session2;
        private const RfidEpcTarget INVENTORY_START_TARGET = RfidEpcTarget.TargetA;
        private const bool INVENTORY_START_TARGET_TOGGLE = false;
        private const int INVENTORY_START_Q_VALUE = 0;
        private const bool INVENTORY_START_Q_VALUE_DYNAMIC = true;
        private const int INVENTORY_READ_TIMEOUT = 400;
        private const int INVENTORY_READ_POWER = 23;

        private const int MONITOR_START_MAX_EPC_LENGTH = 96;
        private const RfidEpcMValue MONITOR_START_M_VALUE = RfidEpcMValue.Value4;
        private const RfidEpcSession MONITOR_START_SESSION = RfidEpcSession.Session0;
        private const RfidEpcTarget MONITOR_START_TARGET = RfidEpcTarget.TargetA;
        private const bool MONITOR_START_TARGET_TOGGLE = true;
        private const int MONITOR_READ_TIMEOUT = 300;

        private const int READ_SINGLE_MAX_EPC_LENGTH = 96;
        private const RfidEpcMValue READ_SINGLE_M_VALUE = RfidEpcMValue.Value4;
        private const RfidEpcSession READ_SINGLE_SESSION = RfidEpcSession.Session0;
        private const RfidEpcTarget READ_SINGLE_TARGET = RfidEpcTarget.TargetA;
        private const bool READ_SINGLE_TARGET_TOGGLE = false;
        private const int READ_SINGLE_READ_TIMEOUT = 200;

        private const int READ_DETAILED_TAG_INFORMATION_MAX_EPC_LENGTH = 96;
        private const RfidEpcMValue READ_DETAILED_TAG_INFORMATION_M_VALUE = RfidEpcMValue.Value4;
        private const RfidEpcSession READ_DETAILED_TAG_INFORMATION_SESSION = RfidEpcSession.Session0;
        private const RfidEpcTarget READ_DETAILED_TAG_INFORMATION_TARGET = RfidEpcTarget.TargetA;
        private const bool READ_DETAILED_TAG_INFORMATION_TARGET_TOGGLE = true;
        private const int READ_DETAILED_TAG_INFORMATION_READ_TIMEOUT = 100;

        private const int WRITE_EPC_MAX_EPC_LENGTH = 96;
        private const RfidEpcMValue WRITE_EPC_M_VALUE = RfidEpcMValue.Value8;
        private const RfidEpcSession WRITE_EPC_SESSION = RfidEpcSession.Session0;
        private const RfidEpcTarget WRITE_EPC_TARGET = RfidEpcTarget.TargetA;
        private const bool WRITE_EPC_TARGET_TOGGLE = true;
        private const int WRITE_EPC_READ_TIMEOUT = 50;

        private const String BUFFER_NAME = "buffer";
        private const BarcodeType BARCODE_READ_TYPE_VALUE = BarcodeType.Code39;
        private const int BARCODE_READ_TYPE_DURATION = 100;
        private const UITune UI_PLAY_TUNE_TUNE = UITune.Success;

        private const String BARCODE = "1234567890";
        private const BarcodeType BARCODETYPE_UNKNOWN = BarcodeType.Unknown;

        private const int BATTERY_PERCENTAGE = 80;

        private const bool MORE_TAGS_AVAILABLE = true;
        private const bool MORE_TAGS_UNAVAILABLE = false;
        private const int REGION_FLAGS = 100;

        private const string ERROR_MESSAGE = "Error Message";

        private const int ADDRESS_IN_WORDS = 1;
        private const int LENGTH_IN_WORDS = 2;
        private const int TIMEOUT = 3;
        private const int POWER = 4;
        #endregion

        #region Fields

        private MockSession session;
        private PresentationFacade facade;
        private EpcCode epcCode;
        private Select select;
        private byte[] rfidData;
        private EpcObservation epcObs1;
        private EpcObservation epcObs2;
        private Password password;
        DeviceInfo info;

        private bool OnClearInventoryHappened;
        private bool OnReceiveBatteryPercentageHappened;
        private int OnReceiveBatteryPercentageValue;
        private bool OnLowBatteryWarningHappened;
        private bool OnEmptyBatteryStatusHappened;
        private bool OnChargingStatusHappened;
        private bool OnDidReadRfidTagHappened;
        private EpcObservation OnDidReadRfidTagValue;
        private bool OnDidReadRfidTagsHappened;
        private Collection<EpcObservation> OnDidReadRFIDTagsEpcObservations;
        private bool OnDidReadDetailedTagInformationHappened;
        private RfidEpcMdid OnDidReadDetailedTagInformationTagMdid;
        private int OnDidReadDetailedTagInformationTagModelNumber;
        private bool OnDidReadDetailedTagInformationTagXtid;
        private bool OnDidReadBarcodeHappened;
        private string OnDidReadBarcodeValue;
        private BarcodeType OnDidReadBarcodeType;
        private bool OnDidNotReadBarcodeHappened;
        private bool OnDidPushButtonHappened;
        private bool OnDidShakeHappened;
        private bool OnDidReadNfcCardHappened;
        private int OnDidReadNfcCardValue;
        //private bool OnIsDisconnectedHappened;
        //private bool OnDidFinishSingleReadHappened;
        private bool OnDidContinueHappened;
        private bool OnDidPauseHappened;
        private bool OnDidFailWriteEpcMultipleReadHappened;
        private bool OnDidFailWriteEpcZeroReadHappened;
        private bool OnDidFailWriteEpcWriteFailedHappened;
        private bool OnDidFailWriteEpcVerificationFailedHappened;
        //private bool OnDidSucceedWriteEpcHappened;
        private bool OnDidConnectHappened;
        private bool OnDidDisconnectHappened;
        private bool OnDidReceiveSupportedRegionsHappened;
        private int OnDidReceiveSupportedRegionsValue;

        #endregion

        public PresentationTestCases()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }

        [TestInitialize()]
        public void MyTestInitialize()
        {
            session = new MockSession();
            facade = new PresentationFacade(session);
            byte[] epcCodeData = new byte[2];
            epcCodeData[0] = 1;
            epcCodeData[1] = 6;
            epcCode = new EpcCode(epcCodeData);
            select = new Select(epcCode);

            rfidData = new byte[] { 0x30, 0x74, 0x04, 0x81, 0x10, 0xF5, 0x1E, 0x80, 0x00, 0x00, 0x00, 0x01 };
            epcObs1 = new EpcObservation(new EpcCode(new byte[] { 0x30, 0x74, 0x04, 0x81, 0x10, 0xF5, 0x1E, 0x80, 0x00, 0x00, 0x00, 0x01 }), DateTime.Now, 10, 1);
            epcObs2 = new EpcObservation(new EpcCode(new byte[] { 0x30, 0x74, 0x04, 0x81, 0x10, 0xF5, 0x1E, 0x80, 0x00, 0x00, 0x00, 0x02 }), DateTime.Now, 10, 1);
            password = new Password();
            info = new DeviceInfo("0", "0", "0.0.0", "0.0.0");

            OnClearInventoryHappened = false;
            OnReceiveBatteryPercentageHappened = false;
            OnLowBatteryWarningHappened = false;
            OnEmptyBatteryStatusHappened = false;
            OnChargingStatusHappened = false;
            OnDidReadRfidTagHappened = false;
            OnDidReadRfidTagsHappened = false;
            OnDidReadDetailedTagInformationHappened = false;
            OnDidReadBarcodeHappened = false;
            OnDidNotReadBarcodeHappened = false;
            OnDidPushButtonHappened = false;
            OnDidShakeHappened = false;
            OnDidReadNfcCardHappened = false;
            //OnIsDisconnectedHappened = false;
            //OnDidFinishSingleReadHappened = false;
            //OnDidContinueHappened = false;
            OnDidPauseHappened = false;
            OnDidFailWriteEpcMultipleReadHappened = false;
            OnDidFailWriteEpcZeroReadHappened = false;
            OnDidFailWriteEpcWriteFailedHappened = false;
            OnDidFailWriteEpcVerificationFailedHappened = false;
            //OnDidSucceedWriteEpcHappened = false;
            OnDidConnectHappened = false;
            OnDidDisconnectHappened = false;
            OnDidReceiveSupportedRegionsHappened = false;


            facade.OnClearInventory += new EventHandler<EventArgs>(test_OnClearInventory);
            facade.OnReceiveBatteryPercentage += new EventHandler<ReceiveBatteryPercentageEventArgs>(test_OnReceiveBatteryPercentage);
            facade.OnLowBatteryWarning += new EventHandler<EventArgs>(test_OnLowBatteryWarning);
            facade.OnEmptyBatteryStatus += new EventHandler<EventArgs>(test_OnEmptyBatteryStatus);
            facade.OnChargingStatus += new EventHandler<EventArgs>(test_OnChargingStatus);
            facade.OnReadRfidTag += new EventHandler<ReadRfidTagEventArgs>(test_OnDidReadRFIDTag);
            facade.OnReadRfidTags += new EventHandler<ReadRfidTagsEventArgs>(test_OnDidReadRFIDTags);
            facade.OnReadDetailedTagInformation += new EventHandler<ReadDetailedTagInformationEventArgs>(test_OnDidReadDetailedTagInformation);
            facade.OnReadBarcode += new EventHandler<ReadBarcodeEventArgs>(test_OnDidReadBarcode);
            facade.OnNotReadBarcode += new EventHandler<EventArgs>(test_OnDidNotReadBarcode);
            facade.OnPushButton += new EventHandler<EventArgs>(test_OnDidPushButton);
            facade.OnShake += new EventHandler<EventArgs>(test_OnDidShake);
            facade.OnReadNfcCard += new EventHandler<ReadNfcCardEventArgs>(test_OnDidReadNFCCard);
            facade.OnIsDisconnected += new EventHandler<EventArgs>(test_OnIsDisconnected);
            facade.OnFinishSingleRead += new EventHandler<EventArgs>(test_OnDidFinishSingleRead);
            facade.OnContinue += new EventHandler<EventArgs>(test_OnDidContinue);
            facade.OnPause += new EventHandler<EventArgs>(test_OnDidPause);
            facade.OnFailWriteEpcMultipleRead += new EventHandler<EventArgs>(test_OnDidFailWriteEpcMultipleRead);
            facade.OnFailWriteEpcZeroRead += new EventHandler<EventArgs>(test_OnDidFailWriteEpcZeroRead);
            facade.OnFailWriteEpcWriteFailed += new EventHandler<EventArgs>(test_OnDidFailWriteEpcWriteFailed);
            facade.OnFailWriteEpcVerificationFailed += new EventHandler<EventArgs>(test_OnDidFailWriteEpcVerificationFailed);
            facade.OnSucceedWriteEpc += new EventHandler<EventArgs>(test_OnDidSucceedWriteEPC);
            facade.OnConnect += new EventHandler<EventArgs>(test_OnDidConnect);
            facade.OnDisconnect += new EventHandler<EventArgs>(test_OnDidDisconnect);
            facade.OnReceiveSupportedRegions += new EventHandler<ReceiveSupportedRegionsEventArgs>(test_OnDidReceiveSupportedRegions);
        }

        [TestCleanup()]
        public void MyTestCleanup()
        {
            facade.OnClearInventory -= new EventHandler<EventArgs>(test_OnClearInventory);
            facade.OnReceiveBatteryPercentage -= new EventHandler<ReceiveBatteryPercentageEventArgs>(test_OnReceiveBatteryPercentage);
            facade.OnLowBatteryWarning -= new EventHandler<EventArgs>(test_OnLowBatteryWarning);
            facade.OnEmptyBatteryStatus -= new EventHandler<EventArgs>(test_OnEmptyBatteryStatus);
            facade.OnChargingStatus -= new EventHandler<EventArgs>(test_OnChargingStatus);
            facade.OnReadRfidTag -= new EventHandler<ReadRfidTagEventArgs>(test_OnDidReadRFIDTag);
            facade.OnReadRfidTags -= new EventHandler<ReadRfidTagsEventArgs>(test_OnDidReadRFIDTags);
            facade.OnReadDetailedTagInformation -= new EventHandler<ReadDetailedTagInformationEventArgs>(test_OnDidReadDetailedTagInformation);
            facade.OnReadBarcode -= new EventHandler<ReadBarcodeEventArgs>(test_OnDidReadBarcode);
            facade.OnNotReadBarcode -= new EventHandler<EventArgs>(test_OnDidNotReadBarcode);
            facade.OnPushButton -= new EventHandler<EventArgs>(test_OnDidPushButton);
            facade.OnShake -= new EventHandler<EventArgs>(test_OnDidShake);
            facade.OnReadNfcCard -= new EventHandler<ReadNfcCardEventArgs>(test_OnDidReadNFCCard);
            facade.OnIsDisconnected -= new EventHandler<EventArgs>(test_OnIsDisconnected);
            facade.OnFinishSingleRead -= new EventHandler<EventArgs>(test_OnDidFinishSingleRead);
            facade.OnContinue -= new EventHandler<EventArgs>(test_OnDidContinue);
            facade.OnPause -= new EventHandler<EventArgs>(test_OnDidPause);
            facade.OnFailWriteEpcMultipleRead -= new EventHandler<EventArgs>(test_OnDidFailWriteEpcMultipleRead);
            facade.OnFailWriteEpcZeroRead -= new EventHandler<EventArgs>(test_OnDidFailWriteEpcZeroRead);
            facade.OnFailWriteEpcWriteFailed -= new EventHandler<EventArgs>(test_OnDidFailWriteEpcWriteFailed);
            facade.OnFailWriteEpcVerificationFailed -= new EventHandler<EventArgs>(test_OnDidFailWriteEpcVerificationFailed);
            facade.OnSucceedWriteEpc -= new EventHandler<EventArgs>(test_OnDidSucceedWriteEPC);
            facade.OnConnect -= new EventHandler<EventArgs>(test_OnDidConnect);
            facade.OnDisconnect -= new EventHandler<EventArgs>(test_OnDidDisconnect);
            facade.OnReceiveSupportedRegions -= new EventHandler<ReceiveSupportedRegionsEventArgs>(test_OnDidReceiveSupportedRegions);

            facade.Dispose();
            session.Dispose();
        }

        #endregion

        #region Event handlers

        private void test_OnClearInventory(object sender, EventArgs e)
        {
            OnClearInventoryHappened = true;
        }

        private void test_OnReceiveBatteryPercentage(object sender, ReceiveBatteryPercentageEventArgs e)
        {
            OnReceiveBatteryPercentageHappened = true;
            OnReceiveBatteryPercentageValue = e.Percentage;
        }

        private void test_OnLowBatteryWarning(object sender, EventArgs e)
        {
            OnLowBatteryWarningHappened = true;
        }

        private void test_OnEmptyBatteryStatus(object sender, EventArgs e)
        {
            OnEmptyBatteryStatusHappened = true;
        }

        private void test_OnChargingStatus(object sender, EventArgs e)
        {
            OnChargingStatusHappened = true;
        }

        private void test_OnDidReadRFIDTag(object sender, ReadRfidTagEventArgs e)
        {
            OnDidReadRfidTagHappened = true;
            OnDidReadRfidTagValue = e.EpcObservation;
        }

        private void test_OnDidReadRFIDTags(object sender, ReadRfidTagsEventArgs e)
        {
            OnDidReadRfidTagsHappened = true;
            OnDidReadRFIDTagsEpcObservations = e.EpcObservations;
        }

        private void test_OnDidReadDetailedTagInformation(object sender, ReadDetailedTagInformationEventArgs e)
        {
            OnDidReadDetailedTagInformationHappened = true;
            OnDidReadDetailedTagInformationTagMdid = e.TagMdid;
            OnDidReadDetailedTagInformationTagModelNumber = e.TagModelNumber;
            OnDidReadDetailedTagInformationTagXtid = e.TagXtid;
        }

        private void test_OnDidReadBarcode(object sender, ReadBarcodeEventArgs e)
        {
            OnDidReadBarcodeHappened = true;
            OnDidReadBarcodeValue = e.Barcode;
            OnDidReadBarcodeType = e.BarcodeType;
        }

        private void test_OnDidNotReadBarcode(object sender, EventArgs e)
        {
            OnDidNotReadBarcodeHappened = true;
        }

        private void test_OnDidPushButton(object sender, EventArgs e)
        {
            OnDidPushButtonHappened = true;
        }

        private void test_OnDidShake(object sender, EventArgs e)
        {
            OnDidShakeHappened = true;
        }

        private void test_OnDidReadNFCCard(object sender, ReadNfcCardEventArgs e)
        {
            OnDidReadNfcCardHappened = true;
            OnDidReadNfcCardValue = e.CardNumber;
        }

        private void test_OnIsDisconnected(object sender, EventArgs e)
        {
            throw new NotImplementedException();
        }

        private void test_OnDidFinishSingleRead(object sender, EventArgs e)
        {
            throw new NotImplementedException();
        }

        private void test_OnDidContinue(object sender, EventArgs e)
        {
            OnDidContinueHappened = true;
        }

        private void test_OnDidPause(object sender, EventArgs e)
        {
            OnDidPauseHappened = true;
        }

        private void test_OnDidFailWriteEpcMultipleRead(object sender, EventArgs e)
        {
            OnDidFailWriteEpcMultipleReadHappened = true;
        }

        private void test_OnDidFailWriteEpcZeroRead(object sender, EventArgs e)
        {
            OnDidFailWriteEpcZeroReadHappened = true;
        }

        private void test_OnDidFailWriteEpcWriteFailed(object sender, EventArgs e)
        {
            OnDidFailWriteEpcWriteFailedHappened = true;
        }

        private void test_OnDidFailWriteEpcVerificationFailed(object sender, EventArgs e)
        {
            OnDidFailWriteEpcVerificationFailedHappened = true;
        }

        private void test_OnDidSucceedWriteEPC(object sender, EventArgs e)
        {
            throw new NotImplementedException();
        }

        private void test_OnDidConnect(object sender, EventArgs e)
        {
            OnDidConnectHappened = true;
        }

        private void test_OnDidDisconnect(object sender, EventArgs e)
        {
            OnDidDisconnectHappened = true;
        }

        private void test_OnDidReceiveSupportedRegions(object sender, ReceiveSupportedRegionsEventArgs e)
        {
            OnDidReceiveSupportedRegionsHappened = true;
            OnDidReceiveSupportedRegionsValue = e.RegionFlags;
        }

        #endregion

        #region PresentationFacade

        #region NFC

        [TestMethod]
        public void TestNfcReadStartDisconnected()
        {
            //Do NfcReadStart - State = Disconnected
            facade.NfcReadStart();
            Assert.IsFalse(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.NfcTryToReadCardHappened);
        }

        [TestMethod]
        public void TestNfcReadStartConnected()
        {
            //Do NfcReadStart - State = Disconnected
            facade.NfcReadStart();
            Assert.IsFalse(session.UILedSetStatusLedBlinkingHappened);

            //Connect
            session.DoOnSessionHandheldConnect();

            //Do NfcReadStart - State = Connected
            facade.NfcReadStart();
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsTrue(session.UILedSetStatusLedBlinkingValue);
            Assert.IsTrue(session.NfcTryToReadCardHappened);
        }

        #endregion

        #region RFID

        #region RfidInventoryStart

        [TestMethod]
        public void TestRfidInventoryStartDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected
            facade.RfidInventoryStart();
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart();
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_START_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region RfidInventoryStartWithSelect

        [TestMethod]
        public void TestRfidInventoryStartWithSelectDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(select);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartWithSelectConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(select);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_START_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region rfidInventoryStartInBuffer

        [TestMethod]
        public void TestRfidInventoryStartInBufferDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(BUFFER_NAME);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartInBufferConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(BUFFER_NAME);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_START_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region rfidInventoryStartInBufferWithSelect

        [TestMethod]
        public void TestRfidInventoryStartInBufferWithSelectDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(BUFFER_NAME, select);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartInBufferWithSelectConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(BUFFER_NAME, select);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_START_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region RfidInventoryStartWithHighVolume

        [TestMethod]
        public void TestRfidInventoryStartWithHighVolumeDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(true);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartWithHighVolumeConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(true);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_HIGH_VOLUME_SESSION2, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region RfidInventoryStartWithSelectAndHighVolume

        [TestMethod]
        public void TestRfidInventoryStartWithSelectAndHighVolumeDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(select, true);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartWithSelectAndHighVolumeConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(select, true);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_HIGH_VOLUME_SESSION2, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region rfidInventoryStartInBufferWithHighVolume

        [TestMethod]
        public void TestRfidInventoryStartInBufferWithHighVolumeDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(BUFFER_NAME, true);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartInBufferWithHighVolumeConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(BUFFER_NAME, true);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_HIGH_VOLUME_SESSION2, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region rfidInventoryStartInBufferWithSelectAndHighVolume

        [TestMethod]
        public void TestRfidInventoryStartInBufferWithSelectAndHighVolumeDisconnected()
        {
            //Do RfidInventoryStart - State = Disconnected

            facade.RfidInventoryStart(BUFFER_NAME, select, true);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
            Assert.IsFalse(session.RfidSetQValueHappened);
        }

        [TestMethod]
        public void TestRfidInventoryStartInBufferWithSelectAndHighVolumeConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidInventoryStart
            facade.RfidInventoryStart(BUFFER_NAME, select, true);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(INVENTORY_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(INVENTORY_HIGH_VOLUME_SESSION2, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(INVENTORY_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(INVENTORY_START_TARGET_TOGGLE, session.RfidSetTargetToggle);
            Assert.IsTrue(session.RfidSetQValueHappened);
            Assert.AreEqual(INVENTORY_START_Q_VALUE, session.RfidSetQValueValue);
            Assert.AreEqual(INVENTORY_START_Q_VALUE_DYNAMIC, session.RfidSetQValueDynamic);

            // RFIDInventoryWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region RfidInventoryReset

        [TestMethod]
        public void TestRfidInventoryReset()
        {
            session.DoOnSessionHandheldConnect();
            facade.RfidInventoryStart();
            facade.RfidInventoryReset();
            Assert.IsTrue(OnClearInventoryHappened);
        }

        #endregion

        #region rfidInventoryResetBuffer

        [TestMethod]
        public void TestRfidInventoryResetBuffer()
        {
            session.DoOnSessionHandheldConnect();
            facade.RfidInventoryStart(BUFFER_NAME);
            facade.RfidInventoryReset(BUFFER_NAME);
            Assert.IsTrue(OnClearInventoryHappened);
        }

        #endregion

        #region RfidMonitorStart

        [TestMethod]
        public void TestRfidMonitorStartDisconnected()
        {
            //Do RfidMonitorStart - State = Disconnected
            facade.RfidMonitorStart();
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
        }

        [TestMethod]
        public void TestRfidMonitorStartConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidMonitorStart
            facade.RfidMonitorStart();
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(MONITOR_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(MONITOR_START_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(MONITOR_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(MONITOR_START_TARGET_TOGGLE, session.RfidSetTargetToggle);

            // RFIDMonitorWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region RfidMonitorStartWithSelect

        [TestMethod]
        public void TestRfidMonitorStartWithSelectDisconnected()
        {
            //Do RfidMonitorStart - State = Disconnected
            facade.RfidMonitorStart(select);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
        }

        [TestMethod]
        public void TestRfidMonitorStartWithSelectConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidMonitorStart
            facade.RfidMonitorStart(select);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(MONITOR_START_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(MONITOR_START_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(MONITOR_START_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(MONITOR_START_TARGET_TOGGLE, session.RfidSetTargetToggle);

            // RFIDMonitorWait is entered
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region RfidReadSingle

        [TestMethod]
        public void TestRfidReadSingleDisconnected()
        {
            //Do RfidReadSIngle - State = Disconnected
            facade.RfidReadSingle();
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
        }

        [TestMethod]
        public void TestRfidReadSingleConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidReadSIngle
            facade.RfidReadSingle();
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(READ_SINGLE_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(READ_SINGLE_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(READ_SINGLE_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(READ_SINGLE_TARGET_TOGGLE, session.RfidSetTargetToggle);
        }

        #endregion

        #region RfidReadSingleWithSelect

        [TestMethod]
        public void TestRfidReadSingleWithSelectDisconnected()
        {
            //Do RfidReadSIngle - State = Disconnected
            facade.RfidReadSingle(select);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
        }

        [TestMethod]
        public void TestRfidReadSingleWithSelectConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidReadSIngle
            facade.RfidReadSingle(select);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(READ_SINGLE_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(READ_SINGLE_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(READ_SINGLE_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(READ_SINGLE_TARGET_TOGGLE, session.RfidSetTargetToggle);
        }

        #endregion

        #region RfidReadDetailedTagInformation

        [TestMethod]
        public void TestRfidReadDetailedTagInformation()
        {
            //Do RfidReadSIngle
            facade.RfidReadTag();
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_TARGET_TOGGLE, session.RfidSetTargetToggle);
        }

        #endregion

        #region RfidReadDetailedTagInformationFromTag

        [TestMethod]
        public void TestRfidReadDetailedTagInformationFromTag()
        {
            //Do RfidReadSIngle
            facade.RfidReadTag(epcCode);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(READ_DETAILED_TAG_INFORMATION_TARGET_TOGGLE, session.RfidSetTargetToggle);
        }

        #endregion

        #region RfidWriteEpc

        [TestMethod]
        public void TestRfidWriteEpcDisconnected()
        {
            //Do RfidReadSIngle - State = Disconnected
            facade.RfidWriteEpc(epcCode);
            Assert.IsFalse(session.RfidSetMValueHappened);
            Assert.IsFalse(session.RfidSetSessionHappened);
            Assert.IsFalse(session.RfidSetTargetHappened);
        }

        [TestMethod]
        public void TestRfidWriteEpcConnected()
        {
            //Connect
            session.DoOnSessionHandheldConnect();

            //Do RfidWriteEpc
            facade.RfidWriteEpc(epcCode);
            Assert.IsTrue(session.RfidSetMValueHappened);
            Assert.AreEqual(WRITE_EPC_M_VALUE, session.RfidSetMValueValue);
            Assert.IsTrue(session.RfidSetSessionHappened);
            Assert.AreEqual(WRITE_EPC_SESSION, session.RfidSetSessionValue);
            Assert.IsTrue(session.RfidSetTargetHappened);
            Assert.AreEqual(WRITE_EPC_TARGET, session.RfidSetTargetValue);
            Assert.AreEqual(WRITE_EPC_TARGET_TOGGLE, session.RfidSetTargetToggle);
        }

        #endregion

        #endregion

        #region Barcode

        #region BarcodeHasReader

        [TestMethod]
        public void TestBarcodeHasReader()
        {
            facade.BarcodeHasReader();
            Assert.IsTrue(session.BarcodeHasReaderHappened);
        }

        #endregion

        #region BarcodeReadType

        [TestMethod]
        public void TestBarcodeReadType()
        {
            facade.BarcodeRead(BARCODE_READ_TYPE_VALUE, BARCODE_READ_TYPE_DURATION);
            foreach (BarcodeType type in session.BarcodeSetStatusForTypeValues.Keys)
            {
                Assert.AreEqual(type.Equals(BARCODE_READ_TYPE_VALUE), session.BarcodeSetStatusForTypeValues[type]);
            }
        }

        #endregion

        #region BarcodeReadTypes

        [TestMethod]
        public void TestBarcodeReadTypes()
        {
            Collection<BarcodeType> barcodeTypes = new Collection<BarcodeType>();
            barcodeTypes.Add(BarcodeType.Codabar);
            barcodeTypes.Add(BarcodeType.Code11);
            barcodeTypes.Add(BarcodeType.Code128);

            facade.BarcodeRead(barcodeTypes, BARCODE_READ_TYPE_DURATION);
            foreach (BarcodeType type in session.BarcodeSetStatusForTypeValues.Keys)
            {
                Assert.AreEqual(barcodeTypes.Contains(type), session.BarcodeSetStatusForTypeValues[type]);
            }
        }

        #endregion

        #endregion

        #region UI

        #region UIPlayTune

        [TestMethod]
        public void TestUIPlayTune()
        {
            facade.UIPlay(UI_PLAY_TUNE_TUNE);
            Assert.IsTrue(session.UIPlayTuneHappened);
            Assert.AreEqual(UI_PLAY_TUNE_TUNE, session.UIPlayTuneValue);
        }

        #endregion

        #region UIVibrate

        [TestMethod]
        public void TestUIVibrate()
        {
            facade.UIVibrate();
            Assert.IsTrue(session.UIVibrateHappened);
        }

        #endregion

        #endregion

        #region Handheld

        #region HandheldLogEnabled

        [TestMethod]
        public void TestHandheldLogEnabled()
        {
            bool exceptionHappened = false;
            try
            {
                facade.HandheldLogEnabled(true);
            }
            catch (NotImplementedException)
            {
                exceptionHappened = true;
            }
            Assert.IsTrue(exceptionHappened);
        }

        #endregion

        #region HandheldStop

        [TestMethod]
        public void TestHandheldStop()
        {
            facade.HandheldStop();
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsFalse(session.UILedSetStatusLedBlinkingValue);
        }

        #endregion

        #region HandheldIsConnected

        [TestMethod]
        public void TestHandheldIsConnected()
        {
            bool returnValue = false;
            returnValue = facade.HandheldIsConnected();
            Assert.IsTrue(session.HandheldIsConnectedHappened);
            Assert.IsTrue(returnValue);
        }

        #endregion

        #region HandheldDisconnect

        [TestMethod]
        public void TestHandheldDisconnect()
        {
            facade.HandheldDisconnect();
            Assert.IsTrue(session.HandheldDisconnectHappened);
        }

        #endregion

        #region HandheldGetBatteryPercentage

        [TestMethod]
        public void TestHandheldGetBatteryPercentage()
        {
            facade.HandheldGetBatteryPercentage();
            Assert.IsTrue(session.HandheldGetBatteryPercentageHappened);
        }

        #endregion

        #region HandheldGetInformation

        [TestMethod]
        public void TestHandheldGetInformation()
        {
            DeviceInfo hadheldInfo = facade.HandheldGetInformation();
            Assert.AreEqual(info.NedapProductionCode, hadheldInfo.NedapProductionCode);
            Assert.AreEqual(info.FirmwareVersion, hadheldInfo.FirmwareVersion);
            Assert.AreEqual(info.HardwareVersion, hadheldInfo.HardwareVersion);
        }
        #endregion

        #endregion

        #endregion

        #region Session


        #region Barcode

        [TestMethod]
        public void TestDoOnSessionBarcodeNotReadABarcode()
        {
            //State = BarcodeReadWait
            Collection<BarcodeType> barcodeTypes = new Collection<BarcodeType>();
            barcodeTypes.Add(BarcodeType.Unknown);
            facade.BarcodeRead(barcodeTypes, BARCODE_READ_TYPE_DURATION);

            //State = BarcodeReadStart
            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidContinueHappened);

            session.DoOnSessionBarcodeNotReadABarcode();
            Assert.IsTrue(OnDidPauseHappened);
            Assert.IsTrue(session.UIPlayTuneHappened);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Failure);
            Assert.IsTrue(session.UIVibrateTuneHappened);
            Assert.IsTrue(session.UIVibrateTuneValue == UITune.Failure);

            Assert.IsTrue(OnDidNotReadBarcodeHappened);
        }

        [TestMethod]
        public void TestDoOnSessionBarcodeReadABarcode()
        {
            //State = BarcodeReadWait
            Collection<BarcodeType> barcodeTypes = new Collection<BarcodeType>();
            barcodeTypes.Add(BarcodeType.Unknown);
            facade.BarcodeRead(barcodeTypes, BARCODE_READ_TYPE_DURATION);

            //State = BarcodeReadStart
            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidContinueHappened);

            session.DoOnSessionBarcodeReadABarcode(BARCODE, BARCODETYPE_UNKNOWN);
            Assert.IsTrue(OnDidPauseHappened);
            Assert.IsTrue(session.UIPlayTuneHappened);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Success);
            Assert.IsTrue(session.UIVibrateTuneHappened);
            Assert.IsTrue(session.UIVibrateTuneValue == UITune.Success);

            Assert.IsTrue(OnDidReadBarcodeHappened);
            Assert.AreEqual(BARCODE, OnDidReadBarcodeValue);
            Assert.AreEqual(BARCODETYPE_UNKNOWN, OnDidReadBarcodeType);
        }

        #endregion

        #region NFCCard

        [TestMethod]
        public void TestDoOnSessionNfcReadCard()
        {
            //State = Connect
            session.DoOnSessionHandheldConnect();

            //State = NCFRead
            facade.NfcReadStart();

            session.DoOnSessionNfcReadCard(CARD_NUMBER);
            Assert.IsTrue(session.UILedSetStatusLedBlinkingHappened);
            Assert.IsTrue(session.UILedSetStatusLedBlinkingValueChanged);

            Assert.IsTrue(OnDidReadNfcCardHappened);
            Assert.AreEqual(CARD_NUMBER, OnDidReadNfcCardValue);
        }

        [TestMethod]
        public void TestDoOnSessionNfcNotFoundACard()
        {
            //State = Connect
            session.DoOnSessionHandheldConnect();

            //State = NCFRead
            facade.NfcReadStart();

            session.DoOnSessionNfcNotFoundACard();
            Assert.IsFalse(OnDidReadNfcCardHappened);
        }

        #endregion

        #region Handheld

        [TestMethod]
        public void TestDoOnSessionHandheldConnect()
        {
            //State = Disconnected
            session.DoOnSessionHandheldConnect();
            Assert.IsTrue(OnDidConnectHappened);
            Assert.IsTrue(session.RfidSetRegionHappened);
            Assert.IsTrue(session.RfidSetRegionValue == RfidRegion.Europe);
        }

        [TestMethod]
        public void TestDoOnSessionHandheldDisonnect()
        {
            //State = Disconnected
            session.DoOnSessionHandheldDisconnect();
            Assert.IsFalse(OnDidDisconnectHappened);

            //State = Connect
            session.DoOnSessionHandheldConnect();

            session.DoOnSessionHandheldDisconnect();
            Assert.IsTrue(OnDidDisconnectHappened);

            //State = Connect
            session.DoOnSessionHandheldConnect();

            //State = BarcodeReadWait
            Collection<BarcodeType> barcodeTypes = new Collection<BarcodeType>();
            barcodeTypes.Add(BarcodeType.Unknown);
            facade.BarcodeRead(barcodeTypes, BARCODE_READ_TYPE_DURATION);

            session.DoOnSessionHandheldDisconnect();
            Assert.IsTrue(OnDidDisconnectHappened);
        }

        #region Battery

        [TestMethod]
        public void TestDoOnSessionHandheldReceiveChargingStatus()
        {
            session.DoOnSessionHandheldReceiveChargingStatus();
            Assert.IsTrue(OnChargingStatusHappened);
        }

        [TestMethod]
        public void TestDoOnSessionHandheldReceiveLowBatteryWarning()
        {
            session.DoOnSessionHandheldReceiveLowBatteryWarning();
            Assert.IsTrue(OnLowBatteryWarningHappened);
        }

        [TestMethod]
        public void TestDoOnSessionHandheldReceiveEmptyBatteryStatus()
        {
            session.DoOnSessionHandheldReceiveEmptyBatteryStatus();
            Assert.IsTrue(OnEmptyBatteryStatusHappened);
        }

        [TestMethod]
        public void TestDoOnReceiveBatteryPercentageHappened()
        {
            session.DoOnSessionHandheldReceiveBatteryPercentage(BATTERY_PERCENTAGE);
            Assert.IsTrue(OnReceiveBatteryPercentageHappened);
            Assert.AreEqual(BATTERY_PERCENTAGE, OnReceiveBatteryPercentageValue);
        }

        #endregion

        #endregion

        #region RFID

        [TestMethod]
        public void TestDoOnSessionRfidReadData()
        {
            facade.RfidReadTag(epcCode);

            //State = RFIDReadDetailedTagInformationStart
            session.DoOnSessionUIPushButton();

            session.DoOnSessionRfidReadData(rfidData, MORE_TAGS_AVAILABLE);
            Assert.IsTrue(session.UIVibrateTuneHappened);
            Assert.IsTrue(session.UIVibrateTuneValue == UITune.Success);
            Assert.IsTrue(session.UIVibrateTuneHappened);
            Assert.IsTrue(session.UIVibrateTuneValue == UITune.Success);

            Assert.IsTrue(OnDidReadDetailedTagInformationHappened);
            Assert.IsTrue(OnDidReadDetailedTagInformationTagMdid == (RfidEpcMdid)0x740);
            Assert.IsTrue(OnDidReadDetailedTagInformationTagModelNumber == 0x481);
            Assert.IsFalse(OnDidReadDetailedTagInformationTagXtid);

            session.DoOnSessionRfidReadData(rfidData, MORE_TAGS_UNAVAILABLE);
            Assert.IsTrue(session.UIPlayTuneHappened);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Success);
            Assert.IsTrue(session.UIVibrateTuneHappened);
            Assert.IsTrue(session.UIVibrateTuneValue == UITune.Success);

            Assert.IsTrue(OnDidReadDetailedTagInformationHappened);
            Assert.IsTrue(OnDidReadDetailedTagInformationTagMdid == (RfidEpcMdid)0x740);
            Assert.IsTrue(OnDidReadDetailedTagInformationTagModelNumber == 0x481);
            Assert.IsFalse(OnDidReadDetailedTagInformationTagXtid);
        }

        [TestMethod]
        public void TestDoOnSessionRfidReadTag()
        {
            //State = Connect
            session.DoOnSessionHandheldConnect();

            //State = Connect
            facade.RfidInventoryStart();

            session.DoOnSessionRfidReadTag(epcObs1);
            Assert.IsTrue(OnDidReadRfidTagHappened);
            Assert.AreEqual(epcObs1, OnDidReadRfidTagValue);
        }

        [TestMethod]
        public void TestDoOnSessionRfidReceiveSupportedRegions()
        {
            session.DoOnSessionRfidReceiveSupportedRegions(REGION_FLAGS);
            Assert.IsTrue(OnDidReceiveSupportedRegionsHappened);
            Assert.AreEqual(REGION_FLAGS, OnDidReceiveSupportedRegionsValue);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFinishReadCycleCount0()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            //count = 0

            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidContinueHappened);
            //State = RFIDInventoryRead


            session.DoOnSessionRfidFinishReadCycle();

            Assert.IsTrue(OnDidReadRfidTagsHappened);
            Assert.IsTrue(OnDidReadRFIDTagsEpcObservations != null);

            facade.RfidMonitorStart(select);
            //State = RFIDMonitorWait

            session.DoOnSessionUIPushButton();
            //State = RFIDMonitorRead

            session.DoOnSessionRfidFinishReadCycle();

            Assert.IsTrue(OnDidReadRfidTagsHappened);
            Assert.IsTrue(OnDidReadRFIDTagsEpcObservations != null);

            facade.RfidReadSingle();
            //State = RFIDSingleReadWait

            session.DoOnSessionUIPushButton();
            //State = RFIDSingleReadRead

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(session.UIPlayTuneHappened);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Failure);
            Assert.IsTrue(session.UIVibrateTuneHappened);
            Assert.IsTrue(session.UIVibrateTuneValue == UITune.Failure);

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(OnDidReadRfidTagsHappened);

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            #region State = RFIDWriteEPCVerify
            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(session.RfidWriteEpcHappened);
            Assert.IsTrue(session.RfidWriteEpcCode == epcCode);

            session.DoOnSessionRfidWriteDataMoreTagsAvailable(false);

            #endregion

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(OnDidReadRfidTagsHappened);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFinishReadCycleCount1()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(OnDidReadRfidTagsHappened);
            Assert.IsTrue(OnDidReadRFIDTagsEpcObservations.Contains(epcObs1));

            facade.RfidReadSingle();
            //State = RFIDSingleReadWait

            session.DoOnSessionUIPushButton();
            //State = RFIDSingleReadRead

            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(OnDidReadRfidTagsHappened);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFinishReadCycleCount2()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidReadTag(epcObs1);
            session.DoOnSessionRfidReadTag(epcObs2);
            //count = 2

            session.DoOnSessionRfidFinishReadCycle();
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Short);

            Assert.IsTrue(OnDidReadRfidTagsHappened);
            Assert.IsTrue(OnDidReadRFIDTagsEpcObservations.Contains(epcObs1));
            Assert.IsTrue(OnDidReadRFIDTagsEpcObservations.Contains(epcObs2));
        }

        #endregion

        #region UI

        [TestMethod]
        public void TestDoOnSessionUIPushButton()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            session.DoOnSessionUIPushButton();
            //State = RFIDInventoryRead

            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidPauseHappened);
            Assert.IsTrue(OnDidPushButtonHappened);
        }

        [TestMethod]
        public void TestDoOnSessionUIPushButton1()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            session.DoOnSessionUIPushButton();
            //State = RFIDInventoryRead

            session.DoOnSessionRfidFinishReadCycle();
            //State = RFIDInventoryIdle
            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidPauseHappened);

        }

        [TestMethod]
        public void TestDoOnSessionUIPushButton2()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidMonitorStart(select);
            //State = RFIDMonitorWait

            session.DoOnSessionUIPushButton();
            //State = RFIDMonitorRead

            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidPauseHappened);
        }

        [TestMethod]
        public void TestDoOnSessionUIPushButton3()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidMonitorStart(select);
            //State = RFIDMonitorWait

            session.DoOnSessionUIPushButton();
            //State = RFIDMonitorRead

            session.DoOnSessionRfidFinishReadCycle();
            //State = RFIDMonitorIdle

            session.DoOnSessionUIPushButton();
            Assert.IsTrue(OnDidPauseHappened);
        }

        [TestMethod]
        public void TestDoOnSessionUIShake()
        {
            session.DoOnSessionUIShake();
            Assert.IsTrue(OnDidShakeHappened);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFailButTagsAvailable0()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            //State = RFIDWriteEPCWrite

            session.DoOnSessionRfidFailButTagsAvailable(true);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Short);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFailButTagsAvailable1()
        {
            facade.RfidReadTag(epcCode);
            //State = RFIDReadDetailedTagInformationWait

            session.DoOnSessionUIPushButton();
            //State = RFIDReadDetailedTagInformationWaitStart

            session.DoOnSessionRfidFailButTagsAvailable(true);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Failure);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFailButTagsAvailable2()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            //State = RFIDWriteEPCWrite

            session.DoOnSessionRfidFailButTagsAvailable(false);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Short);
            ///Assert.IsTrue(OnDidFailWriteEpcWriteFailedHappened);
        }

        [TestMethod]
        public void TestDoOnSessionRfidFailButTagsAvailable3()
        {
            facade.RfidReadTag(epcCode);
            //State = RFIDReadDetailedTagInformationWait

            session.DoOnSessionUIPushButton();
            //State = RFIDReadDetailedTagInformationWaitStart

            session.DoOnSessionRfidFailButTagsAvailable(false);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Failure);
        }

        [TestMethod]
        public void TestDoOnSessionRfidWriteDataMoreTagsAvailable0()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            //State = RFIDWriteEPCWrite

            session.DoOnSessionRfidWriteDataMoreTagsAvailable(true);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Short);
            //Assert.IsTrue(OnDidFailWriteEpcWriteFailedHappened);
        }

        [TestMethod]
        public void TestDoOnSessionRfidWriteDataMoreTagsAvailable1()
        {
            facade.RfidReadTag(epcCode);
            //State = RFIDReadDetailedTagInformationWait

            session.DoOnSessionUIPushButton();
            //State = RFIDReadDetailedTagInformationWaitStart

            session.DoOnSessionRfidWriteDataMoreTagsAvailable(true);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Success);
        }

        [TestMethod]
        public void TestDoOnSessionRfidWriteDataMoreTagsAvailable2()
        {
            session.DoOnSessionHandheldConnect();
            //State = Connect

            facade.RfidInventoryStart();
            //State = RFIDInventoryWait

            facade.RfidWriteEpc(epcCode);
            //State = RFIDWriteEPCWait

            session.DoOnSessionUIPushButton();
            //State = RFIDWriteEPCRead

            session.DoOnSessionRfidReadTag(epcObs1);
            //count = 1

            session.DoOnSessionRfidFinishReadCycle();
            //State = RFIDWriteEPCWrite

            session.DoOnSessionRfidWriteDataMoreTagsAvailable(false);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Short);
        }

        [TestMethod]
        public void TestDoOnSessionRfidWriteDataMoreTagsAvailable3()
        {
            facade.RfidReadTag(epcCode);
            //State = RFIDReadDetailedTagInformationWait

            session.DoOnSessionUIPushButton();
            //State = RFIDReadDetailedTagInformationWaitStart

            session.DoOnSessionRfidWriteDataMoreTagsAvailable(false);
            Assert.IsTrue(session.UIPlayTuneValue == UITune.Success);
        }

        #endregion

        #endregion
    }
}