﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Nedap.HandheldApi.CommunicationQueue;

namespace NedapTestCases.LowLevelApiTests
{
    public class PortMock : ICommunicationPort
    {
        public bool ConnectEnabled { get; set; }
        public bool DisconnectEnabled { get; set; }
        public bool IsConnected { get; set; }

        public byte[] WriteBuffer { get; set; }
        public int WriteBufferSize { get; set; }

        public byte[] ReadBuffer { get; set; }

        public PortMock()
        {
            WriteBuffer = new byte[0];
            ReadBuffer = new byte[0];
        }

        #region ICommunicationPort Members

        public bool Connect()
        {
            return ConnectEnabled;
        }

        public bool Disconnect()
        {
            return DisconnectEnabled;
        }

        public bool Connected
        {
            get
            {
                return IsConnected;
            }
            set
            {
                IsConnected = value;
            }
        }

        public int WriteData(byte[] buffer, int offset, int length)
        {
            int writeCount = ((WriteBufferSize != 0) && (WriteBufferSize < buffer.Length)) ? WriteBufferSize : buffer.Length;

            WriteBuffer = new byte[writeCount];
            Buffer.BlockCopy(buffer, 0, WriteBuffer, 0, writeCount);
            return writeCount;
        }

        public int ReadData(byte[] buffer, int offset, int length)
        {
            if (!(ReadBuffer.Length > 0))
            {
                return 0;
            }
            int bufferLength = ReadBuffer.Length;
            Buffer.BlockCopy(ReadBuffer, 0, buffer, 0, bufferLength);
            ReadBuffer = new byte[0];
            return bufferLength;
        }

        public event EventHandler OnDataReceived;

        #endregion

        public void RaiseDataReceivedEvent()
        {
            if (OnDataReceived != null)
            {
                OnDataReceived(this, EventArgs.Empty);
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
        }

        #endregion
    }
}
