﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Nedap.HandheldApi;

namespace NedapTestCases.LowLevelApiTests
{
    public class SessionEventHandlers
    {
        private Session session;

        public SessionEventHandlers(Session testSesion)
        {
            session = testSesion;
        }

        public bool HandheldConnect { get; set; }
        public bool HandheldDisconnect { get; set; }
        public bool HandheldReceiveBatteryPercentage { get; set; }
        public int HandheldReceiveBatteryPercentageValue { get; set; }
        public bool HandheldReceiveLowBatteryWarning { get; set; }
        public bool HandheldReceiveEmptyBatteryStatus { get; set; }
        public bool HandheldReceiveChargingStatus { get; set; }
        public bool HandheldReceiveError { get; set; }
        public string HandheldReceiveErrorMessage { get; set; }
        
        public bool RfidReadTag { get; set; }
        public EpcObservation RfidReadTagEpcObservation { get; set; }
        public int RfidReadTagEventCount { get; set; }
        public bool RfidReadData { get; set; }
        public byte[] RfidReadDataValues { get; set; }
        public bool RfidReadDataMoreTagsAvailable { get; set; }
        public bool RfidFinishReadCycle { get; set; }
        public bool RfidFailButTagsAvailable { get; set; }
        public bool RfidFailButTagsAvailableValue { get; set; }
        public bool RfidWriteDataMoreTagsAvailable { get; set; }
        public bool RfidWriteDataMoreTagsAvailableValue { get; set; }
        public bool RfidReceiveSupportedRegions { get; set; }
        public int RfidReceiveSupportedRegionsFlags { get; set; }
        
        public bool BarcodeReadABarcode { get; set; }
        public BarcodeType BarcodeReadABarcodeType { get; set; }
        public string BarcodeReadABarcodeValue { get; set; }
        public bool BarcodeNotReadABarcode { get; set; }
        
        public bool UIPushButton { get; set; }
        public bool UIShake { get; set; }
        
        public bool NfcNotFoundACard { get; set; }
        public bool NfcReadCard { get; set; }
        public int NfcReadCardSerialNumber { get; set; }

        public void Subscribe()
        {
            RfidReadTagEventCount = 0;
            session.OnSessionHandheldConnect += new EventHandler<EventArgs>(session_OnSessionHandheldConnect);
            session.OnSessionHandheldDisconnect += new EventHandler<EventArgs>(session_OnSessionHandheldDisconnect);
            session.OnSessionHandheldReceiveBatteryPercentage += new EventHandler<HandheldReceiveBatteryPercentageEventArgs>(session_OnSessionHandheldReceiveBatteryPercentage);
            session.OnSessionHandheldReceiveLowBatteryWarning += new EventHandler<EventArgs>(session_OnSessionHandheldReceiveLowBatteryWarning);
            session.OnSessionHandheldReceiveEmptyBatteryStatus += new EventHandler<EventArgs>(session_OnSessionHandheldReceiveEmptyBatteryStatus);
            session.OnSessionHandheldReceiveChargingStatus += new EventHandler<EventArgs>(session_OnSessionHandheldReceiveChargingStatus);
            session.OnSessionHandheldReceiveError += new EventHandler<HandheldReceiveErrorEventArgs>(session_OnSessionHandheldReceiveError);
            session.OnSessionRfidReadTag += new EventHandler<RfidReadTagEventArgs>(session_OnSessionRfidReadTag);
            session.OnSessionRfidReadData += new EventHandler<RfidReadDataEventArgs>(session_OnSessionRfidReadData);
            session.OnSessionRfidFinishReadCycle += new EventHandler<EventArgs>(session_OnSessionRfidFinishReadCycle);
            session.OnSessionRfidFailButTagsAvailable += new EventHandler<RfidFailButTagsAvailableEventArgs>(session_OnSessionRfidFailButTagsAvailable);
            session.OnSessionRfidWriteDataMoreTagsAvailable += new EventHandler<RfidWriteDataMoreTagsAvailableEventArgs>(session_OnSessionRfidWriteDataMoreTagsAvailable);
            session.OnSessionRfidReceiveSupportedRegions += new EventHandler<RfidReceiveSupportedRegionsEventArgs>(session_OnSessionRfidReceiveSupportedRegions);
            session.OnSessionBarcodeRead += new EventHandler<RfidReadABarcodeEventArgs>(session_OnSessionBarcodeReadABarcode);
            session.OnSessionBarcodeNotRead += new EventHandler<EventArgs>(session_OnSessionBarcodeNotReadABarcode);
            session.OnSessionUIPushButton += new EventHandler<EventArgs>(session_OnSessionUIPushButton);
            session.OnSessionUIShake += new EventHandler<EventArgs>(session_OnSessionUIShake);
            session.OnSessionNfcCardNotFound += new EventHandler<EventArgs>(session_OnSessionNfcNotFoundACard);
            session.OnSessionNfcCardRead += new EventHandler<RfidReadCardEventArgs>(session_OnSessionNfcReadCard);
        }

        public void Unsubscribe()
        {
            session.OnSessionHandheldConnect -= new EventHandler<EventArgs>(session_OnSessionHandheldConnect);
            session.OnSessionHandheldDisconnect -= new EventHandler<EventArgs>(session_OnSessionHandheldDisconnect);
            session.OnSessionHandheldReceiveBatteryPercentage -= new EventHandler<HandheldReceiveBatteryPercentageEventArgs>(session_OnSessionHandheldReceiveBatteryPercentage);
            session.OnSessionHandheldReceiveLowBatteryWarning -= new EventHandler<EventArgs>(session_OnSessionHandheldReceiveLowBatteryWarning);
            session.OnSessionHandheldReceiveEmptyBatteryStatus -= new EventHandler<EventArgs>(session_OnSessionHandheldReceiveEmptyBatteryStatus);
            session.OnSessionHandheldReceiveChargingStatus -= new EventHandler<EventArgs>(session_OnSessionHandheldReceiveChargingStatus);
            session.OnSessionHandheldReceiveError -= new EventHandler<HandheldReceiveErrorEventArgs>(session_OnSessionHandheldReceiveError);
            session.OnSessionRfidReadTag -= new EventHandler<RfidReadTagEventArgs>(session_OnSessionRfidReadTag);
            session.OnSessionRfidReadData -= new EventHandler<RfidReadDataEventArgs>(session_OnSessionRfidReadData);
            session.OnSessionRfidFinishReadCycle -= new EventHandler<EventArgs>(session_OnSessionRfidFinishReadCycle);
            session.OnSessionRfidFailButTagsAvailable -= new EventHandler<RfidFailButTagsAvailableEventArgs>(session_OnSessionRfidFailButTagsAvailable);
            session.OnSessionRfidWriteDataMoreTagsAvailable -= new EventHandler<RfidWriteDataMoreTagsAvailableEventArgs>(session_OnSessionRfidWriteDataMoreTagsAvailable);
            session.OnSessionRfidReceiveSupportedRegions -= new EventHandler<RfidReceiveSupportedRegionsEventArgs>(session_OnSessionRfidReceiveSupportedRegions);
            session.OnSessionBarcodeRead -= new EventHandler<RfidReadABarcodeEventArgs>(session_OnSessionBarcodeReadABarcode);
            session.OnSessionBarcodeNotRead -= new EventHandler<EventArgs>(session_OnSessionBarcodeNotReadABarcode);
            session.OnSessionUIPushButton -= new EventHandler<EventArgs>(session_OnSessionUIPushButton);
            session.OnSessionUIShake -= new EventHandler<EventArgs>(session_OnSessionUIShake);
            session.OnSessionNfcCardNotFound -= new EventHandler<EventArgs>(session_OnSessionNfcNotFoundACard);
            session.OnSessionNfcCardRead -= new EventHandler<RfidReadCardEventArgs>(session_OnSessionNfcReadCard);
        }

        void session_OnSessionNfcReadCard(object sender, RfidReadCardEventArgs e)
        {
            NfcReadCard = true;
            NfcReadCardSerialNumber = e.CardNumber;
        }

        void session_OnSessionNfcNotFoundACard(object sender, EventArgs e)
        {
            NfcNotFoundACard = true;
        }

        void session_OnSessionUIShake(object sender, EventArgs e)
        {
            UIShake = true;
        }

        void session_OnSessionUIPushButton(object sender, EventArgs e)
        {
            UIPushButton = true;
        }

        void session_OnSessionBarcodeNotReadABarcode(object sender, EventArgs e)
        {
            BarcodeNotReadABarcode = true;
        }

        void session_OnSessionBarcodeReadABarcode(object sender, RfidReadABarcodeEventArgs e)
        {
            BarcodeReadABarcode = true;
            BarcodeReadABarcodeType = e.BarcodeType;
            BarcodeReadABarcodeValue = e.Barcode;

        }

        void session_OnSessionRfidReceiveSupportedRegions(object sender, RfidReceiveSupportedRegionsEventArgs e)
        {
            RfidReceiveSupportedRegions = true;
            RfidReceiveSupportedRegionsFlags = e.RegionFlags;
        }

        void session_OnSessionRfidWriteDataMoreTagsAvailable(object sender, RfidWriteDataMoreTagsAvailableEventArgs e)
        {
            RfidWriteDataMoreTagsAvailable = true;
            RfidWriteDataMoreTagsAvailableValue = e.Available;
        }

        void session_OnSessionRfidFailButTagsAvailable(object sender, RfidFailButTagsAvailableEventArgs e)
        {
            RfidFailButTagsAvailable = true;
            RfidFailButTagsAvailableValue = e.Available;
        }

        void session_OnSessionRfidFinishReadCycle(object sender, EventArgs e)
        {
            RfidFinishReadCycle = true;
        }

        void session_OnSessionRfidReadData(object sender, RfidReadDataEventArgs e)
        {
            RfidReadData = true;
            RfidReadDataValues = e.Data;
            RfidReadDataMoreTagsAvailable = e.MoreTagsAvailable;
        }

        void session_OnSessionRfidReadTag(object sender, RfidReadTagEventArgs e)
        {
            RfidReadTagEventCount++;
            RfidReadTag = true;
            RfidReadTagEpcObservation = e.EpcObservation;
        }

        void session_OnSessionHandheldReceiveError(object sender, HandheldReceiveErrorEventArgs e)
        {
            HandheldReceiveError = true;
            HandheldReceiveErrorMessage = e.ErrorMessage;
        }

        void session_OnSessionHandheldReceiveChargingStatus(object sender, EventArgs e)
        {
            HandheldReceiveChargingStatus = true;
        }

        void session_OnSessionHandheldReceiveEmptyBatteryStatus(object sender, EventArgs e)
        {
            HandheldReceiveEmptyBatteryStatus = true;
        }

        void session_OnSessionHandheldReceiveLowBatteryWarning(object sender, EventArgs e)
        {
            HandheldReceiveLowBatteryWarning = true;
        }

        void session_OnSessionHandheldReceiveBatteryPercentage(object sender, HandheldReceiveBatteryPercentageEventArgs e)
        {
            HandheldReceiveBatteryPercentage = true;
            HandheldReceiveBatteryPercentageValue = e.Percentage;
        }

        void session_OnSessionHandheldDisconnect(object sender, EventArgs e)
        {
            HandheldDisconnect = true;
        }

        void session_OnSessionHandheldConnect(object sender, EventArgs e)
        {
            HandheldConnect = true;
        }
    }
}
