﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Nedap.HandheldApi;
using System.Threading;
using Nedap.EPC.TDT;
using System.Diagnostics;

namespace NedapTestCases.LowLevelApiTests
{
    /// <summary>
    /// Summary description for SessionTest
    /// </summary>
    [TestClass]
    public class SessionTestCases
    {
        #region Fields

        private Dictionary<string, byte[]> CommandSequences = new Dictionary<string, byte[]>();
        private Dictionary<string, byte[]> ResponseSequences = new Dictionary<string, byte[]>();

        private PortMock portMock = new PortMock();
        private Session session;
        private SessionEventHandlers sessionEventHandlers;
        #endregion

        #region SessionTestCases()
        public SessionTestCases()
        {
            //Commands

            //General
            CommandSequences.Add("TestGeneralGetBatteryPercentage", new byte[] { 0x06, 0x03, 0x01, 0xFC });
            CommandSequences.Add("TestGeneralGetDeviceDetails", new byte[] { 0x06, 0x03, 0x02, 0xFB });
            CommandSequences.Add("TestGeneralDisconnect", new byte[] { 0x06, 0x03, 0x03, 0xFA });

            //UI
            CommandSequences.Add("TestUILedSetStatusLed", new byte[] { 0x06, 0x04, 0x41, 0x02, 0xB9 });//start blinking
            CommandSequences.Add("TestUIBeeperTune", new byte[] { 0x06, 0x04, 0x42, 0x01, 0xB9 });// play longer beep - success
            CommandSequences.Add("TestUIVibrationVibrateNoDuration", new byte[] { 0x06, 0x03, 0x43, 0xBA });
            CommandSequences.Add("TestUIVibrationVibrateWithDuration", new byte[] { 0x06, 0x04, 0x43, 0x0A, 0xAF });//1 second

            //RFID
            CommandSequences.Add("TestRfidReadEpc", new byte[] { 0x06, 0x07, 0x23, 0x00, 0x01, 0x90, 0x17, 0x2E });
            CommandSequences.Add("TestRfidReadEpcWithSelect", new byte[] { 0x06, 0x19, 0x23, 0x01, 0x11, 0x01, 0x00, 0x20, 0x00, 0x60, 0x30, 0x74, 0x04, 0x81, 0x10, 0xF5, 0x1E, 0x80, 0x00, 0x00, 0x00, 0x01, 0x01, 0x90, 0x17, 0xBC });
            CommandSequences.Add("TestRfidReadData", new byte[] { 0x06, 0x0B, 0x24, 0x00, 0x00, 0x64, 0x0A, 0x02, 0x00, 0x00, 0x02, 0x5F });
            CommandSequences.Add("TestRfidWriteData", new byte[] { 0x06, 0x13, 0x25, 0x00, 0x01, 0x90, 0x0A, 0x02, 0x00, 0x00, 0x04, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x67 });

            CommandSequences.Add("TestRfidWriteEpc", new byte[] { 0x06, 0x0D, 0x2D, 0x00, 0x00, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x06 });

            CommandSequences.Add("TestRfidSetRegion", new byte[] { 0x06, 0x04, 0x28, 0x01, 0xD3 });

            CommandSequences.Add("TestRfidSetSession", new byte[] { 0x06, 0x05, 0x29, 0x00, 0x00, 0xD2 });
            CommandSequences.Add("TestRfidSetTarget", new byte[] { 0x06, 0x06, 0x29, 0x01, 0x00, 0x00, 0xD0 });
            CommandSequences.Add("TestRfidSetSetMValue", new byte[] { 0x06, 0x05, 0x29, 0x02, 0x00, 0xD0 });
            CommandSequences.Add("TestRfidSetSetQValue", new byte[] { 0x06, 0x06, 0x29, 0x03, 0x01, 0x02, 0xCB });
            CommandSequences.Add("TestRfidSetCWSignal", new byte[] { 0x06, 0x05, 0x2A, 0x01, 0x01, 0xCF });
            CommandSequences.Add("TestRfidSetFrequencyHopTable", new byte[] { 0x06, 0x0B, 0x2B, 0x00, 0x0D, 0x35, 0xA4, 0x00, 0x0D, 0x3C, 0xAC, 0xEF });
            CommandSequences.Add("TestRfidGetSupportedRegions", new byte[] { 0x06, 0x03, 0x2C, 0xD1 });

            //Barcode
            CommandSequences.Add("TestBarcodeSetStatusForType", new byte[] { 0x06, 0x05, 0x31, 0x04, 0x01, 0xC5 });
            CommandSequences.Add("TestBarcodeTryDecode", new byte[] { 0x06, 0x04, 0x32, 0x14, 0xB6 });

            //NCF
            CommandSequences.Add("TestNfcTryToReadCard", new byte[] { 0x06, 0x03, 0x51, 0xAC });


            //Responces

            //General
            ResponseSequences.Add("TestGeneralBatteryPercentage", new byte[] { 0x06, 0x04, 0x81, 0x64, 0x17 });
            ResponseSequences.Add("TestGeneralDeviceDetails", new byte[] { 0x06, 0x15, 0x82, 0x42, 0x33, 0x31, 0x30, 0x30, 0x31, 0x30, 0x31, 0x00, 0x00, 0xFF, 0xFF, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0xd1 });
            ResponseSequences.Add("TestGeneralLowBattery", new byte[] { 0x06, 0x03, 0x83, 0x7A });
            ResponseSequences.Add("TestGeneralEmptyBattery", new byte[] { 0x06, 0x03, 0x84, 0x79 });
            ResponseSequences.Add("TestGeneralDeviceCharging", new byte[] { 0x06, 0x03, 0x85, 0x78 });

            //UI
            ResponseSequences.Add("TestUIButtonPush", new byte[] { 0x06, 0x03, 0xC1, 0x3C });
            ResponseSequences.Add("TestUIAccelerometerShake", new byte[] { 0x06, 0x03, 0xC2, 0x3B });

            //Errors
            ResponseSequences.Add("TestHandheldReceiveError", new byte[] { 0x06, 0x04, 0xFF, 0x31, 0xCC });

            //RFID
            ResponseSequences.Add("TestRfidFinishReadCycleNoTagsAvailable", new byte[] { 0x06, 0x04, 0xA1, 0x00, 0x5B });
            ResponseSequences.Add("TestRfidReadDataResponseFail", new byte[] { 0x06, 0x05, 0xA3, 0x01, 0x00, 0x57 });
            ResponseSequences.Add("TestRfidReadDataResponseSuccess", new byte[] { 0x06, 0x08, 0xA3, 0x01, 0x01, 0x01, 0x23, 0x45, 0xEA });
            ResponseSequences.Add("TestRfidReadDataOneTag", new byte[] { 0x06, 0x15, 0xA2, 0x01, 0x01, 0x01, 0x20, 0x00, 0x60, 0x01, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0xF0, 0x05 });
            ResponseSequences.Add("TestRfidReadDataTwoTags", new byte[] { 0x06, 0x25, 0xA2, 0x01, 0x02, 0x01, 0x20, 0x00, 0x60, 0x01, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0xF0,
                                                                                                        0x02, 0x20, 0x00, 0x60, 0x01, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0xF0, 0xB1 });

            ResponseSequences.Add("TestRfidWriteDataResponce", new byte[] { 0x06, 0x05, 0xA4, 0x02, 0x01, 0x54 });
            ResponseSequences.Add("TestRfidWriteDataResponceFail", new byte[] { 0x06, 0x05, 0xA4, 0x02, 0x00, 0x55 });

            ResponseSequences.Add("TestRfidReceiveSupportedRegions", new byte[] { 0x06, 0x04, 0xA5, 0x02, 0x55 });

            //Barcode
            ResponseSequences.Add("TestBarcodeNotReadABarcode", new byte[] { 0x06, 0x03, 0xB2, 0x4B });
            ResponseSequences.Add("TestBarcodeReadABarcode", new byte[] { 0x06, 0x0C, 0xB1, 0xFF, 0x31, 0x33, 0x35, 0x37, 0x39, 0x3B, 0x3D, 0x3F, 0x84 });

            //NCF
            ResponseSequences.Add("TestNfcNotFoundACard", new byte[] { 0x06, 0x03, 0xD1, 0x2C });
            ResponseSequences.Add("TestNfcReadCard", new byte[] { 0x06, 0x07, 0xD2, 0x01, 0x23, 0x45, 0x67, 0x57 });

            //Partial
            ResponseSequences.Add("TestRfidReadDataFirstHalfOfOneTag", new byte[] { 0x06, 0x15, 0xA2, 0x01, 0x01, 0x01, 0x20, 0x00, 0x60 });
            ResponseSequences.Add("TestRfidReadDataSecondHalfOfOneTag", new byte[] { 0x01, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0xF0, 0x05 });
            ResponseSequences.Add("TestRfidReadDataFirstHalfOfOneTagWithBadDataPreceding", new byte[] { 0xFF, 0x83, 0x00, 0x01, 0x66, 0x06, 0x15, 0xA2, 0x01, 0x01, 0x01, 0x20, 0x00, 0x60 });
            ResponseSequences.Add("TestRfidReadDataSecondHalfOfOneTagAndFirstHalfOfOneTag", new byte[] { 0x01, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF, 0x00, 0xF0, 0x05, 
                                                                                                            0x06, 0x15, 0xA2, 0x01, 0x01, 0x01, 0x20, 0x00, 0x60});
            //Other
            ResponseSequences.Add("TestCommunicationQueueMultipleResponses", new byte[] { 0x06, 0x03, 0xC1, 0x3C,
                                                                                            0x06, 0x04, 0x81, 0x64, 0x17,
                                                                                            0x06, 0x0C, 0xB1, 0xFF, 0x31, 0x33, 0x35, 0x37, 0x39, 0x3B, 0x3D, 0x3F, 0x84 });

            portMock.ConnectEnabled = true;
            portMock.DisconnectEnabled = true;
            portMock.IsConnected = true;
        }
        #endregion

        #region Test initialize and cleanup

        [TestInitialize()]
        public void MyTestInitialize()
        {
            session = new Session(portMock, true);
            sessionEventHandlers = new SessionEventHandlers(session);
            sessionEventHandlers.Subscribe();
        }

        [TestCleanup()]
        public void MyTestCleanup()
        {
            sessionEventHandlers.Unsubscribe();
            session.Dispose();
        }

        #endregion

        #region General command/response tests

        [TestMethod]
        public void TestGeneralGetBatteryPercentage()
        {
            //command
            session.HandheldGetBatteryPercentage();
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestGeneralGetBatteryPercentage"]);
        }

        [TestMethod]
        public void TestGeneralGetDeviceDetails()
        {
            
            //To call session.HandheldGetDeviceDetails(); we need to use connection state and to sent batery percentage to connection state.
            //After that connection state will call HandheldGetDeviceDetails();
            portMock.ReadBuffer = ResponseSequences["TestGeneralBatteryPercentage"];
            portMock.RaiseDataReceivedEvent();

            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestGeneralGetDeviceDetails"]);
        }

        [TestMethod]
        public void TestGeneralDeviceDetails()
        {
            portMock.ReadBuffer = ResponseSequences["TestGeneralDeviceDetails"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(3000);

            DeviceInfo info = session.HandheldGetInformation();
            Assert.AreEqual(info.NedapProductionCode, "B310 0 101");
            Assert.AreEqual(info.SerialNumber, "101");
            Assert.AreEqual(info.ModelNumber, "65535");
            Assert.AreEqual(info.FirmwareVersion, "1.0.0");
            Assert.AreEqual(info.HardwareVersion, "1.0.0");
        }

        [TestMethod]
        public void TestGeneralBatteryPercentage()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestGeneralBatteryPercentage"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveBatteryPercentage);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveBatteryPercentageValue == 100);
        }

        [TestMethod]
        public void TestGeneralLowBattery()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestGeneralLowBattery"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveLowBatteryWarning);
        }

        [TestMethod]
        public void TestGeneralEmptyBattery()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestGeneralEmptyBattery"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveEmptyBatteryStatus);
        }
        [TestMethod]
        public void TestGeneralDeviceCharging()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestGeneralDeviceCharging"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveChargingStatus);
        }

        [TestMethod]
        public void TestGeneralDisconnect()
        {
            //command
            session.HandheldDisconnect();
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestGeneralDisconnect"]);
        }

        #endregion

        #region UI command/response tests

        [TestMethod]
        public void TestUILedSetStatusLed()
        {
            //command
            session.UILedSetBlinking(true);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestUILedSetStatusLed"]);
        }

        [TestMethod]
        public void TestUIPlayTune()
        {
            //command
            session.UIPlay(UITune.Success);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestUIBeeperTune"]);
        }

        [TestMethod]
        public void TestUIVibrationVibrateNoDuration()
        {
            //command
            session.UIVibrate();
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestUIVibrationVibrateNoDuration"]);
        }
        [TestMethod]

        public void TestUIVibrationVibrateWithDuration()
        {
            //command 
            session.UIVibrate(10);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestUIVibrationVibrateWithDuration"]);
        }

        [TestMethod]
        public void TestUIButtonPush()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestUIButtonPush"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.UIPushButton);
        }

        [TestMethod]
        public void TestUIAccelerometerShake()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestUIAccelerometerShake"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.UIShake);
        }
        #endregion

        #region RFID commmand/response tests

        [TestMethod]
        public void TestRfidReadEpc()
        {
            //command
            session.RfidReadEpcs(400, 23, null);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidReadEpc"]);
        }

        [TestMethod]
        public void TestRfidReadEpcWithSelect()
        {
            //command
            session.RfidReadEpcs(400, 23, new Select(new Nedap.EPC.TDT.EpcCode(new byte[] { 0x30, 0x74, 0x04, 0x81, 0x10, 0xF5, 0x1E, 0x80, 0x00, 0x00, 0x00, 0x01 })));
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidReadEpcWithSelect"]);
        }

        [TestMethod]
        public void TestRfidReadDataOneTag()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataOneTag"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsTrue(sessionEventHandlers.RfidReadTag);
            EpcObservation observation = new EpcObservation(new EpcCode(new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }), DateTime.Now, 20, 1);
            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadTagEpcObservation.EpcCode.BinaryData, observation.EpcCode.BinaryData);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.RSSI == 0x20);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.ReadCount == 1);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);
        }

        [TestMethod]
        public void TestRfidReadDataTwoTags()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataTwoTags"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsTrue(sessionEventHandlers.RfidReadTag);
            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadTagEpcObservation.EpcCode.BinaryData, new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF });
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.RSSI == 0x20);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.ReadCount == 2);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);
        }

        [TestMethod]
        public void TestRfidReadData()
        {
            //command
            session.RfidReadData(RfidEpcMemoryBank.Tid, 0, 2, 100, 10, null, null);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidReadData"]);
        }

        [TestMethod]
        public void TestRfidReadDataResponseFail()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataResponseFail"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.RfidFailButTagsAvailable);
            Assert.IsTrue(sessionEventHandlers.RfidFailButTagsAvailableValue = true);
        }

        [TestMethod]
        public void TestRfidReadDataResponseSuccess()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataResponseSuccess"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.RfidReadData);
            Assert.IsTrue(sessionEventHandlers.RfidReadDataMoreTagsAvailable == false);
            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadDataValues, new byte[] { 0x01, 0x23, 0x45 });
        }

        [TestMethod]
        public void TestRfidFinishReadCycleNoTagsAvailable()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidFinishReadCycleNoTagsAvailable"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);
        }

        [TestMethod]
        public void TestRfidWriteData()
        {
            //command
            session.RfidWriteData(new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }, RfidEpcMemoryBank.Tid, 0, 400, 10, null, null);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidWriteData"]);
        }

        [TestMethod]
        public void TestRfidWriteEpc()
        {
            //command
            session.RfidWriteEpc(new EpcCode(new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF }));
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidWriteEpc"]);
        }

        [TestMethod]
        public void TestRfidWriteDataResponce()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidWriteDataResponce"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.RfidWriteDataMoreTagsAvailable);
            Assert.IsTrue(sessionEventHandlers.RfidWriteDataMoreTagsAvailableValue = true);
        }

        [TestMethod]
        public void TestRfidWriteDataResponceFail()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidWriteDataResponceFail"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.RfidFailButTagsAvailable);
            Assert.IsTrue(sessionEventHandlers.RfidFailButTagsAvailableValue = true);
        }

        [TestMethod]
        public void TestRfidSetRegion()
        {
            //command
            session.RfidSetRegion(RfidRegion.Europe);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetRegion"]);
        }

        [TestMethod]
        public void TestRfidReceiveSupportedRegions()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReceiveSupportedRegions"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.RfidReceiveSupportedRegions);
            Assert.IsTrue(sessionEventHandlers.RfidReceiveSupportedRegionsFlags == 2);
        }

        [TestMethod]
        public void TestRfidSetSession()
        {
            //command
            session.RfidSetSession(RfidEpcSession.Session0);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetSession"]);
        }

        [TestMethod]
        public void TestRfidSetTarget()
        {
            //command
            session.RfidSetTarget(RfidEpcTarget.TargetA, false);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetTarget"]);
        }

        [TestMethod]
        public void TestRfidSetMValue()
        {
            //command
            session.RfidSetMValue(RfidEpcMValue.Value0);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetSetMValue"]);
        }

        [TestMethod]
        public void TestRfidSetQValue()
        {
            //command
            session.RfidSetQValue(2, true);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetSetQValue"]);
        }

        [TestMethod]
        public void TestRfidSetCWSignal()
        {
            //command
            session.RfidSetCWSignal(true, 2);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetCWSignal"]);
        }

        [TestMethod]
        public void TestRfidSetFrequencyHopTable()
        {
            //command
            session.RfidSetRegion(RfidRegion.Europe);
            session.RfidSetFrequencyHopTable(new List<RfidFrequency>() { RfidFrequency.Europe865700, RfidFrequency.Europe867500 });
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidSetFrequencyHopTable"]);
        }

        [TestMethod]
        public void TestRfidGetSupportedRegions()
        {
            //command
            session.RfidGetSupportedRegions();
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestRfidGetSupportedRegions"]);
        }

        #endregion

        #region Barcode commmand/response tests

        [TestMethod]
        public void TestBarcodeHasReader()
        {
            //Simulate retrieval of device information
            portMock.ReadBuffer = ResponseSequences["TestGeneralDeviceDetails"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(1000);

            Assert.IsTrue(session.BarcodeHasReader());
        }

        [TestMethod]
        public void TestBarcodeSetStatusForType()
        {
            //command
            session.BarcodeSetStatusForType(BarcodeType.Upce, true);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestBarcodeSetStatusForType"]);
        }

        [TestMethod]
        public void TestBarcodeTryDecode()
        {
            //command
            session.BarcodeTryDecode(2);
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestBarcodeTryDecode"]);
        }

        [TestMethod]
        public void TestBarcodeNotReadABarcode()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestBarcodeNotReadABarcode"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.BarcodeNotReadABarcode);
        }

        [TestMethod]
        public void TestBarcodeReadABarcode()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestBarcodeReadABarcode"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.BarcodeReadABarcode);
            Assert.IsTrue(sessionEventHandlers.BarcodeReadABarcodeType == BarcodeType.Unknown);
            Assert.AreEqual(sessionEventHandlers.BarcodeReadABarcodeValue, "13579;=?");

        }

        #endregion

        #region NFC commmand/response tests

        [TestMethod]
        public void TestNfcTryToReadCard()
        {
            //command
            session.NfcTryToReadCard();
            Thread.Sleep(500);
            CollectionAssert.AreEqual(portMock.WriteBuffer, CommandSequences["TestNfcTryToReadCard"]);
        }

        [TestMethod]
        public void TestNfcNotFoundACard()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestNfcNotFoundACard"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.NfcNotFoundACard);
        }

        [TestMethod]
        public void TestNfcReadCard()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestNfcReadCard"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.NfcReadCard && (sessionEventHandlers.NfcReadCardSerialNumber == 0x01234567));
        }

        #endregion

        #region Error responce

        [TestMethod]
        public void TestHandheldReceiveError()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestHandheldReceiveError"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveError);
            Assert.AreEqual(sessionEventHandlers.HandheldReceiveErrorMessage, "Barcode: Not available.");
        }
        #endregion

        #region Partial responce tests

        [TestMethod]
        public void TestPartialRfidReadData()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataFirstHalfOfOneTag"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsFalse(sessionEventHandlers.RfidReadTag);

            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataSecondHalfOfOneTag"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsTrue(sessionEventHandlers.RfidReadTag);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEventCount == 1);

            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadTagEpcObservation.EpcCode.BinaryData, new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF });
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.RSSI == 0x20);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.ReadCount == 1);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);
        }

        [TestMethod]
        public void TestPartialRfidReadDataWithBadDataPreceding()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataFirstHalfOfOneTagWithBadDataPreceding"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsFalse(sessionEventHandlers.RfidReadTag);

            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataSecondHalfOfOneTag"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsTrue(sessionEventHandlers.RfidReadTag);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEventCount == 1);

            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadTagEpcObservation.EpcCode.BinaryData, new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF });
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.RSSI == 0x20);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.ReadCount == 1);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);
        }

        [TestMethod]
        public void TestPartialRfidReadDataWithBadDataPrecedingTwo()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataFirstHalfOfOneTagWithBadDataPreceding"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsFalse(sessionEventHandlers.RfidReadTag);

            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataSecondHalfOfOneTagAndFirstHalfOfOneTag"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsTrue(sessionEventHandlers.RfidReadTag);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEventCount == 1);

            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadTagEpcObservation.EpcCode.BinaryData, new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF });
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.RSSI == 0x20);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.ReadCount == 1);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);

            portMock.ReadBuffer = ResponseSequences["TestRfidReadDataSecondHalfOfOneTag"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);

            Assert.IsTrue(sessionEventHandlers.RfidReadTag);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEventCount == 2);

            CollectionAssert.AreEqual(sessionEventHandlers.RfidReadTagEpcObservation.EpcCode.BinaryData, new byte[] { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF });
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.RSSI == 0x20);
            Assert.IsTrue(sessionEventHandlers.RfidReadTagEpcObservation.ReadCount == 1);
            Assert.IsTrue(sessionEventHandlers.RfidFinishReadCycle);
        }

        #endregion

        #region Other tests

        [TestMethod]
        public void TestHandheldIsConnected()
        {
            Assert.IsTrue(portMock.Connected == session.HandheldIsConnected());
            portMock.IsConnected = false;
            Assert.IsTrue(portMock.Connected == session.HandheldIsConnected());
        }

        [TestMethod]
        public void TestConnectionState()
        {
            Thread.Sleep(2500);
            Assert.IsFalse(sessionEventHandlers.HandheldConnect);
            portMock.ReadBuffer = ResponseSequences["TestGeneralBatteryPercentage"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.HandheldConnect);
        }

        [TestMethod]
        public void TestCommunicationQueueMultipleResponses()
        {
            //response
            portMock.ReadBuffer = ResponseSequences["TestCommunicationQueueMultipleResponses"];
            portMock.RaiseDataReceivedEvent();
            Thread.Sleep(500);
            Assert.IsTrue(sessionEventHandlers.UIPushButton);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveBatteryPercentage);
            Assert.IsTrue(sessionEventHandlers.HandheldReceiveBatteryPercentageValue == 100);
            Assert.IsTrue(sessionEventHandlers.BarcodeReadABarcode);
            Assert.IsTrue(sessionEventHandlers.BarcodeReadABarcodeType == BarcodeType.Unknown);
            Assert.AreEqual(sessionEventHandlers.BarcodeReadABarcodeValue, "13579;=?");
        }

        #endregion
    }
}