﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Nedap.EPC.TDT;
using System.Xml;
using System.IO;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace NedapTestCases.TdtLibraryTests
{
    /// <summary>
    /// Summary description for Test1
    /// </summary>
    [TestClass]
    public class TdtLibraryTests
    {
        static string DefinitionsDirectory;
        private TestContext testContextInstance;
        private string inputVal;

        public TdtLibraryTests()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        [ClassInitialize()]
        public static void MyClassInitialize(TestContext testContext)
        {
            TDTEngine engine = new TDTEngine();
            engine.Initialize();
        }
        #endregion

        [TestMethod]
        public void SGTINValidSchemeType()
        {
            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "96");

            string result = TDTEngine.Translate("001100000111010000000100100000010001000011110101000111101000000000000000000000000000000000000001", data, LevelTypeList.LEGACY);
            Assert.AreEqual("gtin=20073796510026;serial=1", result);

        }

        [TestMethod]
        public void SGTINValidBinaryOutput()
        {
            string inputParms = "taglength=96;filter=3;gs1companyprefixlength=7;";

            string result = TDTEngine.Translate("gtin=20073796510026;serial=1", inputParms, "BINARY");
            Assert.AreEqual("001100000111010000000100100000010001000011110101000111101000000000000000000000000000000000000001", result);
        }


        [TestMethod]
        public void SGTINValidTagURIOutput()
        {
            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "96");
            data.Add("filter", "3");
            data.Add("serial", "1");
            data.Add("gs1companyprefixlength", "7");

            string result = TDTEngine.Translate("urn:epc:id:sgtin:0073796.251002.1", data, LevelTypeList.TAG_ENCODING);
            Assert.AreEqual("urn:epc:tag:sgtin-96:3.0073796.251002.1", result);
        }
        [TestMethod]
        public void SGTINValidPureIdentityOutput()
        {
            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "96");
            data.Add("filter", "3");
            data.Add("serial", "1");
            data.Add("gs1companyprefixlength", "7");

            string result = TDTEngine.Translate("001100000111010000000100100000010001000011110101000111101000000000000000000000000000000000000001", data, LevelTypeList.PURE_IDENTITY);
            Assert.AreEqual("urn:epc:id:sgtin:0073796.251002.1", result);
        }

        [TestMethod]
        public void SGTINValidateFilterValue()
        {
            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "96");
            data.Add("filter", "0");
            data.Add("serial", "1");
            data.Add("gs1companyprefixlength", "7");

            string result = TDTEngine.Translate("urn:epc:id:sgtin:0073796.251002.1", data, LevelTypeList.BINARY);
            Assert.AreEqual("001100000001010000000100100000010001000011110101000111101000000000000000000000000000000000000001", result);
        }


        [TestMethod]
        public void MultipleTestsFromXmlTestCases
()
        {
            string filePath = (new System.Uri(Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase))).LocalPath + Path.DirectorySeparatorChar + "MultipleTestsFromXmlTestCasesResults.txt";
            string traceFilePath = (new System.Uri(Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase))).LocalPath + Path.DirectorySeparatorChar + "Trace.txt";
            StreamWriter file = new StreamWriter(filePath, true);
            StreamWriter traceFile = new StreamWriter(traceFilePath, true);
            DefinitionsDirectory = (new System.Uri(Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase))).LocalPath + Path.DirectorySeparatorChar + "TestDefinitions" + Path.DirectorySeparatorChar;

            // check all files in the definitions-directory
            DirectoryInfo directoryInfo = new DirectoryInfo(DefinitionsDirectory);
            FileInfo[] fileInfos = directoryInfo.GetFiles("*.xml");

            foreach (FileInfo fileInfo in fileInfos)
            {


                XmlReader xmlReader = new XmlTextReader(DefinitionsDirectory + fileInfo.Name);
                System.Xml.Serialization.XmlSerializer ser = new System.Xml.Serialization.XmlSerializer(typeof(testcases));
                testcases testcases = (testcases)ser.Deserialize(xmlReader);
                foreach (testcasesTestcase inputValue in testcases.testcase)
                {



                    string number = Regex.Replace(inputValue.type, "[^0-9]", "");//input level bytes number
                    string typeFormat = Regex.Replace(inputValue.type, "[^A-Z]", "");//input level name
                    //string methodName = Regex.Replace(inputValue.name, "\b[^A-Z][^a-z]+", "");//input method name without numbers
                    string methodName = Regex.Replace(inputValue.name, "[0-9 -]", "");//input method name without numbers

                    string filter;
                    string serial;
                    string companyprefixlength;

                    if (inputValue.input.filter != "" && inputValue.input.filter != null)
                    {
                        filter = inputValue.input.filter;
                    }
                    else
                    {
                        filter = "";
                    }

                    if (inputValue.input.serial != "" && inputValue.input.serial != null)
                    {
                        serial = inputValue.input.serial;
                    }
                    else
                    {
                        serial = "";
                    }
                    if (inputValue.input.companyprefixlength != 0)
                    {
                        companyprefixlength = inputValue.input.companyprefixlength.ToString();
                    }
                    else
                    {
                        companyprefixlength = "";
                    }



                    Dictionary<String, String> data = new Dictionary<String, String>();
                    data.Add("taglength", number);
                    if (filter != "")
                    {
                        data.Add("filter", filter);
                    }
                    if (serial != "" && typeFormat != "SGLN" && typeFormat != "SGTIN" && typeFormat != "USDOD")
                    {
                        data.Add("serial", serial);
                    }
                    if (companyprefixlength != "")
                    {
                        data.Add("gs1companyprefixlength", companyprefixlength);
                    }

                    if (typeFormat == "GIAI" && !Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {
                        StringBuilder str = new StringBuilder();
                        str.Append("giai=");
                        str.Append(inputValue.input.value);

                        inputVal = str.ToString();
                    }
                    if (typeFormat == "SGTIN" && !Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {
                        StringBuilder str = new StringBuilder();
                        str.Append("gtin=");
                        str.Append(inputValue.input.value);

                        if (serial != "")
                        {
                            str.Append(";");
                            str.Append("serial=");
                            str.Append(serial);
                            //str.Append(";");
                        }

                        inputVal = str.ToString();
                    }
                    if (typeFormat == "SSCC" && !Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {
                        StringBuilder str = new StringBuilder();
                        str.Append("sscc=");
                        str.Append(inputValue.input.value);

                        inputVal = str.ToString();
                    }
                    if (typeFormat == "SGLN" && !Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {
                        StringBuilder str = new StringBuilder();
                        str.Append("gln=");
                        str.Append(inputValue.input.value);


                        if (serial != "")
                        {
                            str.Append(";");
                            str.Append("serial=");
                            str.Append(serial);
                            //str.Append(";");
                        }

                        inputVal = str.ToString();
                    }
                    if (typeFormat == "GRAI" && !Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {
                        StringBuilder str = new StringBuilder();
                        str.Append("grai=");
                        str.Append(inputValue.input.value);

                        inputVal = str.ToString();
                    }
                    if (typeFormat == "USDOD" && !Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {
                        StringBuilder str = new StringBuilder();
                        str.Append("cageordodaac=");
                        str.Append(inputValue.input.value);

                        if (serial != "")
                        {
                            str.Append(";");
                            str.Append("serial=");
                            str.Append(serial);
                            str.Append(";");
                        }

                        inputVal = str.ToString();
                    }

                    if (Regex.IsMatch(inputValue.input.value, @"^[0-1]+$"))
                    {

                        inputVal = inputValue.input.value;

                    }



                    try
                    {


                        if (filter == "" && serial == "" && companyprefixlength == "")
                        {
                            string result = null;
                            result = TDTEngine.Translate(inputVal, (LevelTypeList)Enum.Parse(typeof(LevelTypeList), inputValue.input.outputlevel, true));
                            Assert.AreEqual(inputValue.output.value, result);

                            Trace.WriteLine("************************************************************************");
                            Trace.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + " result:" + result + "                PASSED");
                            file.WriteLine("************************************************************************");
                            file.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + " result:" + result + "                PASSED");




                        }
                        if (filter != "" || serial != "" || companyprefixlength != "")
                        {
                            string result = null;
                            result = TDTEngine.Translate(inputVal, data, (LevelTypeList)Enum.Parse(typeof(LevelTypeList), inputValue.input.outputlevel, true));
                            Assert.AreEqual(inputValue.output.value, result);


                            Trace.WriteLine("************************************************************************");
                            Trace.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + " result:" + result + "                PASSED");
                            file.WriteLine("************************************************************************");
                            file.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + " result:" + result + "                PASSED");


                        }


                    }

                    catch (Nedap.EPC.TDT.TDTTranslationException te)
                    {
                        if (inputValue.name.ToString().Contains("Invalid") == false)
                        {
                            if (inputValue.name.ToString().Contains("Valid") == true)
                            {

                                Trace.WriteLine("************************************************************************");
                                Trace.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + te.Message + "    FAILED");
                                file.WriteLine("************************************************************************");
                                file.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + te.Message + "    FAILED");
                            }
                        }
                        else
                        {
                            Trace.WriteLine("************************************************************************");
                            Trace.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + te.Message + "      PASSED");
                            file.WriteLine("************************************************************************");
                            file.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + te.Message + "      PASSED");

                        }



                    }
                    catch (Exception e)
                    {
                        if (inputValue.name.ToString().Contains("Invalid") == false)
                        {
                            if (inputValue.name.ToString().Contains("Valid") == true)
                            {
                                Trace.WriteLine("************************************************************************");
                                Trace.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + e.Message + "    FAILED");
                                file.WriteLine("************************************************************************");
                                file.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + e.Message + "    FAILED");
                                var trace = new System.Diagnostics.StackTrace(e);
                                traceFile.Write(trace + " " + inputValue.type + " " + inputValue.name);
                            }

                        }
                        else
                        {
                            Trace.WriteLine("************************************************************************");
                            Trace.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + e.Message + "      PASSED");
                            file.WriteLine("************************************************************************");
                            file.WriteLine("Level Type:  " + inputValue.type + "; " + "Method Name: " + inputValue.name + "; " + "Exception: " + e.Message + "      PASSED");
                            var trace = new System.Diagnostics.StackTrace(e);
                            traceFile.Write(trace + " " + inputValue.type + " " + inputValue.name);

                        }


                    }

                }


            }

            file.Close();
            traceFile.Close();
        }

        [TestMethod]
        public void Test()
        {

            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "64");
            data.Add("filter", "0");
            // data.Add("serial", "1");
            data.Add("gs1companyprefixlength", "7");

            string result = TDTEngine.Translate("giai=007379600000000000000001", data, LevelTypeList.BINARY);
            Assert.AreEqual("0000101100000000010011110000000000000000000000000000000000000001", result);

        }


        [TestMethod]
        public void Test2()
        {

            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "96");
            //data.Add("filter", "2");
            //data.Add("serial", "99999999999");
            //data.Add("gs1companyprefixlength", "6");
            try
            {
                string result = TDTEngine.Translate("1111111100001000001100101011001011011101110110010111001011100101110010111001011100101110010111001011100101110010111001011100101110010111001011100100000000000000000000000000000000000000000000000000000000", data, LevelTypeList.LEGACY);
                Assert.AreEqual("", result);
            }

            catch (Nedap.EPC.TDT.TDTTranslationException te)
            {

                Trace.WriteLine("************************************************************************");
                Trace.WriteLine("Exception: " + te.Message);



            }
            catch (Exception e)
            {

                Trace.WriteLine("************************************************************************");
                Trace.WriteLine("Exception: " + e.Message);



            }

        }

        [TestMethod]

        public void Test3()
        {

            Dictionary<String, String> data = new Dictionary<String, String>();
            data.Add("taglength", "64");
            data.Add("filter", "0");
            //data.Add("serial", "1");
            data.Add("gs1companyprefixlength", "7");
            try
            {
                string result = TDTEngine.Translate("0000101000000000010011110000000000000100101100000000000000000001", data, LevelTypeList.LEGACY);
                Assert.AreEqual("grai=000737960015061", result);
            }

            catch (Nedap.EPC.TDT.TDTTranslationException te)
            {

                Trace.WriteLine("************************************************************************");
                Trace.WriteLine("Exception: " + te.Message);



            }
            catch (Exception e)
            {

                Trace.WriteLine("************************************************************************");
                Trace.WriteLine("Exception: " + e.Message);



            }
        }

    }


}
