﻿using System;
using System.IO.Ports;
using Nedap.HandheldApi;
using Nedap.HandheldApi.CommunicationPort;
using Nedap.EPC.TDT;
using Nedap.HandheldApi.CommunicationQueue;

namespace RetailDemoApp
{
    /// <summary>
    /// This is crucial class since it contains implementation for everything that works
    /// behind application UI (bluetooth port, low and high level APIs for handheld device).
    /// </summary>
    public class Controller : IDisposable
    {
        #region Properties

        public ICommunicationPort Port { get; private set; }
        public Session Session { get; private set; }
        public PresentationFacade Facade { get; private set; }

        #endregion

        #region Controller() - constructor
        public Controller()
        {
            Nedap.EPC.TDT.TDTEngine engine = new Nedap.EPC.TDT.TDTEngine();
            XmlDefinitions[] definitionsList = { XmlDefinitions.SGLN_96, XmlDefinitions.SGTIN_96 };
            engine.Initialize(definitionsList);

            //if 32feet.net library is not supported on the target device, exception will be thrown
            //For these devices SerialCommunicationPort can be used instead:
            //e.g. Port = new SerialCommunicationPort(portName, 38400, Parity.None, 8, StopBits.One);
            //This demo app will use 32feet.net library and BluetoothCommunicationPort.
            Port = new BluetoothCommunicationPort();
            Session = new Session(Port, true);
            Facade = new PresentationFacade(Session);
        }
        #endregion

        #region IDisposable Members
        public void Dispose()
        {
            if (Session != null)
            {
                Facade.Dispose();
                Port.Dispose();
                Session.Dispose();
            }
        }
        #endregion
    }
}