﻿using System.Windows.Forms;

namespace RetailDemoApp
{
    public partial class frmBarcode : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private string labelTypeText = "type:";
        private int barcodeReadDuration = 15;
        private delegate void UpdateUIDelegate(string typeValue, string barcode);

        #endregion

        #region frmBarcode() - constructor
        public frmBarcode()
        {
            InitializeComponent();

            commonMenu.MenuItems[0].MenuItems.RemoveAt(3);
            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region frmBarcode_Activated(object sender, System.EventArgs e)
        private void frmBarcode_Activated(object sender, System.EventArgs e)
        {
            ClearControls();

            Program.Controller.Facade.OnReadBarcode += new System.EventHandler<Nedap.HandheldApi.ReadBarcodeEventArgs>(facade_OnReadBarcode);
            Program.Controller.Facade.BarcodeRead(Nedap.HandheldApi.BarcodeType.All, barcodeReadDuration);
        }
        #endregion

        #region frmBarcode_Deactivate(object sender, System.EventArgs e)
        private void frmBarcode_Deactivate(object sender, System.EventArgs e)
        {
            ClearControls();

            Program.Controller.Facade.OnReadBarcode -= new System.EventHandler<Nedap.HandheldApi.ReadBarcodeEventArgs>(facade_OnReadBarcode);
            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #region facade_OnReadBarcode(object sender, Nedap.HandheldApi.DidReadBarcodeEventArgs e)
        // When device reads a code display in on the form.
        private void facade_OnReadBarcode(object sender, Nedap.HandheldApi.ReadBarcodeEventArgs e)
        {
            object[] objectArray = new object[2];
            objectArray[0] = e.BarcodeType.ToString();
            objectArray[1] = e.Barcode;

            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), objectArray);
            Program.Controller.Facade.BarcodeRead(Nedap.HandheldApi.BarcodeType.All, barcodeReadDuration);
        }
        #endregion

        #endregion

        #region Methods

        #region FormUpdateMethod(string typeValue, string barcode)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="typeValue">Type string value.</param>
        /// <param name="barcode">Barcode string value.</param>
        private void FormUpdateMethod(string typeValue, string barcode)
        {
            lblType.Text = labelTypeText;
            lblTypeValue.Text = typeValue;
            lblBarcode.Text = barcode;
        }
        #endregion

        #region ClearControls()
        /// <summary>
        /// Clear form controls
        /// </summary>
        private void ClearControls()
        {
            lblType.Text = string.Empty;
            lblTypeValue.Text = string.Empty;
            lblBarcode.Text = string.Empty;
        }
        #endregion

        #endregion
    }
}