﻿using System;
using System.Windows.Forms;

namespace RetailDemoApp
{
    public partial class frmBatteryLife : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void UpdateUIDelegate(int batteryLifePercentage);

        #endregion

        #region frmBatteryLife() - constructor
        public frmBatteryLife()
        {
            InitializeComponent();

            commonMenu.MenuItems[0].MenuItems.RemoveAt(7);
            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region btnRefresh_Click(object sender, EventArgs e)
        private void btnRefresh_Click(object sender, EventArgs e)
        {
            lblBattery.Text = String.Empty;
            Program.Controller.Session.HandheldGetBatteryPercentage();
        }
        #endregion

        #region frmBatteryLife_Activated(object sender, EventArgs e)
        private void frmBatteryLife_Activated(object sender, EventArgs e)
        {
            Program.Controller.Facade.OnReceiveBatteryPercentage += new EventHandler<Nedap.HandheldApi.ReceiveBatteryPercentageEventArgs>(facade_OnReceiveBatteryPercentage);
            Program.Controller.Session.HandheldGetBatteryPercentage();
        }
        #endregion

        #region facade_OnReceiveBatteryPercentage(object sender, Nedap.HandheldApi.ReceiveBatteryPercentageEventArgs e)
        // When device reads battery status display it on the form.
        private void facade_OnReceiveBatteryPercentage(object sender, Nedap.HandheldApi.ReceiveBatteryPercentageEventArgs e)
        {
            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), e.Percentage);
        }
        #endregion

        #region frmBatteryLife_Deactivate(object sender, EventArgs e)
        private void frmBatteryLife_Deactivate(object sender, EventArgs e)
        {
            Program.Controller.Facade.OnReceiveBatteryPercentage -= new EventHandler<Nedap.HandheldApi.ReceiveBatteryPercentageEventArgs>(facade_OnReceiveBatteryPercentage);
            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #endregion

        #region FormUpdateMethod(int batteryLifePercentage)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="batteryLifePercentage">Battery life percentage.</param>
        private void FormUpdateMethod(int batteryLifePercentage)
        {
            lblBattery.Text = String.Format(batteryLifePercentage.ToString() + " %");
            pbBatteryLife.Value = batteryLifePercentage;
        }
        #endregion
    }
}