﻿using System;
using System.Collections.ObjectModel;
using System.Windows.Forms;
using Nedap.HandheldApi;
using Nedap.EPC.TDT;

namespace RetailDemoApp
{
    public partial class frmReadSingle : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void UpdateUIDelegate(Collection<EpcObservation> epcObservations);

        #endregion

        #region frmReadSingle() - constructor
        public frmReadSingle()
        {
            InitializeComponent();

            commonMenu.MenuItems[0].MenuItems.RemoveAt(5);
            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region btnClear_Click(object sender, EventArgs e)
        private void btnClear_Click(object sender, EventArgs e)
        {
            lbReadSingle.Items.Clear();
            Program.Controller.Facade.RfidReadSingle();
        }
        #endregion

        #region frmReadSingle_Activated(object sender, EventArgs e)
        private void frmReadSingle_Activated(object sender, EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTags += new EventHandler<ReadRfidTagsEventArgs>(facade_OnReadRfidTags);
            Program.Controller.Facade.RfidReadSingle();
        }
        #endregion

        #region facade_OnReadRfidTags(object sender, DidReadRfidTagsEventArgs e)
        // When device reads single RFID display it on the form.
        private void facade_OnReadRfidTags(object sender, ReadRfidTagsEventArgs e)
        {
            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), e.EpcObservations);
        }
        #endregion

        #region frmReadSingle_Deactivate(object sender, EventArgs e)
        private void frmReadSingle_Deactivate(object sender, EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTags -= new EventHandler<ReadRfidTagsEventArgs>(facade_OnReadRfidTags);
            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #endregion

        #region FormUpdateMethod(Collection<EpcObservation> epcObservations)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="epcObservations">EpcObservation collection.</param>
        private void FormUpdateMethod(Collection<EpcObservation> epcObservations)
        {
            foreach (EpcObservation epc in epcObservations)
            {
                try
                {
                    lbReadSingle.Items.Add(epc.EpcCode.PureIdentityURI);
                }
                catch (TDTTranslationException)
                {
                    lbReadSingle.Items.Add(epc.EpcCode.HexString);
                }
            }

        }
        #endregion
    }
}