using System.Windows.Forms;
using Nedap.HandheldApi;
using Nedap.EPC.TDT;
using System.Collections.ObjectModel;

namespace RetailDemoApp
{
    public partial class frmRetailDemo : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void UpdateUIDelegate(Collection<EpcObservation> epcObservationCollection);
        private bool highVolume = false;

        #endregion

        #region frmRetailDemo() - constructor

        public frmRetailDemo(bool isHighVolume)
        {
            InitializeComponent();
            highVolume = isHighVolume;
            if (highVolume)
            {
                commonMenu.MenuItems[0].MenuItems.RemoveAt(1);
                lblInventory.Text = "High Volume Inventory";
            }
            else
            {
                commonMenu.MenuItems[0].MenuItems.RemoveAt(0);
            }
            this.Menu = commonMenu;
            lblNumberOfItems.Text = "0 ";
        }

        #endregion

        #region Events

        #region frmRetailDemo_Activated(object sender, System.EventArgs e)
        private void frmRetailDemo_Activated(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTagsFiltered += new System.EventHandler<ReadRfidTagsFilteredEventArgs>(facade_OnReadRFIDTagsFiltered);
            ResetInventory();

            if (!highVolume)
            {
                Program.Controller.Facade.RfidInventoryStart();
            }
            else
            {
                Program.Controller.Facade.RfidInventoryStart(true);
            }
        }

        #endregion

        #region facade_OnReadRFIDTagsFiltered(object sender, ReadRfidTagsFilteredEventArgs e)
        // When device reads RFID tags display results on the form.
        void facade_OnReadRFIDTagsFiltered(object sender, ReadRfidTagsFilteredEventArgs e)
        {
            this.BeginInvoke(new UpdateUIDelegate(FormUpdateMethod), e.EpcObservationCollection);
        }
        #endregion

        #region frmRetailDemo_Deactivate(object sender, System.EventArgs e)
        private void frmRetailDemo_Deactivate(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnReadRfidTagsFiltered -= new System.EventHandler<ReadRfidTagsFilteredEventArgs>(facade_OnReadRFIDTagsFiltered);

            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #region btnClear_Click(object sender, System.EventArgs e)
        private void btnClear_Click(object sender, System.EventArgs e)
        {
            ResetInventory();
        }
        #endregion

        private void ResetInventory()
        {
            if (lbInventoryItemsList.Items.Count > 0)
            {
                lbInventoryItemsList.Items.Clear();

            }
            lblNumberOfItems.Text = "0 ";
            Program.Controller.Facade.RfidInventoryReset();
        }

        #endregion

        #region Methods

        #region FormUpdateMethod(Collection<EpcObservation> epcObservationCollection)
        /// <summary>
        /// Updates form controls.
        /// </summary>
        /// <param name="epcObservation">EpcObservation.</param>
        private void FormUpdateMethod(Collection<EpcObservation> epcObservationCollection)
        {
            foreach (EpcObservation epcObservation in epcObservationCollection)
            {
                if (!lbInventoryItemsList.Items.Contains(epcObservation.EpcCode.HexString))
                {
                    lbInventoryItemsList.Items.Add(epcObservation.EpcCode.HexString);
                }
            }
            lblNumberOfItems.Text = lbInventoryItemsList.Items.Count.ToString();
        }
        #endregion

        #endregion
    }
}