﻿using System.Windows.Forms;
using Nedap.EPC.TDT;

namespace RetailDemoApp
{
    public partial class frmWriteEPC : Form
    {
        #region Properties

        private CommonMenu commonMenu = new CommonMenu();
        private delegate void MessageBoxDelegate(string error);
        private delegate void ValidateStringDelegate();

        #endregion

        #region frmWriteEPC() - constructor
        public frmWriteEPC()
        {
            InitializeComponent();

            commonMenu.MenuItems[0].MenuItems.RemoveAt(6);
            this.Menu = commonMenu;
        }
        #endregion

        #region Events

        #region frmWriteEPC_Activated(object sender, System.EventArgs e)
        private void frmWriteEPC_Activated(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnSucceedWriteEpc += new System.EventHandler<System.EventArgs>(facade_OnSucceedWriteEpc);
            Program.Controller.Facade.OnFailWriteEpcMultipleRead += new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcMultipleRead);
            Program.Controller.Facade.OnFailWriteEpcZeroRead += new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcZeroRead);
            Program.Controller.Facade.OnFailWriteEpcWriteFailed += new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcWriteFailed);
            Program.Controller.Facade.OnFailWriteEpcVerificationFailed += new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcVerificationFailed);

            tbEPCCode.Text = string.Empty;
            readyToWrite.Text = string.Empty;
        }
        #endregion

        #region frmWriteEPC_Deactivate(object sender, System.EventArgs e)
        private void frmWriteEPC_Deactivate(object sender, System.EventArgs e)
        {
            Program.Controller.Facade.OnSucceedWriteEpc -= new System.EventHandler<System.EventArgs>(facade_OnSucceedWriteEpc);
            Program.Controller.Facade.OnFailWriteEpcMultipleRead -= new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcMultipleRead);
            Program.Controller.Facade.OnFailWriteEpcZeroRead -= new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcZeroRead);
            Program.Controller.Facade.OnFailWriteEpcWriteFailed -= new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcWriteFailed);
            Program.Controller.Facade.OnFailWriteEpcVerificationFailed -= new System.EventHandler<System.EventArgs>(Facade_OnFailWriteEpcVerificationFailed);

            Program.Controller.Facade.HandheldStop();
        }
        #endregion

        #region Facade_OnFailWriteEpcVerificationFailed(object sender, System.EventArgs e)
        void Facade_OnFailWriteEpcVerificationFailed(object sender, System.EventArgs e)
        {
            this.BeginInvoke(new MessageBoxDelegate(ShowMessageBoxAsync), "fail");
        }
        #endregion

        #region Facade_OnFailWriteEpcWriteFailed(object sender, System.EventArgs e)
        void Facade_OnFailWriteEpcWriteFailed(object sender, System.EventArgs e)
        {
            this.BeginInvoke(new MessageBoxDelegate(ShowMessageBoxAsync), "fail");
        }
        #endregion

        #region Facade_OnFailWriteEpcZeroRead(object sender, System.EventArgs e)
        void Facade_OnFailWriteEpcZeroRead(object sender, System.EventArgs e)
        {
            this.BeginInvoke(new MessageBoxDelegate(ShowMessageBoxAsync), "fail");
        }
        #endregion

        #region Facade_OnFailWriteEpcMultipleRead(object sender, System.EventArgs e)
        void Facade_OnFailWriteEpcMultipleRead(object sender, System.EventArgs e)
        {
            this.BeginInvoke(new MessageBoxDelegate(ShowMessageBoxAsync), "fail");
        }
        #endregion

        #region facade_OnSucceedWriteEpc(object sender, System.EventArgs e)
        private void facade_OnSucceedWriteEpc(object sender, System.EventArgs e)
        {
            this.BeginInvoke(new MessageBoxDelegate(ShowMessageBoxAsync), "done");
        }
        #endregion

        #region checkValue_Click
        private void checkValue_Click(object sender, System.EventArgs e)
        {
            CheckForWrite();
        }
        #endregion

        #endregion

        #region Methods

        #region IsValidHexString()
        /// <summary>
        /// Validates hex string for write.
        /// </summary>
        private bool IsValidHexString()
        {
            if (tbEPCCode.Text.Length == 0)
            {
                ShowMessageBoxAsync("zero");
                return false;
            }

            if (tbEPCCode.Text.Length % 4 > 0)
            {
                ShowMessageBoxAsync("multi");
                return false;
            }

            if (!System.Text.RegularExpressions.Regex.IsMatch(tbEPCCode.Text, "^[0-9a-fA-F]+$"))
            {
                ShowMessageBoxAsync("hex");
                return false;
            }
            return true;
        }
        #endregion

        #region CheckForWrite
        private void CheckForWrite()
        {
            if (IsValidHexString())
            {
                readyToWrite.Text = "Value is in proper format.";
                Invalidate();
                try
                {
                    Program.Controller.Facade.RfidWriteEpc(new Nedap.EPC.TDT.EpcCode(tbEPCCode.Text));
                }
                catch (TDTTranslationException)
                {
                    this.BeginInvoke(new MessageBoxDelegate(ShowMessageBoxAsync), "invalid");
                }
            }
        }
        #endregion

        #region ShowMessageBoxAsync(string error)
        private void ShowMessageBoxAsync(string error)
        {
            if (error == "zero")
                MessageBox.Show("Input cannot have a zero length.", "Input error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            else if (error == "multi")
                MessageBox.Show("Input length should be a multiple of four.", "Input error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            else if (error == "hex")
                MessageBox.Show("Input is not hexadecimal.", "Input error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
            else if (error == "done")
                MessageBox.Show("Writing tag succeeded.", "Write succeeded", MessageBoxButtons.OK, MessageBoxIcon.None, MessageBoxDefaultButton.Button1);
            else if (error == "fail")
                MessageBox.Show("Writing tag failed.", "Write failed", MessageBoxButtons.OK, MessageBoxIcon.Hand, MessageBoxDefaultButton.Button1);
            else if (error == "invalid")
                MessageBox.Show("Input value is not in valid format.", "Input error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
        }
        #endregion

        #endregion
    }
}