#!/bin/sh

PROJECTS="EPCGlobalTDS NedapHandheldAPI"
CONFIGURATIONS="Debug Release"
SDKS="iphoneos5.0 iphonesimulator5.0"

mkdir "${WORKSPACE}/iOS/_build"

mkdir "${WORKSPACE}/IDHandAPI"
mkdir "${WORKSPACE}/IDHandAPI/Firmware"
mkdir "${WORKSPACE}/IDHandAPI/iOS"
mkdir "${WORKSPACE}/IDHandAPI/WindowsCE"

mkdir "${WORKSPACE}/IDHandAPI/iOS/includes"
mkdir "${WORKSPACE}/IDHandAPI/iOS/NedapHandheldAPIDemo"

mkdir "${WORKSPACE}/IDHandAPI/iOS/includes/RegexKitLite"
mkdir "${WORKSPACE}/IDHandAPI/iOS/includes/TDTXMLfiles"

mkdir "${WORKSPACE}/IDHandAPI/WindowsCE/API"
mkdir "${WORKSPACE}/IDHandAPI/WindowsCE/Example"

mv "${WORKSPACE}/firmware.h" "${WORKSPACE}/iOS/NedapHandheldAPI/firmware.h"

for project in $PROJECTS; do
	mkdir "${WORKSPACE}/IDHandAPI/iOS/$project"
	cd "${WORKSPACE}/iOS/$project"
	for config in $CONFIGURATIONS; do
		lipo_command="lipo -create -output ${WORKSPACE}/iOS/_build/lib$project-$config.a "

		for sdk in $SDKS; do
			mkdir "${WORKSPACE}/iOS/_build/$sdk"
			xcodebuild -configuration $config -sdk $sdk CONFIGURATION_BUILD_DIR="${WORKSPACE}/iOS/_build/$sdk"
			lipo_command="$lipo_command${WORKSPACE}/iOS/_build/$sdk/lib$project.a "
            if test -f ${WORKSPACE}/iOS/_build/$sdk/lib$project.a; then
                echo "Building $project in sdk $sdk succeeded."
            else
                echo "Building $project in sdk $sdk failed."
                exit -1
            fi
		done

		echo $lipo_command
		eval "$lipo_command"
	done
	cp -v ${WORKSPACE}/iOS/_build/lib$project-Debug.a ${WORKSPACE}/IDHandAPI/iOS/$project
    mv -v ${WORKSPACE}/iOS/_build/lib$project-Release.a ${WORKSPACE}/iOS/_build/lib$project.a
    cp -v ${WORKSPACE}/iOS/_build/lib$project.a ${WORKSPACE}/IDHandAPI/iOS/$project/
    cp -v ${WORKSPACE}/iOS/_build/lib$project.a ${WORKSPACE}/iOS/NedapHandheldAPIDemo/
done

for sdk in $SDKS; do
    cp -v ${WORKSPACE}/iOS/_build/$sdk/N*.h ${WORKSPACE}/IDHandAPI/iOS/NedapHandheldAPI/
    cp -v ${WORKSPACE}/iOS/_build/$sdk/E*.h ${WORKSPACE}/IDHandAPI/iOS/EPCGlobalTDS/
    cp -v ${WORKSPACE}/iOS/_build/$sdk/*.h ${WORKSPACE}/iOS/NedapHandheldAPIDemo/
done

#cd ${WORKSPACE}/iOS/NedapHandheldAPIDemo
#xcodebuild
#if test -f ${WORKSPACE}/iOS/NedapHandheldAPIDemo/build/Release-iphoneos/NedapHandheldAPIDemo.app/NedapHandheldAPIDemo; #then
#    echo "Building NedapHandheldAPIDemo succeeded."
#    rm -v -rf ${WORKSPACE}/iOS/NedapHandheldAPIDemo/build
#else
#    echo "Building NedapHandheldAPIDemo failed."
#    exit -1
#fi

cp -v ${WORKSPACE}/iOS/EPCGlobalTDS/*.xml ${WORKSPACE}/IDHandAPI/iOS/includes/TDTXMLfiles/
cp -v ${WORKSPACE}/iOS/EPCGlobalTDS/RegexKitLite.* ${WORKSPACE}/IDHandAPI/iOS/includes/RegexKitLite/

cp -v ${WORKSPACE}/Windows\ CE/Versions/*.zip ${WORKSPACE}/IDHandAPI/WindowsCE/API/
cp -v -r ${WORKSPACE}/Windows\ CE/Windows\ CE/RetailDemoApp/* ${WORKSPACE}/IDHandAPI/WindowsCE/Example/

mv -v ${WORKSPACE}/firmware*.zip ${WORKSPACE}/IDHandAPI/Firmware/
cd ${WORKSPACE}/IDHandAPI/Firmware
unzip firmware*.zip
rm firmware*.zip

cp -v -r ${WORKSPACE}/iOS/NedapHandheldAPIDemo/* ${WORKSPACE}/IDHandAPI/iOS/NedapHandheldAPIDemo/

cd ${WORKSPACE}
cp -v *.pdf IDHandAPI/
zip -y -r IDHandAPI.zip IDHandAPI/

if [ "${GIT_BRANCH}" == "develop" ]; then
    mv -v IDHandAPI.zip IDHandAPI-develop-${BUILD_NUMBER}.zip
else
    tag=`git for-each-ref refs/tags --sort=-authordate --format='%(refname:short)' --count=1`
    mv -v IDHandAPI.zip IDHandAPI-$tag.zip
fi

