//
//  EPCConvert.m
//  EPCGlobalTDS
//
//  Created by Danny Haak on 12-04-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "EPCConvert.h"

@implementation EPCConvert

+ (NSString *)convertToHexStringFromBinaryString:(NSString *)binary
{
	// if length is not multiple of four, pad zeros
	NSInteger length = [binary length];
	NSInteger mod = length % 4;
	NSInteger pad = 0;
	if (mod > 0)
	{
		pad = 4 -  mod;
	}
	NSString *binaryPadded = [[@"" stringByPaddingToLength: pad withString: @"0" startingAtIndex:0] stringByAppendingString:binary];
	NSInteger hexChars = [binaryPadded length] / 4;
	NSMutableString *hexString = [[[NSMutableString alloc] init] autorelease];
	for (NSInteger i = 0; i < hexChars; i++)
	{
		NSRange range = {
			i * 4,
			4
		};
		if ([binaryPadded compare:@"0000" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"0"];
		if ([binaryPadded compare:@"0001" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"1"];
		if ([binaryPadded compare:@"0010" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"2"];
		if ([binaryPadded compare:@"0011" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"3"];
		if ([binaryPadded compare:@"0100" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"4"];
		if ([binaryPadded compare:@"0101" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"5"];
		if ([binaryPadded compare:@"0110" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"6"];
		if ([binaryPadded compare:@"0111" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"7"];
		if ([binaryPadded compare:@"1000" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"8"];
		if ([binaryPadded compare:@"1001" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"9"];
		if ([binaryPadded compare:@"1010" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"A"];
		if ([binaryPadded compare:@"1011" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"B"];
		if ([binaryPadded compare:@"1100" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"C"];
		if ([binaryPadded compare:@"1101" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"D"];
		if ([binaryPadded compare:@"1110" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"E"];
		if ([binaryPadded compare:@"1111" options:0 range:range] == NSOrderedSame)
			[hexString appendString:@"F"];
	}
	
	return hexString;
}

+ (NSString *)convertToBinaryStringFromHexString:(NSString *)hex
{
	NSMutableString *binary = [[[NSMutableString alloc] initWithCapacity:256] autorelease];
	for (int i = 0; i < [hex length]; i++)
	{
		if ([hex characterAtIndex:i] == '0')
			[binary appendString:@"0000"];
		if ([hex characterAtIndex:i] == '1')
			[binary appendString:@"0001"];
		if ([hex characterAtIndex:i] == '2')
			[binary appendString:@"0010"];
		if ([hex characterAtIndex:i] == '3')
			[binary appendString:@"0011"];
		if ([hex characterAtIndex:i] == '4')
			[binary appendString:@"0100"];
		if ([hex characterAtIndex:i] == '5')
			[binary appendString:@"0101"];
		if ([hex characterAtIndex:i] == '6')
			[binary appendString:@"0110"];
		if ([hex characterAtIndex:i] == '7')
			[binary appendString:@"0111"];
		if ([hex characterAtIndex:i] == '8')
			[binary appendString:@"1000"];
		if ([hex characterAtIndex:i] == '9')
			[binary appendString:@"1001"];
		if ([hex characterAtIndex:i] == 'A')
			[binary appendString:@"1010"];
		if ([hex characterAtIndex:i] == 'B')
			[binary appendString:@"1011"];
		if ([hex characterAtIndex:i] == 'C')
			[binary appendString:@"1100"];
		if ([hex characterAtIndex:i] == 'D')
			[binary appendString:@"1101"];
		if ([hex characterAtIndex:i] == 'E')
			[binary appendString:@"1110"];
		if ([hex characterAtIndex:i] == 'F')
			[binary appendString:@"1111"];	
	}
	return binary;
}

+ (NSString *)convertToBinaryStringFromData:(NSData *)data
{
	NSMutableString *binary = [[[NSMutableString alloc] init] autorelease];

	const unsigned char* bytes = [data bytes];
	NSUInteger length = [data length];
	
	for (int i = 0; i < length; i++)
	{		
		for (NSUInteger z = 128; z > 0; z >>= 1)
		{	
			if ((bytes[i] & z) == z)
			{
				NSString *one = [[NSString alloc] initWithString:@"1"];
                [binary appendString:one];
                [one release];
			}
			else 
			{
                NSString *zero = [[NSString alloc] initWithString:@"0"];
				[binary appendString:zero];
                [zero release];
			}
		}
	}
	return [[binary copy] autorelease];
}

+ (NSData *)convertToDataFromHexString:(NSString *)string
{
	char dataArray[255];
	
	char byteChars[3] = {'\0','\0','\0'};
	for (int i = 0; i < [string length]; i += 2)
	{
		byteChars[0] = [string characterAtIndex:i];
		byteChars[1] = [string characterAtIndex:i+1];	
		dataArray[i / 2] = strtol(byteChars, NULL, 16);
	}
	
	return [NSData dataWithBytes:dataArray length:[string length]/2];
}

+ (NSString *)convertToHexStringFromData:(NSData *)data
{
	NSMutableString *hexString = [[[NSMutableString alloc] initWithCapacity:256] autorelease];
	const unsigned char* bytes = [data bytes];
	NSUInteger length = [data length];
	
	for (int i = 0; i < length; i++)
	{		
		[hexString appendFormat:@"%02X", bytes[i]];		
	}
	return hexString;
}
	
@end
