//
//  NHPresentation.m
//  NedapHandheldAPI
//
//  Created by Danny Haak on 04-07-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

/*
 * STUFF TO IMPLEMENT
 */

//+ (void)rfidReadUserMemoryFromTag:(EPCCode *)code addressInWords:(NSUInteger)address lengthInWords:(NSUInteger)length;

//+ (void)rfidWriteEPC:(EPCCode *)newEPCCode select:(NHSelect *)select password:(NHPassword *)password;

//+ (void)rfidWriteEPC:(EPCCode *)newEPCCode select:(NHSelect *)select password:(NHPassword *)password andLock:(BOOL)lock withPassword:(NHPassword *)lockPassword;
//+ (void)rfidWriteUserMemoryWithData:(NSData *)data addressInWords:(NSUInteger)address toTag:(EPCCode *)tag password:(NHPassword *)password;
//+ (void)rfidSetAccessPasswordToTag:(EPCCode *)tag accessPassword:(NHPassword *)access newAccessPassword:(NHPassword *)newPassword;
//+ (void)rfidSetKillPasswordToTag:(EPCCode *)tag accessPassword:(NHPassword *)access lockPassword:(NHPassword *)lockPassword;
//+ (void)rfidSetBothPasswordsToTag:(EPCCode *)tag accessPassword:(NHPassword *)access newAccessPassword:(NHPassword *)newPassword lockPassword:(NHPassword *)lockPassword;

//+ (void)rfidLockTag:(EPCCode *)tag lock:(NSDictionary *)lock;

//+ (void)rfidKillTag:(EPCCode *)tag;

#import "NHPresentation.h"
#import "NHPresentation_sm.h"

#import "NHDefines.h"
#import "NHSession.h"
#import "NHSelect.h"
#import "NHPassword.h"
#import "NHEPCObservation.h"

#import "EPCGlobalTDS.h"

#import "firmware.h"
#import "NedapHandheldAPI.h"
#import <CommonCrypto/CommonDigest.h>


//#import "FlurryAPI.h"

#pragma mark -

@interface NHPresentation()

+ (void)localRfidSetBuffer:(NSString *)bufferName;

@end

@implementation NHPresentation

@synthesize rfidInventoryHighVolume, rfidInventoryPowerLevel, rfidInventorySelect, rfidInventoryBuffer;
@synthesize rfidMonitorPowerLevel, rfidMonitorSelect;
@synthesize barcodeTypes, barcodeReadDuration, barcodeValidationBlock;

static dispatch_queue_t q_tagList;
static dispatch_queue_t q_beeper;

static NHPresentationContext *_fsm;

static NSMutableDictionary *tagListInventoryDictionary;
static NSMutableSet *tagListInventory;
static NSMutableSet *tagListSingle;
static NSMutableSet *tagListSingleFiltered;
static NSMutableIndexSet *tagSet;
static NSMutableIndexSet *tagInventorySet;

static NSMutableDictionary *_timerDict;

static BOOL tuneTimeout;
static BOOL beeperEnabled;
static BOOL vibrationEnabled;

static NSString *currentTimedEvent;

static NHRFIDRegion rfidCurrentRegion = 0;

// variables that glue NHPresentation and NHSession together
static NSUInteger rfidOnTime;
static NSUInteger rfidOffTime;

static NHSelect *rfidCurrentSelect;
static NHPassword *rfidCurrentPassword;
static EPCCode *rfidEPCToWrite;
static NSUInteger rfidReadPower;

static NHRFIDMemoryBank rfidReadDataMemoryBank;
static NSUInteger rfidReadDataAddressInWords;
static NSUInteger rfidReadDataLengthInWords;

//static NSArray *barcodeCurrentTypes;
//static NSUInteger barcodeCurrentDuration;

static NSUInteger currentSession;
static BOOL changeSession;

static NSMutableData *newFirmware;
static NSUInteger newFirmwarePointer;

#define DEFAULT_BUFFER @"default"


+ (NHPresentation *)sharedPresentation {
	static NHPresentation *sharedPresentation;
	
	@synchronized(self)	
	{
		if (!sharedPresentation)
			sharedPresentation = [[NHPresentation alloc] init];
		
		return sharedPresentation;
	}
	
	return nil;
}

+ (void)initialize {
	//[FlurryAPI setAppVersion:@"0.9.2"];
	//[FlurryAPI startSession:@"HF7C8NPH62W7Q1XQMBFP"];
	//[FlurryAPI setUserID:[[UIDevice currentDevice] uniqueIdentifier]];
    
	[NHSession self];
	
	tuneTimeout = YES;
	beeperEnabled = YES;
	vibrationEnabled = YES;
	
	q_tagList = dispatch_queue_create("com.nedap.retail.handheld.taglist", NULL);
	q_beeper = dispatch_queue_create("com.nedap.retail.handheld.beeper", NULL);

	tagListInventoryDictionary = [[NSMutableDictionary alloc] init];
	tagListInventory = [[NSMutableSet alloc] initWithCapacity:10000];
	[tagListInventoryDictionary setObject:tagListInventory forKey:@"default"];
	[tagListInventory release];
    tagInventorySet = [[NSMutableIndexSet alloc] init];
    
	// register callback
	[NHSession setDelegate:self.sharedPresentation];
    
	// initialize values
	rfidOffTime = 0;
	rfidOnTime = 500;
	currentSession = 2;
	
	// initalize state machine
	_timerDict = [[NSMutableDictionary alloc] init];
	_fsm = [[NHPresentationContext alloc] initWithOwner:(NHPresentation *)self];
	
	#ifdef DEBUG
		[_fsm setDebugFlag:YES];
	#endif
	
    [_fsm enterStartState];
	
	if ([NHSession handheldIsConnected])
	{
		//[FlurryAPI logEvent:@"handheldConnectedOnBoot"];
		[_fsm connected];
	}
	else 
	{
		//[FlurryAPI logEvent:@"handheldNotConnectedOnBoot"];
	}
}

#pragma mark -
#pragma mark Local calls

+ (void)localRfidSetBuffer:(NSString *)bufferName
{
	if ([tagListInventoryDictionary valueForKey:bufferName] == nil)
	{
		NSMutableSet *newTagListInventory = [[NSMutableSet alloc] initWithCapacity:1000];
		[tagListInventoryDictionary setObject:newTagListInventory forKey:bufferName];
		tagListInventory = newTagListInventory;
		[newTagListInventory release];
	}
	else 
	{
		tagListInventory = [tagListInventoryDictionary valueForKey:bufferName];
	}
}

#pragma mark -
#pragma mark API calls

#pragma mark general

+ (void) notify:(NSString *)notificationName object:(id)theObject userInfo:(id)theUserInfo {
    @try {
        [[NSNotificationCenter defaultCenter] postNotificationName:notificationName object:theObject userInfo:theUserInfo];
    }
    @catch (NSException *e) {
        NSLog(@"[ERROR] could not send notification %@: %@",notificationName,e);
    }
}

+ (BOOL)handheldIsConnected
{
	return [NHSession handheldIsConnected];
}

+ (void)handheldDisconnect
{
	[NHSession handheldDisconnect];
}

+ (void)handheldLogEnabled:(BOOL)enabled
{
	//[FlurryAPI setEventLoggingEnabled:enabled];
}

+ (void)handheldGetBatteryPercentage
{
	[NHSession handheldGetBatteryPercentage];
}

+ (void)handheldStop
{
	[_fsm handheldStop];
}

+ (NSDictionary *)handheldGetInformation
{
	return [NHSession handheldGetInformation];
}

+ (NSString *)handheldUpdateFirmwareAvailableVersion
{
    return [NSString stringWithFormat:@"%i.%i.%i",  SW_MAJOR, SW_MINOR, SW_BUGFIX];   
}

+ (BOOL)handheldUpdateFirmwareAvailable
{
    if ([NHSession handheldIsConnected] == NO)
        return NO;
    
    NSDictionary *dictionary = [NHSession handheldGetInformation];
    NSString *hardwareVersionString = [dictionary valueForKey:@"hardwareRevision"];
    
    NSUInteger hardwareRevisionInt = 0;
    NSArray *hardwareVersionArray = [hardwareVersionString componentsSeparatedByString:@"."];
    for (NSString *line in hardwareVersionArray)
    {
        hardwareRevisionInt *= 256;
        hardwareRevisionInt += [line intValue];
    }
    
    NSUInteger hardwareMinimalInt = 0;
    NSString *hardwareMinimalString = @"1.2.0";
    NSArray *hardwareMinimalArray = [hardwareMinimalString componentsSeparatedByString:@"."];
    for (NSString *line in hardwareMinimalArray)
    {
        hardwareMinimalInt *= 256;
        hardwareMinimalInt += [line intValue];
    }

    if (hardwareRevisionInt >= hardwareMinimalInt)
    {
        return YES;
    }
    else
    {
        return NO;
    }
}

+ (void)handheldUpdateFirmware
{
    NSArray *firmwareArray = [NSArray arrayWithObjects:FIRMWARE];
    
    // load firmware file
    newFirmware = [[NSMutableData alloc] init];
    newFirmwarePointer = 0;
    
    for (NSString *firmwareString in firmwareArray)
    {
        IHexRecord ihex;
        char inString[255];
        [firmwareString getCString:inString maxLength:255 encoding:NSASCIIStringEncoding];
        if (Read_IHexRecord_char(&ihex, inString) == IHEX_OK)
        {
            if (ihex.type == IHEX_TYPE_00)
                [newFirmware appendBytes:ihex.data length:ihex.dataLen];
        }
    }
    
    [_fsm handheldBootloader];
}

#pragma mark user interaction

+ (void)uiBeeperEnabled:(BOOL)enabled
{
	beeperEnabled = enabled;
}

+ (void)uiVibrationEnabled:(BOOL)enabled
{
	vibrationEnabled = enabled;
}

+ (void)uiPlayTune:(NHUITune)tune
{
	[NHSession uiPlayTune:tune];
}

+ (void)uiVibrate
{
	[NHSession uiVibrate];
}

+ (void)uiVibrateTune:(NHUITune)tune
{
	[NHSession uiVibrateTune:tune];
}

#pragma mark rfid

+ (void)rfidSetRegion:(NHRFIDRegion)region
{
    rfidCurrentRegion = region;
    [NHSession rfidSetRegion:region];
}

+ (void)rfidInventoryStart;
{
	[self rfidInventoryStartWithSelect:nil];
}

+ (void)rfidInventoryStartWithSelect:(NHSelect *)select; 
{
	[self rfidInventoryStartInBuffer:DEFAULT_BUFFER withSelect:select withSession:NHRFIDSession1];
}

+ (void)rfidInventoryStartInBuffer:(NSString *)bufferName
{
	[self rfidInventoryStartInBuffer:bufferName withSelect:nil withSession:NHRFIDSession1];
}

+ (void)rfidInventoryStartInBuffer:(NSString *)bufferName withSelect:(NHSelect *)select
{
    [self rfidInventoryStartInBuffer:bufferName withSelect:select withSession:NHRFIDSession1];
}

+ (void)rfidInventoryStartWithSession:(NHRFIDSession)session
{
    [self rfidInventoryStartWithSelect:nil withSession:session];
}

+ (void)rfidInventoryStartWithSelect:(NHSelect *)select withSession:(NHRFIDSession)session 
{
	[self rfidInventoryStartInBuffer:DEFAULT_BUFFER withSelect:select withSession:session];
}

+ (void)rfidInventoryStartInBuffer:(NSString *)bufferName withSession:(NHRFIDSession)session 
{
	[self rfidInventoryStartInBuffer:bufferName withSelect:nil withSession:session];
}

+ (void)rfidInventoryStartInBuffer:(NSString*)bufferName withSelect:(NHSelect *)select withSession:(NHRFIDSession)session
{
	[self localRfidSetBuffer:bufferName];
	
	changeSession = NO;
	
	rfidCurrentSelect = select;
    
    switch (session) {
        default:
        case NHRFIDSession1:
            // [_fsm rfidInventoryStart];
            [_fsm rfidInventoryStartSession1];
            break;
        case NHRFIDSession0:
            [_fsm rfidInventoryStartSession0];
            break;
        case NHRFIDSession2:
            [_fsm rfidInventoryStartSession2];
            break;
        case NHRFIDSession3:
            [_fsm rfidInventoryStartSession3];
            break;
    }
}

+ (void)rfidInventoryHighVolumeStart
{
	[self rfidInventoryHighVolumeStartWithSelect:nil];
}

+ (void)rfidInventoryHighVolumeStartWithSelect:(NHSelect *)select
{
	[self rfidInventoryHighVolumeStartInBuffer:DEFAULT_BUFFER withSelect:select];
}

+ (void)rfidInventoryHighVolumeStartInBuffer:(NSString *)bufferName
{
	[self rfidInventoryHighVolumeStartInBuffer:bufferName withSelect:nil];
}

+ (void)rfidInventoryHighVolumeStartInBuffer:(NSString *)bufferName withSelect:(NHSelect *)select
{
	[self localRfidSetBuffer:bufferName];
	
	changeSession = NO;
	
	rfidCurrentSelect = select;
	[_fsm rfidInventoryHighVolumeStart];
}

+ (void)rfidInventoryLowPowerStart
{
	[self rfidInventoryLowPowerStartInBuffer:DEFAULT_BUFFER];
}

+ (void)rfidInventoryLowPowerStartInBuffer:(NSString *)bufferName
{
	[self localRfidSetBuffer:bufferName];
	
	changeSession = NO;
	
	rfidCurrentSelect = nil;
	[_fsm rfidInventoryLowPowerStart];
}

+ (void)rfidInventoryReset
{
	[tagListInventory removeAllObjects];
    [tagInventorySet removeAllIndexes];

	changeSession = YES;
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationRFIDDidResetInventoryNotification" object:self userInfo:nil];
	});
}

+ (void)rfidInventoryResetBuffer:(NSString *)bufferName
{
	NSMutableSet *resetTagListInventory = [tagListInventoryDictionary valueForKey:bufferName];
	
	if (resetTagListInventory != nil)
		[resetTagListInventory removeAllObjects];
	
	changeSession = YES;
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationRFIDDidResetInventoryNotification" object:self userInfo:nil];
	});
}

+ (void)addEPC:(EPCCode*)epc {
    [tagListInventory addObject:epc];
    [tagInventorySet addIndex:[epc hash]];
}

+ (void)rfidInventoryBufferFill:(NSSet*)dataSet {
    [self rfidInventoryBufferFill:dataSet forBuffer:DEFAULT_BUFFER];
}

+ (void)rfidInventoryBufferFill:(NSSet*)dataSet forBuffer:(NSString *)bufferName {
    [self localRfidSetBuffer:bufferName];

    [dataSet enumerateObjectsUsingBlock:^(id binaryData, BOOL *stop){
        @autoreleasepool {
            EPCCode *epcCode = [EPCCode codeWithBinaryData:binaryData];
            [tagListInventory addObject:epcCode];
            [tagInventorySet addIndex:[epcCode hash]];
        }
    }];
    
    NSLog(@"RFID buffer contains %u hashes",[tagInventorySet count]);
    NSLog(@"RFID buffer contains %u EPCs",[tagListInventory count]);
}

+ (NSSet *)rfidInventoryBuffer {
    return [self rfidInventoryBufferWithName:DEFAULT_BUFFER];
}

+ (NSSet *)rfidInventoryBufferWithName:(NSString*)bufferName {
    [self localRfidSetBuffer:bufferName];
//    return tagInventorySet;
    return tagListInventory;
//    return [[NSIndexSet indexSet] initWithIndexSet:tagInventorySet];
}


+ (void)rfidInventoryPause
{
	[_fsm rfidInventoryPause];
}

+ (void)rfidInventoryContinue
{
	[_fsm rfidInventoryContinue];
}

+ (void)rfidMonitorStart
{
	[self rfidMonitorStartWithSelect:nil];
}

+ (void)rfidMonitorStartWithSelect:(NHSelect *)select
{
	[self localRfidSetBuffer:@"monitor"];
	
	[select retain];
	
	if (rfidCurrentSelect != nil)
		[rfidCurrentSelect release];
	
	rfidCurrentSelect = select;
	
	[_fsm rfidMonitorStart];
}

+ (void)rfidMonitorPause
{
	[_fsm rfidMonitorPause];	
}

+ (void)rfidMonitorContinue
{
	[_fsm rfidMonitorContinue];
}

+ (void)rfidReadSingle
{
	[self rfidReadSingleWithSelect:nil];
}

+ (void)rfidReadSingleWithSelect:(NHSelect *)select
{
	[self localRfidSetBuffer:@"operation"];
	
	[select retain];
	
	if (rfidCurrentSelect != nil)
		[rfidCurrentSelect release];
	
	rfidCurrentSelect = select;
		
	[_fsm rfidSingleReadStart];	
}

+ (void)rfidReadUserMemoryFromTag:(EPCCode *)code addressInWords:(NSUInteger)address lengthInWords:(NSUInteger)length
{
	[self localRfidSetBuffer:@"operation"];
}

+ (void)rfidReadDetailedTagInformation
{
	[self rfidReadDetailedTagInformationFromTag:nil];
}

+ (void)rfidReadDetailedTagInformationFromTag:(EPCCode *)code
{
	[self localRfidSetBuffer:@"operation"];
	
	[code retain];
	
	if (rfidCurrentSelect != nil)
		[rfidCurrentSelect release];
	
	if (code != nil)
		rfidCurrentSelect = [[NHSelect alloc] initWithEPCCode:code];
	else
		rfidCurrentSelect = nil;	

	if (rfidCurrentPassword != nil)
	{
		[rfidCurrentPassword release];
		rfidCurrentPassword = nil;
	}
	
	rfidReadDataMemoryBank = NHRFIDMemoryBankTID;
	rfidReadDataAddressInWords = 0;
	rfidReadDataLengthInWords = 2;
	
	[_fsm rfidReadDetailedTagInformationStart];
}

+ (void)rfidWriteEPC:(EPCCode *)newEPCCode
{
	[newEPCCode retain];
	
	if (rfidEPCToWrite != nil)
		[rfidEPCToWrite release];
	
	rfidEPCToWrite = newEPCCode;
	rfidCurrentSelect = nil;
	rfidCurrentPassword = nil;
	
	[_fsm rfidWriteEPCStart];
}

/*
+ (void)rfidWriteEPC:(EPCCode *)newEPCCode select:(NHSelect *)select password:(NHPassword *)password
{
	[self localRfidSetBuffer:@"operation"];
	
	[newEPCCode retain];
	[select retain];
	[password retain];
	
	if (rfidEPCToWrite != nil)
		[rfidEPCToWrite release];
	
	if (select != nil)
		[rfidCurrentSelect release];
	
	if (password != nil)
		[rfidCurrentPassword release];
	
	rfidEPCToWrite = newEPCCode;
	rfidCurrentSelect = select;
	rfidCurrentPassword = password;

	[self.sharedPresentation._fsm rfidWriteEPCStart];
}
*/

#pragma mark barcode

+ (BOOL)barcodeHasReader
{
	return [NHSession barcodeHasReader];
}

+ (void)barcodeStartReading
{
    [NHSession barcodeSetStatusForType:NHBarcodeTypeAll enabled:NO];
	
	for (NSNumber *barcodeType in [NHPresentation sharedPresentation].barcodeTypes)
	{	
		[NHSession barcodeSetStatusForType:[barcodeType unsignedIntValue] enabled:YES];
	}
    
    if ([NHSession barcodeHasReader])
		[_fsm barcodeReadStart];
}

+ (void)barcodeResetParameters
{
    [NHPresentation sharedPresentation].barcodeTypes = nil;
    [NHPresentation sharedPresentation].barcodeReadDuration = 3;
    [NHPresentation sharedPresentation].barcodeValidationBlock = nil;
}

// deprecated
+ (void)barcodeReadType:(NHBarcodeType)barcodeType duringSeconds:(NSUInteger)duration
{
    [self barcodeReadTypes:[NSArray arrayWithObjects:[NSNumber numberWithUnsignedInt:barcodeType], nil] duringSeconds:duration];
}

// deprecated
+ (void)barcodeReadTypes:(NSArray *)barcodeTypes duringSeconds:(NSUInteger)duration
{
    [NHPresentation sharedPresentation].barcodeTypes = barcodeTypes;
    [NHPresentation sharedPresentation].barcodeReadDuration = duration;
    [NHPresentation sharedPresentation].barcodeValidationBlock = nil;
    
	[self barcodeStartReading];
}

#pragma mark nfc

+ (void)nfcReadStart
{
	[_fsm nfcReadStart];
}

#pragma mark -
#pragma mark Actions

#pragma mark general

+ (void)actionHandheldConfigure
{
    if (rfidCurrentRegion != 0)
        [NHSession rfidSetRegion:rfidCurrentRegion];
}

+ (void)actionHandheldConnected
{
}

+ (void)actionHandheldDisconnected
{
}

+ (void)actionHandheldBootloader
{
    [NHSession handheldBootloader];
}

+ (void)actionHandheldBootloaderNextPage
{
    NSUInteger newFirmwareLength = [newFirmware length];
    NSUInteger rangeLength = 64;
    if ((newFirmwarePointer + rangeLength) > newFirmwareLength)
        rangeLength = newFirmwareLength - newFirmwarePointer;
    NSRange pageRange = NSMakeRange(newFirmwarePointer, rangeLength);
    [NHSession handheldBootloaderSendPage:[newFirmware subdataWithRange:pageRange] address:newFirmwarePointer];
}

+ (void)actionHandheldBootloaderCRC
{
    const unsigned char* buffer = [newFirmware bytes];
    
    NSUInteger crc = 0;
    for (NSUInteger i = 0; i < [newFirmware length]; i++)
        crc += buffer[i];
    
    [NHSession handheldBootloaderFinish:crc];
}

#pragma mark user interaction

+ (void)actionUILedSetBlinking:(BOOL)blinking
{
	[NHSession uiSetStatusLedBlinking:blinking];
}

+ (void)actionUIVibrateTuneShort
{
    if (vibrationEnabled)
        [NHSession uiVibrateTune:NHUITuneShort];
}

+ (void)actionUIVibrateTune:(NHUITune)vibrateTune
{
    if (vibrationEnabled)
        [NHSession uiVibrateTune:vibrateTune];
}

+ (void)actionUITuneShort
{
    if (beeperEnabled)
        [NHSession uiPlayTune:NHUITuneShort];
		
    if (vibrationEnabled)
        [NHSession uiVibrateTune:NHUITuneShort];
}

+ (void)actionUITune:(NHUITune)beepTune
{
	if (beeperEnabled)
		[NHSession uiPlayTune:beepTune];
	
	if (vibrationEnabled)
		[NHSession uiVibrateTune:beepTune];
}

#pragma mark rfid

+ (void)actionRFIDSetMValue:(NHRFIDMValue)theMValue
{
	[NHSession rfidSetMValue:theMValue];
}

+ (void)actionRFIDSetQValue:(NSUInteger)theQValue :(BOOL)dynamic
{
	[NHSession rfidSetQValue:theQValue dynamic:dynamic];
}

+ (void)actionRFIDSetSession:(NHRFIDSession)theSession
{
	currentSession = theSession;
	
	[NHSession rfidSetSession:theSession];
}

+ (void)actionRFIDSwitchSession
{
	// switch session
	if (changeSession)
	{
		if (currentSession == 2)
			[self actionRFIDSetSession:NHRFIDSession3];
		else if (currentSession	== 3)
			[self actionRFIDSetSession:NHRFIDSession2];
		changeSession = NO;
	}	
}

+ (void)actionRFIDSetTarget:(NHRFIDTarget)theTarget :(BOOL)toggle
{
	[NHSession rfidSetTarget:theTarget toggle:toggle];
}

+ (void)actionRFIDSetTimeout:(NSUInteger)timeout
{
	rfidOnTime = timeout;
}

+ (void)actionRFIDReadEPC
{
	dispatch_async(q_tagList, ^{
        if (tagSet == nil) {
            tagSet = [[NSMutableIndexSet alloc] init];
        }
        else {
            [tagSet removeAllIndexes];
        }
		if (tagListSingleFiltered == nil)
			tagListSingleFiltered = [[NSMutableSet alloc] initWithCapacity:100];
		else
			[tagListSingleFiltered removeAllObjects];
		
		if (tagListSingle == nil)
			tagListSingle = [[NSMutableSet alloc] initWithCapacity:100];
		else
			[tagListSingle removeAllObjects];
	});
	
	[NHSession rfidReadEPCsWithTimeout:rfidOnTime power:rfidReadPower select:rfidCurrentSelect];
}

+ (void)actionRFIDWriteEPC;
{
	[NHSession rfidWriteEPC:rfidEPCToWrite];
}

+ (void)actionRFIDReadDetailedTagInformation:(NSUInteger)power
{
	[NHSession rfidReadData:rfidReadDataMemoryBank addressInWords:rfidReadDataAddressInWords lengthInWords:rfidReadDataLengthInWords timeout:rfidOnTime power:power select:rfidCurrentSelect password:rfidCurrentPassword];
}

+ (void)actionRFIDSetSelectLastRead
{
	if (rfidCurrentSelect != nil)
		[rfidCurrentSelect release];
	
	NHEPCObservation *observation = [tagListSingle anyObject];
	
	rfidCurrentSelect = [NHSelect selectWithEPCCode:observation.epcCode];
	[rfidCurrentSelect retain];
}

+ (void)actionRFIDSetSelectLastWritten
{		
	if (rfidCurrentSelect != nil)
		[rfidCurrentSelect release];
	
	rfidCurrentSelect = [NHSelect selectWithEPCCode:rfidEPCToWrite];
	[rfidCurrentSelect retain];
}

+ (void)actionRFIDResetSelect
{
	if (rfidCurrentSelect != nil)
		[rfidCurrentSelect release];
	
	rfidCurrentSelect = nil;
}

+ (void)actionRFIDSetReadPower:(NSUInteger)power
{
	rfidReadPower = power;
}

+ (NSUInteger)actionRFIDGetReadPower
{
    return rfidReadPower;
}

#pragma mark barcode

+ (void)actionBarcodeRead
{
	[NHSession barcodeTryDecode:[NHPresentation sharedPresentation].barcodeReadDuration];
}

#pragma mark nfc

+ (void)actionNFCTryToReadCard
{
	[NHSession nfcTryToReadCard];
}

#pragma mark other

+ (void)actionOtherPostNotification:(NSString *)notificationName
{
	[notificationName retain];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:notificationName object:self userInfo:nil];
		[notificationName release];
	});
} 

+ (void)actionLogError:(NSString *)errorName;
{
	NHDLog(@"LogError: %@", errorName);
	
	//[FlurryAPI logError:@"0" message:errorName error:nil];
}

+ (void)actionLogEvent:(NSString *)eventName
{
    NHDLog(@"LogEvent: %@", eventName);
	
	//[FlurryAPI logEvent:eventName];
}

+ (void)actionLogTimedEvent:(NSString *)eventName
{
	NHDLog(@"LogTimedEvent: %@", eventName);
	
	//[FlurryAPI logEvent:eventName timed:YES];
	currentTimedEvent = eventName;
}

+ (void)actionLogEndTimedEvent:(NSString *)eventName
{
	NHDLog(@"LogEndTimedEvent: %@", eventName);
	
	//[FlurryAPI endTimedEvent:eventName withParameters:nil];
}

+ (void)actionLogEndCurrentTimedEvent
{
	if (currentTimedEvent != nil)
		[self actionLogEndTimedEvent:currentTimedEvent];
	
	currentTimedEvent = nil;
}

#pragma mark -
#pragma mark Timer actions

+ (void)startTimer:(NSString *)timer :(int)duration
{
    NSInvocation *inv = [NSInvocation invocationWithMethodSignature:[NHPresentationContext instanceMethodSignatureForSelector:NSSelectorFromString(timer)]];
    [inv setTarget:_fsm];
    [inv setSelector:NSSelectorFromString(timer)];
    NSTimer *aTimer = [NSTimer scheduledTimerWithTimeInterval:(float)duration / 1000.0
                                                   invocation:inv
                                                      repeats:NO];
    // Do not use NSDefaultRunLoopMode! Because this runloop handles GUI events, and may block timer events.
	[[NSRunLoop mainRunLoop] addTimer:aTimer forMode:NSRunLoopCommonModes];
    
    [_timerDict setObject:aTimer forKey:timer];
}

+ (void)stopTimer:(NSString *)timer
{
    NSTimer *aTimer = [_timerDict objectForKey:timer];
    [aTimer invalidate];
    [_timerDict removeObjectForKey:timer];
}

+ (void)resetTimer:(NSString *)timer
{
    NSTimer *aTimer = [_timerDict objectForKey:timer];
    [aTimer invalidate];
    NSDate *fireDate = [[NSDate date] dateByAddingTimeInterval:[aTimer timeInterval]];
    [aTimer setFireDate:fireDate];
    // Do not use NSDefaultRunLoopMode! Because this runloop handles GUI events, and may block timer events.
	[[NSRunLoop mainRunLoop] addTimer:aTimer forMode:NSRunLoopCommonModes];
}

#pragma mark -
#pragma mark Delegates

#pragma mark general

- (void)sessionHandheldDidConnect:(NSString *)modelNumber serialNumber:(NSString *)serialNumber firmwareRevision:(NSString *)firmwareRevision hardwareRevision:(NSString *)hardwareRevision;
{
	NSDictionary *accessoryInformation = [[NSDictionary alloc] initWithObjects:[NSArray arrayWithObjects:
																			  modelNumber,
																			  serialNumber,
																			  firmwareRevision,
																			  hardwareRevision, 
																			  nil]
																	 forKeys:[NSArray arrayWithObjects:
																			  @"modelNumber",
																			  @"serialNumber",
																			  @"firmwareRevision",
																			  @"hardwareRevision",
																			  nil]];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidConnectNotification" object:self userInfo:accessoryInformation];
		[accessoryInformation release];
	});

	//[FlurryAPI logEvent:@"handheldDidConnect" withParameters:accessoryInformation];
	
	[_fsm connected];
}

- (void)sessionHandheldDidDisconnect
{
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidDisconnectNotification" object:self userInfo:nil];
	});
	
	//[FlurryAPI logEvent:@"handheldDidDisconnect"];
	
	[_fsm disconnected];	
}

- (void)sessionHandheldDidReceiveBatteryPercentage:(NSNumber *)percentage
{
	[percentage retain];
	
	NSArray *objects = [NSArray arrayWithObjects:percentage, nil];
	NSArray *keys = [NSArray arrayWithObjects:@"percentage", nil];
	NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidReceiveBatteryPercentageNotification" object:self userInfo:dictionary];
		[percentage release];
	});
}

- (void)sessionHandheldDidReceiveLowBatteryWarning
{
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidReceiveLowBatteryNotification" object:self userInfo:nil];
	});
}

- (void)sessionHandheldDidReceiveEmptyBatteryStatus
{
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidReceiveEmptyBatteryNotification" object:self userInfo:nil];
	});
}

- (void)sessionHandheldDidReceiveChargingStatus
{
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidReceiveChargingNotification" object:self userInfo:nil];
	});
}

- (void)sessionHandheldDidReceiveError:(NSString *)error
{
	[NHPresentation actionLogError:error];
}

- (void)sessionHandheldDidReceiveBootloaderReady:(NSString *)hardwareVersion;
{
    [_fsm bootloaderReady];
}

- (void)sessionHandheldDidReceiveBootloaderPageWritten:(NSUInteger)address length:(NSUInteger)length
{
    float percentage = (float)(address + length) / (float)[newFirmware length];
    
    NSArray *objects = [NSArray arrayWithObjects:[NSNumber numberWithFloat:percentage], nil];
	NSArray *keys = [NSArray arrayWithObjects:@"percentage", nil];
	NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidReceiveFirmwareUpdateProgressNotification" object:self userInfo:dictionary];
	});
    
    if (address + length == [newFirmware length])
    {
        [_fsm bootloaderAllPageDone];
    }
    else
    {
        newFirmwarePointer = address + length; // proceed to the next page
        [_fsm bootloaderPageComplete];
    }
}

- (void)sessionHandheldDidReceiveBootloaderUpdateCompleted
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationHandheldDidReceiveFirmwareUpdateCompletedNotification" object:self userInfo:nil];
	});
    
    [_fsm bootloaderUpdateCompleted];
}

#pragma mark user interaction

- (void)sessionUIDidPushButton
{
	[_fsm uiButtonPushed];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationUIDidPushAButtonNotification" object:self userInfo:nil];
	});
}

- (void)sessionUIDidShake
{
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationUIDidShakeNotification" object:self userInfo:nil];
	});
}

#pragma mark rfid

- (void)sessionRFIDDidReceiveSupportedRegions:(NHRFIDRegion)regionFlags
{
	NSArray *objects = [NSArray arrayWithObjects:[NSNumber numberWithUnsignedInt:regionFlags], nil];
	NSArray *keys = [NSArray arrayWithObjects:@"regionFlags", nil];
	NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationDidReceiveSupportedRegionsNotification" object:self userInfo:dictionary];
	});
}

- (NSString *) dataToString:(NSData *)bytes {
    NSString *byteparts = @"";
    const unsigned char* buffer = [bytes bytes];
    NSUInteger size =  [bytes length];
    for (int i = 0; i < size; i++){
        byteparts = [byteparts stringByAppendingFormat:@"%u,",buffer[i]];
    }
    return byteparts;
}

- (BOOL) containsHash:(NSUInteger)hash {
    return [tagInventorySet containsIndex:hash];
}

- (BOOL) containsEpc:(EPCCode *)epc {
    return [tagListInventory containsObject:epc];
}

- (void)sessionRFIDDidReadTag:(NHEPCObservation *)epcObservation
{
	[epcObservation retain];
	dispatch_async(q_tagList, ^{
        NSUInteger epcHash = [epcObservation hash];
		BOOL inList = [tagSet containsIndex:epcHash];

		if (!inList) {
			[tagListSingle addObject:epcObservation];
            [tagSet addIndex:epcHash];
			
			// check whether it is a new tag
//			inList = [self containsHash:epcHash];
            inList = [self containsEpc:epcObservation.epcCode];

			if (!inList) {
				[tagListSingleFiltered addObject:epcObservation];
                [tagInventorySet addIndex:epcHash];
				
				[tagListInventory addObject:epcObservation.epcCode];
				NHEPCObservation *epcObservationCopy = [epcObservation copy];
				
				NSArray *objects = [NSArray arrayWithObjects:epcObservationCopy, nil];
				NSArray *keys = [NSArray arrayWithObjects:@"epcObservation", nil];
				NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
				
				dispatch_async(dispatch_get_main_queue(), ^{
                    [NHPresentation notify:@"NHPresentationRFIDDidReadTagNotification" object:self userInfo:dictionary];
					[epcObservationCopy release];
				});
			}
		}
		[epcObservation release];
	});	
}

- (void)sessionRFIDDidFinishReadCycle
{	
	[_fsm rfidReadFinished];
    
	dispatch_async(q_tagList, ^{
		@autoreleasepool {
			if ([tagListSingleFiltered count] > 0)
				if ([[[_fsm state] name] hasPrefix:@"MainMap::RFIDInventory"])
					[NHPresentation actionUITuneShort];
			
            NSSet *tagListSingleFilteredCopy = [[NSSet alloc] initWithSet:tagListSingleFiltered copyItems:YES];
			NSDictionary *dictionary = [NSDictionary dictionaryWithObject:tagListSingleFilteredCopy forKey:@"epcObservations"];
			dispatch_async(dispatch_get_main_queue(), ^{
                [NHPresentation notify:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:self userInfo:dictionary];
				[tagListSingleFilteredCopy release];
			});
		}
		
		NSSet *tagListSingleCopy = [[NSSet alloc] initWithSet:tagListSingle copyItems:YES];
		
		NSUInteger count = [tagListSingleCopy count];
		
		if (count == 0)
			[_fsm rfidReadFinishedZero];
		
		//if (count == 1)
		//	[self.sharedPresentation._fsm rfidReadFinishedOne];
			
		if (count > 0)
			[_fsm rfidReadFinishedMoreThanZero];
			
		//if (count > 1)
		//	[self.sharedPresentation._fsm rfidReadFinishedMoreThanOne];
		
		NSArray *objects = [NSArray arrayWithObjects:tagListSingleCopy, nil];
		NSArray *keys = [NSArray arrayWithObjects:@"epcObservations", nil];
		NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
        
		dispatch_async(dispatch_get_main_queue(), ^{
            [NHPresentation notify:@"NHPresentationRFIDDidReadTagsNotification" object:self userInfo:dictionary];
			[tagListSingleCopy release];
		});
	});
}

- (void)sessionRFIDDidReadData:(NSData *)data moreTagsAvailable:(BOOL)available
{
	[data retain];

	if ([[[_fsm state] name] hasPrefix:@"MainMap::RFIDReadDetailedTagInformation"])
	{
		const unsigned char* dataBytes = [data bytes];
		
		NSUInteger dataInteger = dataBytes[3];
		dataInteger |= dataBytes[2] << 8;
		dataInteger |= dataBytes[1] << 16;
		dataInteger |= dataBytes[0] << 24;
		
		NHRFIDMDID tagMDID = (dataInteger & 0x7FF000) >> 12;
		NSUInteger tagModelNumber = dataInteger & 0xFFF;
		BOOL tagXTID = NO;
		if ((dataInteger & 0x00800000) > 0)
		{
			tagXTID = YES;
		}
		
		NSArray *objects = [NSArray arrayWithObjects:
							[NSNumber numberWithUnsignedInt:tagMDID], 
							[NSNumber numberWithUnsignedInt:tagModelNumber], 
							[NSNumber numberWithBool:tagXTID], 
							nil];
		NSArray *keys = [NSArray arrayWithObjects:@"tagMDID", @"tagModelNumber", @"tagXTID", nil];
		NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
		
		dispatch_async(dispatch_get_main_queue(), ^{
            [NHPresentation notify:@"NHPresentationRFIDDidReadDetailedTagInformationNotification" object:self userInfo:dictionary];
		});
	}
	else 
	{
		
	}
	
	if (available)
	{
		[_fsm rfidActionSuccessMoreTags];
	}
	else
	{
		[_fsm rfidActionSuccess];
	}
	
	[data release];
}

- (void)sessionRFIDDidWriteDataMoreTagsAvailable:(BOOL)available
{
	if (available)
	{
		[_fsm rfidActionSuccessMoreTags];
	}
	else
	{
		[_fsm rfidActionSuccess];
	}
}

- (void)sessionRFIDDidFailButTagsAvailable:(BOOL)available
{
	if (available)
	{
		[_fsm rfidActionFailMoreTags];
	}
	else
	{
		[_fsm rfidActionFail];
	}
}

- (void)sessionRFIDWriteEPCResultNoTagsInField
{
	[_fsm rfidWriteEPCNoTagsInField];
}

- (void)sessionRFIDWriteEPCResultTooMuchTagsInField
{
	[_fsm rfidWriteEPCTooMuchTagsInField];
}

- (void)sessionRFIDWriteEPCResultWriteToTagFailed
{
	[_fsm rfidWriteEPCWriteToTagFailed];
}

- (void)sessionRFIDWriteEPCResultWriteSuccessful
{
	[_fsm rfidWriteEPCWriteSuccessful];
}

#pragma mark barcode

- (void)sessionBarcodeDidReadABarcode:(NSString *)barcode barcodeType:(NHBarcodeType)type
{
    // generate dictionary with results
    NSArray *objects = [NSArray arrayWithObjects:barcode, [NSNumber numberWithInt:type], nil];
	NSArray *keys = [NSArray arrayWithObjects:@"barcode", @"type", nil];
	NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
    
	// check if block is present for barcode validation
    if (barcodeValidationBlock != nil)
    {
        // invoke selector with barcode string and type
        BOOL validated = barcodeValidationBlock(dictionary);
        
        if (!validated)
        {
            // if not validated, consider it a fail, but with a specific error notification
            [_fsm barcodeReadFail];
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [NHPresentation notify:@"NHPresentationBarcodeDidNotReadAValidBarcodeNotification" object:self userInfo:dictionary];
            });
            
            return;
        }
    }
    
    // either barcode is validated or no validation block is present
    [_fsm barcodeReadSuccess];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationBarcodeDidReadABarcodeNotification" object:self userInfo:dictionary];
	});
}

- (void)sessionBarcodeDidNotReadABarcode
{
	[_fsm barcodeReadFail];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationBarcodeDidNotReadABarcodeNotification" object:self userInfo:nil];
	});
}

#pragma mark nfc

- (void)sessionNFCDidReadCard:(NSUInteger)cardNumber
{
	[_fsm nfcCardFound];
    
	NSArray *objects = [NSArray arrayWithObjects:[NSNumber numberWithUnsignedInt:cardNumber], nil];
	NSArray *keys = [NSArray arrayWithObjects:@"cardNumber", nil];
	NSDictionary *dictionary = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
	
	dispatch_async(dispatch_get_main_queue(), ^{
        [NHPresentation notify:@"NHPresentationNFCDidReadCardNotification" object:self userInfo:dictionary];
	});
}

- (void)sessionNFCDidNotFoundACard
{
	[_fsm nfcNoCardFound];
}

#pragma mark -
#pragma mark Values

+ (NSUInteger)_rfidReadOffTime
{
	return rfidOffTime;	
}

@end
