//
//  NHSession.m
//  NedapHandheldAPI
//
//  Created by Danny Haak on 16-03-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

/*
 * STUFF TO IMPLEMENT
 */
//+ (void)rfidLock:(NHRFIDLock)maskBits lockBits:(NHRFIDLock)lockBits timeout:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password;
//+ (void)rfidKill:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password;
//- (void)sessionRFIDDidLockTagMoreTagsAvailable:(BOOL)available;
//- (void)sessionRFIDDidKillTagMoreTagsAvailable:(BOOL)available;

#import "NHSession.h"
#import "NHSessionDelegateProtocol.h"
#import "NHSelect.h"
#import "NHPassword.h"
#import "NHEPCObservation.h"

#import "EPCGlobalTDS.h"

#import <ExternalAccessory/ExternalAccessory.h>
//#import <QuartzCore/QuartzCore.h>
#import "NedapHandheldAPI.h"

typedef enum {
	NHCommandGeneralGetBatteryPercentage	= 0x01,
	NHCommandGeneralGetDeviceDetails		= 0x02,
	NHCommandGeneralDisconnect				= 0x03,
    NHCommandGeneralBootloader              = 0x04,
    NHCommandGeneralBootloaderSendPage      = 0x05,
    NHCommandGeneralBootloaderFinish        = 0x06,
	
	NHCommandRFIDSetParameter				= 0x21,
	NHCommandRFIDGetParameter				= 0x22,
	NHCommandRFIDReadEPC					= 0x23,
	NHCommandRFIDReadData					= 0x24,
	NHCommandRFIDWrite						= 0x25,
	NHCommandRFIDLock						= 0x26,
	NHCommandRFIDKill						= 0x27,
	NHCommandRFIDSetRegion					= 0x28,
	NHCommandRFIDSetEPCParameters			= 0x29,
	NHCommandRFIDSetCWSignal				= 0x2A,
	NHCommandRFIDSetFrequencyHopTable		= 0x2B,
	NHCommandRFIDGetSupportedRegions		= 0x2C,
	NHCommandRFIDWriteEPC					= 0x2D,
	
	NHCommandBarcodeSetType					= 0x31,
	NHCommandBarcodeTryDecode				= 0x32,
	
	NHCommandUILedSetStatusLed				= 0x41,
	NHCommandUIBeeperTune					= 0x42,
	NHCommandUIVibrationVibrate				= 0x43,
	NHCommandUIVibrationTune				= 0x44,
	
	NHCommandNFCTryToReadCard				= 0x51
} NHCommand;

typedef enum {
	NHResponseError                             = 0xFF,
	
	NHResponseGeneralBatteryPercentage          = 0x81,
	NHResponseGeneralDeviceDetails              = 0x82,
	NHResponseGeneralLowBattery                 = 0x83,
	NHResponseGeneralEmptyBattery               = 0x84,
	NHResponseGeneralDeviceCharging             = 0x85,
    NHResponseGeneralBootloaderReady            = 0x86,
    NHResponseGeneralBootloaderPageWritten      = 0x87,
    NHResponseGeneralBootloaderUpdateCompleted  = 0x88,
	
	NHResponseRFIDTagCount                      = 0xA1,
	NHResponseRFIDTags                          = 0xA2,
	NHResponseRFIDReadData                      = 0xA3,
	NHResponseRFIDWrite                         = 0xA4,
	NHResponseRFIDSupportedRegions              = 0xA5,
	NHResponseRFIDWriteEPC                      = 0xA6,
	
	NHResponseBarcodeBarcodeRead                = 0xB1,
	NHResponseBarcodeNothingRead                = 0xB2,
	
	NHResponseUIButtonPush                      = 0xC1,
	NHResponseUIAccelerometerShake              = 0xC2,
	
	NHResponseNFCNoCardFound                    = 0xD1,
	NHResponseNFCCardFound                      = 0xD2
} NHResponse;

typedef enum {
	NHErrorCRC							= 0x00,
	NHErrorWrongNumberOfBytes			= 0x01,
	
	NHErrorRFIDAnalogFrontendNotEnabled = 0x21,
	NHErrorRFIDTemperatureExceedLimits	= 0x22,
	NHErrorRFIDHighReturnLoss			= 0x23,
	NHErrorRFIDBusy						= 0x24,
    NHErrorRFIDRegionNotSupported       = 0x26,
	
	NHErrorBarcodeNotAvailable			= 0x31
} NHError;

typedef enum {
	NHRFIDEPCParametersSession		= 0x00,
	NHRFIDEPCParametersTarget		= 0x01,
	NHRFIDEPCParametersMValue		= 0x02,
	NHRFIDEPCParametersQValue		= 0x03
} NHRFIDEPCParameters;

typedef enum {
	NHRFIDPasswordTypeKill		= 0,
	NHRFIDPasswordTypeAccess	= 1
} NHRFIDPasswordType;

typedef enum {
	NHRFIDFlagSelect				= 1 << 0,
	NHRFIDFlagSelectInvert			= 1 << 1,
	NHRFIDFlagPassword				= 1 << 4
} NHRFIDFlag;

typedef enum {
	NHRFIDWriteEPCResultNoTagsInField		= 0,
	NHRFIDWriteEPCResultTooMuchTagsInField	= 1,
	NHRFIDWriteEPCResultWriteToTagFailed	= 2,
	NHRFIDWriteEPCResultWriteSuccess		= 3
} NHRFIDWriteEPCResult;

static EAAccessory *accessory;
static EASession *session;

// create gcd queue to parse the packets
static dispatch_queue_t q_parsePackets;
static dispatch_queue_t q_sendPackets;

// static callback
static id <NHSessionDelegateProtocol> presentationDelegate;

static NSMutableData *outputDataBuffer;

// status variables
static NHRFIDRegion currentRegion;

BOOL handheldConnected = NO;

#pragma mark -

@interface NHSession()

#pragma mark -
#pragma mark Internal

+ (void)log:(NSString *)string;
+ (void)connectAccessories;
+ (NSData *)rfidPrepareStandardParameters:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password;

#pragma mark -
#pragma mark Packet sending/receiving

+ (void)sendCommand:(NHCommand)commandID withData:(NSData *)data;

+ (void)parsePackets:(NSData *)receiveBuffer;
+ (NSInteger)parsePacket:(NSData *)receiveBuffer;

+ (void)parsePacketError:(NSData *)data;

+ (void)parsePacketGeneralBatteryPercentage:(NSData *)data;
+ (void)parsePacketGeneralLowBattery;
+ (void)parsePacketGeneralEmptyBattery;
+ (void)parsePacketGeneralDeviceCharging;
+ (void)parsePacketGeneralBootloaderReady:(NSData *)data;
+ (void)parsePacketGeneralBootloaderPageWritten:(NSData *)data;
+ (void)parsePacketGeneralBootloaderUpdateCompleted;

+ (void)parsePacketRfidTagCount:(NSData *)data;
+ (void)parsePacketRfidTags:(NSData *)data;
+ (void)parsePacketRfidReadData:(NSData *)data;
+ (void)parsePacketRfidWrite:(NSData *)data;
+ (void)parsePacketRfidSupportedRegions:(NSData *)data;
+ (void)parsePacketRfidWriteEPC:(NSData *)data;

+ (void)parsePacketBarcodeRead:(NSData *)data;
+ (void)parsePacketBarcodeFailed;

+ (void)parsePacketUiButtonPush;
+ (void)parsePacketUiAccelerometerShake;

+ (void)parsePacketNfcNoCardFound;
+ (void)parsePacketNfcCardFound:(NSData *)data;

#pragma mark -
#pragma mark Accessories notifications

+ (void)accessoryDidConnectNotification:(NSNotification *)notification;
+ (void)accessoryDidDisconnectNotification:(NSNotification *)notification; 

+ (void)writeData;

@end

#pragma mark -

@implementation NHSession

+ (NHSession *)sharedSession {
	static NHSession *sharedSession;
	
	@synchronized(self)	{
		if (!sharedSession)
			sharedSession = [[NHSession alloc] init];
		
		return sharedSession;
	}
	
	return nil;
}

+ (void)initialize {
	[EPCTDTEngine self];
	
	outputDataBuffer = [[NSMutableData alloc] initWithCapacity:1024];
	
	// create GCD queue for parsing and sending packets
	q_parsePackets = dispatch_queue_create("com.nedap.retail.handheld.parsepackets", NULL);
	q_sendPackets = dispatch_queue_create("com.nedap.retail.handheld.sendpackets", NULL);
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(accessoryDidConnectNotification:) name:@"EAAccessoryDidConnectNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(accessoryDidDisconnectNotification:) name:@"EAAccessoryDidDisconnectNotification" object:nil];
	[[EAAccessoryManager sharedAccessoryManager] registerForLocalNotifications];
	
	[self connectAccessories];
}

#pragma mark -
#pragma mark Internal

+ (void)log:(NSString *)string {
	NHDLog(@"%@", string);
}

+ (void)connectAccessories {
	NSArray *accessories = [[EAAccessoryManager sharedAccessoryManager] connectedAccessories];
	NSString *protocolString = @"com.nedap.retail.handheld";
    
	for (EAAccessory *obj in accessories) {
		if ([[obj protocolStrings] containsObject:protocolString]) {
			accessory = obj;
			break;
		}
	}
    
	if (accessory) {
        if (accessory.connected) {
            @try {
                // warning, but do not change, otherwise things will break badly (Danny)
                accessory.delegate = (id<EAAccessoryDelegate>)self;
                
                session = [[EASession alloc] initWithAccessory:accessory forProtocol:protocolString];
                
                @synchronized(session) {
                    if (session != nil
                        && [session inputStream] != nil
                        && [session outputStream] != nil) {

                        // warning, but do not change, otherwise things will break badly (Danny)
                        [[session inputStream] setDelegate:(id<NSStreamDelegate>)self];
                        [[session inputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop]	forMode:NSRunLoopCommonModes];
                        [[session inputStream] open];

                        // warning, but do not change, otherwise things will break badly (Danny)
                        [[session outputStream] setDelegate:(id<NSStreamDelegate>)self];
                        [[session outputStream] scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
                        [[session outputStream] open];
                        currentRegion = NHRFIDRegionUndefined;
                        
                        [presentationDelegate sessionHandheldDidConnect:accessory.modelNumber serialNumber:accessory.serialNumber firmwareRevision:accessory.firmwareRevision hardwareRevision:accessory.hardwareRevision];
                        
                        handheldConnected = YES;
                    }
                    else {
//                        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Connection with accessory failed" message:@"The iPod was unable to connect with the !D Hand. Please press the button on the !D Hand for more than ten seconds, and re-establish the connection in the Bluetooth menu."  delegate:self cancelButtonTitle:@"Dismiss" otherButtonTitles:nil];
//                        [alert show];
//                        [alert release];
                        
                        handheldConnected = NO;
                    }
                }
            }
            @catch (NSException *e) {
                NHRLog(@"[ERROR] could not connect accessory: %@",e);
            }
        }
    }
}

+ (NSData *)rfidPrepareStandardParameters:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password
{
	// check timeout
	if (timeout > 1000) {
		timeout = 1000;
		[self log:@"RFID: Timeout cannot be larger than 1000 ms."];
	}
	
	// check power
	if (power > 23) {
		power = 23;
		[self log:@"RFID: Output power cannot be larger than 23 dBm."];
	}
	if (power < 10) {
		[self log:@"RFID: Output power cannot be lower than 10 dBm."];
		power = 10;
	}
	
	if (password != nil) {
		if ([password.hexString isEqualToString:@"00000000"])
		{
			[self log:@"RFID: When using no password, set it to nil instead of '00000000'."];
			password = nil;
		}
	}
	
	unsigned char dataArray[255];
	
	NSUInteger pointer = 1;
	
	NHRFIDFlag flags = 0;
	
	// add select statement if necessary
	if (select != nil) {
		flags |= NHRFIDFlagSelect;
		
		if (select.invert)
			flags |= NHRFIDFlagSelectInvert;
		
		pointer++;
		
		dataArray[pointer++] = select.memoryBank;
		dataArray[pointer++] = (select.addressInBits >> 8) & 0xFF;
		dataArray[pointer++] = select.addressInBits & 0xFF;
		dataArray[pointer++] = (select.dataLengthInBits >> 8) & 0xFF;
		dataArray[pointer++] = select.dataLengthInBits & 0xFF;
		
		NSData *selectData = select.data;
		const unsigned char *selectDataBytes = [selectData bytes];
		for (int i = 0; i < [selectData length]; i++)
			dataArray[pointer++] = selectDataBytes[i];
		
		dataArray[1] = [selectData length] + 5;
	}
	
	// add password if necessary
	if (password != nil) {
		flags |= NHRFIDFlagPassword;
		
		NSData *passwordData = [EPCConvert convertToDataFromHexString:password.hexString];
		const unsigned char *passwordBytes = [passwordData bytes];
		for (int i = 0; i < [passwordData length]; i++)
			dataArray[pointer++] = passwordBytes[i];
	}
	
	dataArray[0] = flags;
	
	dataArray[pointer++] = (timeout >> 8) & 0xFF;
	dataArray[pointer++] = timeout & 0xFF;
	dataArray[pointer++] = power;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:pointer] autorelease];
	
	return data;
}

#pragma mark -
#pragma mark API calls

#pragma mark management

+ (void)setDelegate:(id <NHSessionDelegateProtocol>)delegate;
{
	presentationDelegate = delegate;
}

#pragma mark general

+ (BOOL)handheldIsConnected 
{
	return handheldConnected;
}

+ (void)handheldGetBatteryPercentage
{
	[self sendCommand:NHCommandGeneralGetBatteryPercentage withData:nil];
}

+ (void)handheldDisconnect
{
	[self sendCommand:NHCommandGeneralDisconnect withData:nil];	
}

+ (NSDictionary *)handheldGetInformation
{
	NSDictionary *accessoryInformation = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:
																			  accessory.modelNumber,
																			  accessory.serialNumber,
																			  accessory.firmwareRevision,
																			  accessory.hardwareRevision, 
																			  nil]
																	 forKeys:[NSArray arrayWithObjects:
																			  @"modelNumber",
																			  @"serialNumber",
																			  @"firmwareRevision",
																			  @"hardwareRevision",
																			  nil]];
	return accessoryInformation;
}

+ (void)handheldBootloader
{
    [self sendCommand:NHCommandGeneralBootloader withData:nil];
}

+ (void)handheldBootloaderSendPage:(NSData *)page address:(NSUInteger)address
{
    NSMutableData *data = [[[NSMutableData alloc] init] autorelease];
    unsigned char addressArray[2] = {(address >> 8) & 0xFF, address & 0xFF};
    [data appendBytes:addressArray length:2];
    [data appendData:page];
    
    [self sendCommand:NHCommandGeneralBootloaderSendPage withData:data];
}

+ (void)handheldBootloaderFinish:(NSUInteger)crc
{
    NSMutableData *data = [[[NSMutableData alloc] init] autorelease];
    unsigned char crcArray[4] = {(crc >> 24) & 0xFF, (crc >> 16) & 0xFF, (crc >> 8) & 0xFF, crc & 0xFF};
    [data appendBytes:crcArray length:4];
    [self sendCommand:NHCommandGeneralBootloaderFinish withData:data];
}

#pragma mark rfid

+ (void)rfidGetSupportedRegions
{
	unsigned char dataArray[1];
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:0] autorelease];
	[self sendCommand:NHCommandRFIDGetSupportedRegions withData:data];	
}

+ (void)rfidSetRegion:(NHRFIDRegion)region
{
	unsigned char dataArray[1];
	
	dataArray[0] = region;						
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:1] autorelease];
	
	[self sendCommand:NHCommandRFIDSetRegion withData:data];
	
	currentRegion = region;
}

+ (void)rfidSetFrequencyHopTable:(NSArray *)frequencies
{
	if (currentRegion == NHRFIDRegionUndefined)
		[NHSession log:@"Region is undefined. Therefore it's not possible to set hop frequencies."];
	
	NSMutableData *data = [[[NSMutableData alloc] init] autorelease];
	
	// TODO: Add frequency hop table stuff for non-European frequencies.
	
	for (NSNumber *frequencyNumber in frequencies) 
	{
		int frequency = [frequencyNumber intValue];
		
		if (currentRegion == NHRFIDRegionEurope)
		{
			if ((frequency < 865700) || (frequency > 867500))
			{
				NSString *logString = [NSString stringWithFormat:@"Frequency %i outside allowable frequency range for current region.", frequency];
				[NHSession log:logString];
			}
			else 
			{
				unsigned char frequencyBytes[4];
				frequencyBytes[0] = (frequency >> 24) & 0xFF;
				frequencyBytes[1] = (frequency >> 16) & 0xFF;
				frequencyBytes[2] = (frequency >> 8) & 0xFF;
				frequencyBytes[3] = frequency & 0xFF;
				[data appendBytes:frequencyBytes length:4];
			}
		}
	}
	
	if ([data length] > 0)
		[self sendCommand:NHCommandRFIDSetFrequencyHopTable withData:data];
	else 
		[NHSession log:@"Frequency hop table is empty."];
}

+ (void)rfidSetSession:(NHRFIDSession)session
{
	unsigned char dataArray[2];
	dataArray[0] = NHRFIDEPCParametersSession;
	dataArray[1] = session;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:2] autorelease];
	
	[self sendCommand:NHCommandRFIDSetEPCParameters withData:data];
}

+ (void)rfidSetTarget:(NHRFIDTarget)target toggle:(BOOL)toggle
{
	unsigned char dataArray[3];
	dataArray[0] = NHRFIDEPCParametersTarget;
	
	if (toggle)
	{
		dataArray[1] = 0x01;
	}
	else 
	{
		dataArray[1] = 0x00;
	}
	
	dataArray[2] = target;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:3] autorelease];
	
	[self sendCommand:NHCommandRFIDSetEPCParameters withData:data];
}

+ (void)rfidSetMValue:(NHRFIDMValue)mValue
{
	unsigned char dataArray[2];
	dataArray[0] = NHRFIDEPCParametersMValue;
	dataArray[1] = mValue;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:2] autorelease];
	
	[self sendCommand:NHCommandRFIDSetEPCParameters withData:data];
}

+ (void)rfidSetQValue:(NSUInteger)qValue dynamic:(BOOL)dynamic
{
	unsigned char dataArray[3];
	dataArray[0] = NHRFIDEPCParametersQValue;
	
	if (dynamic)
	{
		dataArray[1] = 0x01;
	}
	else 
	{
		dataArray[1] = 0x00;
	}
	
	dataArray[2] = qValue;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:3] autorelease];
	
	[self sendCommand:NHCommandRFIDSetEPCParameters withData:data];
}

+ (void)rfidReadEPCsWithTimeout:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select
{
	NSMutableData *data = [NSMutableData dataWithData:[self rfidPrepareStandardParameters:timeout power:power select:select password:nil]];
	
	[self sendCommand:NHCommandRFIDReadEPC withData:data];
}

+ (void)rfidReadData:(NHRFIDMemoryBank)memoryBank addressInWords:(NSUInteger)addressInWords lengthInWords:(NSUInteger)lengthInWords timeout:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password;
{
	NSMutableData *data = [NSMutableData dataWithData:[self rfidPrepareStandardParameters:timeout power:power select:select password:password]];
	
	NSUInteger pointer = 0;
	unsigned char dataArray[255];
	
	dataArray[pointer++] = memoryBank; // bank
	dataArray[pointer++] = (addressInWords >> 8) & 0xFF; // address in words, high byte
	dataArray[pointer++] = addressInWords & 0xFF; // address in words, low byte
	dataArray[pointer++] = lengthInWords & 0xFF; // length in words
	
	[data appendBytes:dataArray length:pointer];
	
	[self sendCommand:NHCommandRFIDReadData withData:data];	
}

+ (void)rfidWriteEPC:(EPCCode *)epc
{
	if ([[epc hexString] length] % 4 > 0)
	{
		[self log:@"RFID: EPC length not a multiple of four characters."];
		return;		
	}
		
	NSUInteger pointer = 0;
	unsigned char dataArray[255];
	
	// rfu bytes
	dataArray[pointer++] = 0;
	dataArray[pointer++] = 0;
	
	NSData *epcData = [epc binaryData];
	const unsigned char *epcDataBytes = [epcData bytes];
	for (int i = 0; i < [epcData length]; i++)
		dataArray[pointer++] = epcDataBytes[i];
	
	NSData *data = [NSData dataWithBytes:dataArray length:pointer];
	
	[self sendCommand:NHCommandRFIDWriteEPC withData:data];
}

+ (void)rfidWriteData:(NSData *)someData memoryBank:(NHRFIDMemoryBank)memoryBank addressInWords:(NSUInteger)addressInWords timeout:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password;
{
	if ([someData length] % 4 > 0)
	{
		[self log:@"RFID: Data length not a multiple of four characters."];
		return;		
	}
	
	NSMutableData *data = [NSMutableData dataWithData:[self rfidPrepareStandardParameters:timeout power:power select:select password:password]];
	
	NSUInteger someDataLengthInWords = [someData length] / 4;
	
	NSUInteger pointer = 0;
	unsigned char dataArray[255];
	
	dataArray[pointer++] = memoryBank; // bank
	dataArray[pointer++] = (addressInWords >> 8) & 0xFF; // address in words, high byte
	dataArray[pointer++] = addressInWords & 0xFF; // address in words, low byte - includes PC word
	dataArray[pointer++] = someDataLengthInWords * 2; // length of the data in bytes
	
	const unsigned char *someDataBytes = [someData bytes];
	for (int i = 0; i < [someData length]; i++)
		dataArray[pointer++] = someDataBytes[i];
	
	[data appendBytes:dataArray length:pointer];
	
	[self sendCommand:NHCommandRFIDWrite withData:data];
}

+ (void)rfidLock:(NHRFIDLock)maskBits lockBits:(NHRFIDLock)lockBits timeout:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password
{
	if (password == nil)
	{
		[self log:@"RFID: Password cannot be zero when locking a tag."];
		return;	
	}
	
	NSMutableData *data = [NSMutableData dataWithData:[self rfidPrepareStandardParameters:timeout power:power select:select password:password]];
	
	NSUInteger pointer = 0;
	unsigned char dataArray[4];
	
	dataArray[pointer++] = (maskBits >> 8) & 0xFF; // bank
	dataArray[pointer++] = maskBits & 0xFF; // address in words, high byte
	dataArray[pointer++] = (lockBits >> 8) & 0xFF; // address in words, low byte - includes PC word
	dataArray[pointer++] = lockBits & 0xFF; // length of the new EPC in bytes + 2 for the PC word
	
	[data appendBytes:dataArray length:pointer];
	
	[self sendCommand:NHCommandRFIDLock withData:data];
}

+ (void)rfidKill:(NSUInteger)timeout power:(NSUInteger)power select:(NHSelect *)select password:(NHPassword *)password
{
	if (password == nil)
	{
		[self log:@"RFID: Password cannot be zero when trying to kill a tag."];
		return;	
	}
	
	NSMutableData *data = [NSMutableData dataWithData:[self rfidPrepareStandardParameters:timeout power:power select:select password:password]];
	
	[self sendCommand:NHCommandRFIDKill withData:data];
}

+ (void)rfidSetCWSignal:(BOOL)enabled onAntenna:(NSUInteger)antenna;
{
	unsigned char dataArray[2];
	if (enabled)
		dataArray[0] = 1;
	else
		dataArray[0] = 0;
	
	if (antenna == 0)
		dataArray[1] = 0;
	else
		dataArray[1] = 1;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:2] autorelease];
	
	[self sendCommand:NHCommandRFIDSetCWSignal withData:data];
}

#pragma mark barcode

+ (BOOL)barcodeHasReader
{
    // check disabled due to fuck up with new serial number structure
    
	//if ([accessory.modelNumber isEqualToString:@"9943978"])
		return true;
	
	//return false;
}

+ (void)barcodeSetStatusForType:(NHBarcodeType)barcodeType enabled:(BOOL)enabled
{	
	unsigned char dataArray[2];
	dataArray[0] = barcodeType;
	if (enabled)
		dataArray[1] = 1;
	else
		dataArray[1] = 0;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:2] autorelease];
	
	[self sendCommand:NHCommandBarcodeSetType withData:data];
}

+ (void)barcodeTryDecode:(NSUInteger)duration;
{
	unsigned char dataArray[1];
	dataArray[0] = duration * 10;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:1] autorelease];
	
	[self sendCommand:NHCommandBarcodeTryDecode withData:data];
}

#pragma mark user interaction

+ (void)uiSetStatusLedBlinking:(BOOL)blinking;
{
	unsigned char dataArray[1];
	if (blinking)
		dataArray[0] = 2;
	else
		dataArray[0] = 1;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:1] autorelease];
	
	[self sendCommand:NHCommandUILedSetStatusLed withData:data];
}

+ (void)uiPlayTune:(NHUITune)tune
{
	unsigned char dataArray[1];
	dataArray[0] = tune;
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:1] autorelease];
	
	[self sendCommand:NHCommandUIBeeperTune withData:data];
}

+ (void)uiVibrate;
{
	unsigned char dataArray[1];
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:0] autorelease];
	
	[self sendCommand:NHCommandUIVibrationVibrate withData:data];	
}

+ (void)uiVibrate:(NSUInteger)timeout
{	
	unsigned char dataArray[1];
	dataArray[0] = timeout & 0xFF;
	
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:1] autorelease];
	
	[self sendCommand:NHCommandUIVibrationVibrate withData:data];
}

+ (void)uiVibrateTune:(NHUITune)tune
{
	unsigned char dataArray[1];
	dataArray[0] = tune;
	NSData *data = [[[NSData alloc] initWithBytes:dataArray length:1] autorelease];
	
	[self sendCommand:NHCommandUIVibrationTune withData:data];	
}

#pragma mark nfc

+ (void)nfcTryToReadCard
{
	[self sendCommand:NHCommandNFCTryToReadCard withData:nil];
}

#pragma mark -
#pragma mark Packet sending/receiving

+ (void)sendCommand:(NHCommand)commandID withData:(NSData*)data
{
	[data retain];
	
	if (![self handheldIsConnected])
	{
		[self log:@"Handheld: Accessory not connected."];
		[data release];
		return;
	}
	
	unsigned char buffer[255];
	
	const unsigned char* dataBytes = [data bytes];
	NSUInteger dataLength = [data length];
	[data release];
	
	// start byte
	buffer[0] = 0x06;
	
	// length
	buffer[1] = 3 + dataLength;
	
	// commandID
	buffer[2] = commandID;
	
	// data
	for (int i = 0; i < dataLength; i++)
	{
		buffer[3 + i] = dataBytes[i];
	}
	
	// crc
	NSInteger crc = 0x100;
	for (int j = 1; j < dataLength + 3; j++)
	{
		crc -= buffer[j];
	}
	
	buffer[3 + dataLength] = crc;
	
	NSData *newData = [[NSData alloc] initWithBytes:buffer length:dataLength+4];
	
	// send command
	dispatch_async(q_sendPackets, ^{
		[outputDataBuffer appendData:newData];
		[newData release];
		[self writeData];
	});
}

+ (void)parsePackets:(NSData *)receiveBuffer
{
	[receiveBuffer retain];
	
	NSInteger length = [receiveBuffer length];
	
	NSInteger pointer = 0;
	while (pointer < length)
	{
		NSRange range =
		{
			pointer,
			length - pointer
		};
		NSInteger add = [self parsePacket:[receiveBuffer subdataWithRange:range]];
		if (add == 0)
			break;
		else
			pointer += add;
	}
	
	[receiveBuffer release];
}

+ (NSInteger)parsePacket:(NSData *)receiveBuffer
{
	//NSData *receiveBuffer = (NSData *)data;
	
    [receiveBuffer retain];
    
	const unsigned char* buffer = [receiveBuffer bytes];
    
	NSInteger packetType = buffer[0];
	
	NHResponse commandID;
	NSInteger length;
	
    NSData *dataInPacket = nil;
    
	// check packet type (0x06 is SHORT package, maximum 256 bytes, and 0x07 is LONG package)
	if (packetType == 0x06)
	{
		length = buffer[1] + 1;
		
		// verify CRC
		uint8_t crc = 0x00;
		for (uint8_t i = 1; i < buffer[1]; i++)
			crc -= buffer[i];
		
		if (buffer[length - 1] != crc)
		{
			[self log:@"Handheld: Incorrect CRC."];
			return 0;
		}
		
		// get command ID
		commandID = buffer[2];
        
        // get data
        NSRange bytesRange = {
            3,
            [receiveBuffer length] - 4
        };
        dataInPacket = [receiveBuffer subdataWithRange:bytesRange];
	}
	else if (packetType == 0x07)
	{
		length = ((buffer[1] << 8) | buffer[2]) + 1;
        
		// verify CRC
		uint8_t crc = 0x00;
		for (uint16_t i = 1; i < (length - 1); i++)
			crc -= buffer[i];
        
		if (buffer[length - 1] != crc)
		{
			[self log:@"Handheld: Incorrect CRC."];
			return 0;
		}
		
		// get command ID
		commandID = buffer[3];
        
        // get data
        NSRange bytesRange = {
            4,
            [receiveBuffer length] - 5
        };
        dataInPacket = [receiveBuffer subdataWithRange:bytesRange];
	}
	else 
	{
		[self log:@"Handheld: Incorrect start byte."];
		return 0;
	}
    
    [dataInPacket retain];
	
	switch (commandID) {
		case NHResponseError:
			[self parsePacketError:dataInPacket];
			break;
		case NHResponseGeneralBatteryPercentage:
			[self parsePacketGeneralBatteryPercentage:dataInPacket];
			break;
		case NHResponseGeneralLowBattery:
			[self parsePacketGeneralLowBattery];
			break;
		case NHResponseGeneralEmptyBattery:
			[self parsePacketGeneralEmptyBattery];
			break;
		case NHResponseGeneralDeviceCharging:
			[self parsePacketGeneralDeviceCharging];
			break;
        case NHResponseGeneralBootloaderReady:
            [self parsePacketGeneralBootloaderReady:dataInPacket];
            break;
        case NHResponseGeneralBootloaderPageWritten:
            [self parsePacketGeneralBootloaderPageWritten:dataInPacket];
            break;
        case NHResponseGeneralBootloaderUpdateCompleted:
            [self parsePacketGeneralBootloaderUpdateCompleted];
            break;
		case NHResponseRFIDTagCount:
			[self parsePacketRfidTagCount:dataInPacket];
			break;
		case NHResponseRFIDTags:
			[self parsePacketRfidTags:dataInPacket];
			break;
		case NHResponseRFIDReadData:
			[self parsePacketRfidReadData:dataInPacket];
			break;
		case NHResponseRFIDWrite:
			[self parsePacketRfidWrite:dataInPacket];
			break;
		case NHResponseRFIDSupportedRegions:
			[self parsePacketRfidSupportedRegions:dataInPacket];
			break;
		case NHResponseRFIDWriteEPC:
			[self parsePacketRfidWriteEPC:dataInPacket];
			break;
		case NHResponseBarcodeBarcodeRead:
			[self parsePacketBarcodeRead:dataInPacket];
			break;
		case NHResponseBarcodeNothingRead:
			[self parsePacketBarcodeFailed];
			break;
		case NHResponseUIButtonPush:
			[self parsePacketUiButtonPush];
			break;
		case NHResponseUIAccelerometerShake:
			[self parsePacketUiAccelerometerShake];
			break;
		case NHResponseNFCNoCardFound:
			[self parsePacketNfcNoCardFound];
			break;
		case NHResponseNFCCardFound:
			[self parsePacketNfcCardFound:dataInPacket];
			break;
		default:
			break;
	}
    
    [receiveBuffer release];

    [dataInPacket release];	
    
	return length;
}

#pragma mark error

+ (void)parsePacketError:(NSData *)data
{
	const unsigned char* buffer = [data bytes];
	
	NHError errorCode = buffer[0];
	
	NSString *error = nil;
	
	switch (errorCode)
	{
		case NHErrorCRC:
			error = @"Handheld: Error in CRC of transmitted packet";
			break;
		case NHErrorWrongNumberOfBytes:
			error = @"Handheld: Wrong number of bytes in tranmitted packet.";
			break;
		case NHErrorRFIDAnalogFrontendNotEnabled:
			error = @"RFID: Analog frontend not enabled";
			break;
		case NHErrorRFIDTemperatureExceedLimits:
			error = @"RFID: Temperature exceed limits.";
			break;
		case NHErrorRFIDHighReturnLoss:
			error = @"RFID: High return loss on transmitter.";
			break;
		case NHErrorBarcodeNotAvailable:
			error = @"Barcode: Not available.";
			break;
        case NHErrorRFIDRegionNotSupported:
            error = @"RFID: Region not supported.";
			break;
        default:
            error = @"Other error.";
            break;
	}
	
	if (error != nil)
	{
		[presentationDelegate sessionHandheldDidReceiveError:error];
		[self log:error];
	}
}

#pragma mark general

+ (void)parsePacketGeneralBatteryPercentage:(NSData *)data
{
	const unsigned char* buffer = [data bytes];
	NSNumber *percentage = [NSNumber numberWithUnsignedInt:buffer[0]];
	
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveBatteryPercentage:percentage];
	});
}

+ (void)parsePacketGeneralLowBattery
{
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveLowBatteryWarning];
	});
}

+ (void)parsePacketGeneralEmptyBattery
{
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveEmptyBatteryStatus];
	});
}

+ (void)parsePacketGeneralDeviceCharging
{
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveChargingStatus];
	});
}

+ (void)parsePacketGeneralBootloaderReady:(NSData *)data
{
    const unsigned char* buffer = [data bytes];
    NSString *hardwareVersion = [NSString stringWithFormat:@"%i.%i.%i", buffer[0], buffer[1], buffer[2]];
    
    dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveBootloaderReady:hardwareVersion];
	});
}

+ (void)parsePacketGeneralBootloaderPageWritten:(NSData *)data
{
    const unsigned char* buffer = [data bytes];
    NSUInteger address = (buffer[0] << 8) | buffer[1];
    NSUInteger length = buffer[2];
    
    dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveBootloaderPageWritten:address length:length];
	});
}

+ (void)parsePacketGeneralBootloaderUpdateCompleted
{
    dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionHandheldDidReceiveBootloaderUpdateCompleted];
	});  
}

#pragma mark user interaction

+ (void)parsePacketUiButtonPush
{
	// generate notification on the main queue
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionUIDidPushButton];	
	});
}

+ (void)parsePacketUiAccelerometerShake
{
	// generate notification on the main queue
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionUIDidShake];	
	});
}

#pragma mark rfid

+ (void)parsePacketRfidTagCount:(NSData *)data
{
	const unsigned char* buffer = [data bytes];
	
	if (buffer[0] == 0)
	{
		// no tags read
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidFinishReadCycle];
		});
	}
}

+ (void)parsePacketRfidTags:(NSData *)data
{
 	const unsigned char* buffer = [data bytes];
	
	BOOL lastPacket = buffer[0];
	
	NSUInteger numberOfTags = buffer[1];
	
	if (numberOfTags == 0)
	{
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidFinishReadCycle];
		});
		return;
	}

    NSDate *theDate = [NSDate date];
	NSInteger pointer = 2;
	for (NSInteger i = 0; i < numberOfTags; i++) 
	{
		NSUInteger readCount = buffer[pointer++];
		NSUInteger rssi = buffer[pointer++];	
		
		NSUInteger epcLength = ((buffer[pointer] << 8) | buffer[pointer + 1]) / 8;
		pointer += 4;
		
		//NSUInteger pcWord = (buffer[pointer] << 8) | buffer[pointer + 1];
		//pointer += 2;
		
		if (((pointer + epcLength - 4) > [data length]) || ((epcLength - 4) < 1))
		{
			[self log:@"Error in data packet while parsing EPC tags."];
			return;
		}

		NSRange currentRange =
		{
			pointer,
			epcLength - 4
		};

		NSData *epc = [data subdataWithRange:currentRange];
		
        
		EPCCode *tag = [[EPCCode alloc] initWithBinaryData:epc];
        
		NHEPCObservation *tagObservation = [[NHEPCObservation alloc] initWithEPCCode:tag
																		   timeStamp:theDate
																				rssi:rssi
																		   readCount:readCount];
		[tag release];
		
		pointer += epcLength - 4;
		
		// skip CRC of tag
		pointer += 2;
		
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidReadTag:tagObservation];
			[tagObservation release];
		});
	}
	
	if (lastPacket)
	{
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidFinishReadCycle];
		});
		return;
	}
}

+ (void)parsePacketRfidReadData:(NSData *)data
{
	// get write status
	const unsigned char* buffer = [data bytes];
	
	NSUInteger count = buffer[0];
	
	BOOL moreTagsAvailable = NO;
	
	BOOL success = buffer[1];
	
	if (success)
	{
		NSRange range = {
			3,
			[data length] - 3
		};
		NSData *readData = [data subdataWithRange:range];
		[readData retain];
		
		if (count > 1)
			moreTagsAvailable = YES;
		
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidReadData:readData moreTagsAvailable:moreTagsAvailable];
			[readData release];
		});
	} else {
		if (count > 0)
			moreTagsAvailable = YES;
		
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidFailButTagsAvailable:moreTagsAvailable];
		});
	}
}

+ (void)parsePacketRfidWrite:(NSData *)data
{
	// get write status
	const unsigned char* buffer = [data bytes];
	
	NSUInteger count = buffer[0];
	BOOL moreTagsAvailable = NO;
	
	BOOL success = buffer[1];
	
	if (success)
	{
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidWriteDataMoreTagsAvailable:moreTagsAvailable];
		});
	} else {
		if (count > 0)
			moreTagsAvailable = YES;
		
		dispatch_async(dispatch_get_main_queue(), ^{
			[presentationDelegate sessionRFIDDidFailButTagsAvailable:moreTagsAvailable];
		});
	}
}

+ (void)parsePacketRfidSupportedRegions:(NSData *)data
{
	// get write status
	const unsigned char* buffer = [data bytes];
	
	NSUInteger regionFlags = buffer[0];
	
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionRFIDDidReceiveSupportedRegions:regionFlags];
	});
}

+ (void)parsePacketRfidWriteEPC:(NSData *)data
{
	// get write status
	const unsigned char* buffer = [data bytes];
	NHRFIDWriteEPCResult result = buffer[0];

//	NSUInteger length = [data length];    
//    NSString *MDID = nil;
//    NSString *tagModelNumber = nil;
//    NSString *serialNumber = nil;
    
	switch (result) {
		case NHRFIDWriteEPCResultNoTagsInField:
			dispatch_async(dispatch_get_main_queue(), ^{
				[presentationDelegate sessionRFIDWriteEPCResultNoTagsInField];
			});
			break;
		case NHRFIDWriteEPCResultTooMuchTagsInField:
			dispatch_async(dispatch_get_main_queue(), ^{
				[presentationDelegate sessionRFIDWriteEPCResultTooMuchTagsInField];
			});
			break;
		case NHRFIDWriteEPCResultWriteToTagFailed:
			dispatch_async(dispatch_get_main_queue(), ^{
				[presentationDelegate sessionRFIDWriteEPCResultWriteToTagFailed];
			});
			break;
		case NHRFIDWriteEPCResultWriteSuccess:
            // check length for availability of MDID, TagID and serialNumber
            /*
            if (length == 5)
            {
                NHRFIDMDID MDIDinteger = (buffer[1] << 8) | buffer[2];
                NHRFIDTagID TagModelNumberinteger = (buffer[3] << 8) | buffer[4];
                switch (MDIDinteger)
                {
                        case 
                }
                
            }
            else if (length > 5)
            {
                
            }
            
            NHDLog(@"length: %i", length);
            */
            
			dispatch_async(dispatch_get_main_queue(), ^{
				[presentationDelegate sessionRFIDWriteEPCResultWriteSuccessful];
			});
			break;
		default:
			break;
	}
}

#pragma mark barcode

+ (void)parsePacketBarcodeRead:(NSData *)data
{
	// get barcode type
	const unsigned char* buffer = [data bytes];
	
	NSUInteger barcodeType = buffer[0];
	
	NSRange dataRange = {
		1,
		[data length] - 1
	};
	NSString *barcode = [[NSString alloc] initWithData:[data subdataWithRange:dataRange] encoding:NSASCIIStringEncoding];	
	
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionBarcodeDidReadABarcode:barcode barcodeType:barcodeType];
		[barcode release];
	});
}

+ (void)parsePacketBarcodeFailed
{
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionBarcodeDidNotReadABarcode];
	});
}

#pragma mark nfc

+ (void)parsePacketNfcNoCardFound
{
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionNFCDidNotFoundACard];	
	});
}

+ (void)parsePacketNfcCardFound:(NSData *)data
{
	const unsigned char* buffer = [data bytes];
	
	NSUInteger cardNumber = (buffer[0] << 24) | (buffer[1] << 16) | (buffer[2] << 8) | buffer[3];
	
	dispatch_async(dispatch_get_main_queue(), ^{
		[presentationDelegate sessionNFCDidReadCard:cardNumber];	
	});
}

#pragma mark -
#pragma mark Accessories notifications

+ (void)accessoryDidConnectNotification:(NSNotification *)notification  {
	NSDictionary *dict = [notification userInfo];
	EAAccessory *accessory = [dict valueForKey:@"EAAccessoryKey"];
    NSString *protocolString = @"com.nedap.retail.handheld";
    
	if ([[accessory protocolStrings] containsObject:protocolString]) {
        [self connectAccessories];    
    }
    
    handheldConnected = YES;
}

+ (void)accessoryDidDisconnectNotification:(NSNotification *)notification {
    @try {
        NSDictionary *dict = [notification userInfo];
        EAAccessory *accessory = [dict valueForKey:@"EAAccessoryKey"];
        
        NSString *protocolString = @"com.nedap.retail.handheld";
        if ([[accessory protocolStrings] containsObject:protocolString]) {
            [presentationDelegate sessionHandheldDidDisconnect];
            
            @synchronized(session) {
                if (session != nil
                    && [session outputStream] != nil
                    && [session inputStream] != nil) {

                    if ([[session inputStream] streamStatus] < NSStreamStatusAtEnd) {// if stream still makes sense
                        [[session inputStream] close];
                        [[session inputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
                        [[session inputStream] setDelegate:nil];
                    }
                    if ([[session outputStream] streamStatus] < NSStreamStatusAtEnd) {// if stream still makes sense
                        [[session outputStream] close];
                        [[session outputStream] removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
                        [[session outputStream] setDelegate:nil];
                    }
                    
                    NSUInteger length = [outputDataBuffer length];
                    [outputDataBuffer replaceBytesInRange:NSMakeRange(0, length) withBytes:NULL length:0];
                }
                session = nil;
//                [session release];
            }
        }
    } @catch (NSException *e) {
        NHRLog(@"[ERROR] could not disconnect accessory: %@",e);
    }
    handheldConnected = NO;
}

+ (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)eventCode {
	switch (eventCode) {
		case NSStreamEventHasBytesAvailable:;
			uint8_t buffer[4096];
			NSUInteger length = [[session inputStream] read:buffer maxLength:4096];
			NSData *receiveBuffer = [[NSData alloc] initWithBytes:buffer length:length];
			
#ifdef DEBUG
            printf("RX: \n");
			for (int i = 0; i < length; i++)
			{
				printf("%02X ", buffer[i]);
			}
			printf("\n");
#endif
			
			dispatch_async(q_parsePackets, ^{
				[NHSession parsePackets:receiveBuffer];
				[receiveBuffer release];
			});
			break;
		case NSStreamEventHasSpaceAvailable:
			[self writeData];
            break;
		default:
			break;
	}
}

+ (void)writeData {
	dispatch_async(q_sendPackets, ^{
		while (([[session outputStream] hasSpaceAvailable]) && ([outputDataBuffer length] > 0)) {
#ifdef DEBUG
			int length = [outputDataBuffer length];
			const unsigned char* buffer = [outputDataBuffer bytes];
			
            NSString *txLog = [NSString stringWithFormat:@"TX: "];
			for (int i = 0; i < length; i++) {
				[txLog stringByAppendingFormat:@"[%02X] ", buffer[i]];
			}
			NHDLog(@"%@",txLog);
#endif
			
			NSInteger bytesWritten = [[session outputStream] write:[outputDataBuffer bytes] maxLength:[outputDataBuffer length]];
			if (bytesWritten == -1) {
				[self log:@"Stream: write error."];
				break;
			}
			else if (bytesWritten > 0)
			{
				[outputDataBuffer replaceBytesInRange:NSMakeRange(0, bytesWritten) withBytes:NULL length:0];
			}
		}
	});
}

@end