//
//  NHSessionDelegateProtocol.h
//  NedapHandheldAPI
//
//  Created by Danny Haak on 07-07-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "NHDefines.h"

@class EPCCode;
@class NHEPCObservation;

@protocol NHSessionDelegateProtocol

#pragma mark general

- (void)sessionHandheldDidConnect:(NSString *)modelNumber serialNumber:(NSString *)serialNumber firmwareRevision:(NSString *)firmwareRevision hardwareRevision:(NSString *)hardwareRevision;

/**
	The handheld disconnects.
 */
- (void)sessionHandheldDidDisconnect;

- (void)sessionHandheldDidReceiveBatteryPercentage:(NSNumber *)percentage;
- (void)sessionHandheldDidReceiveLowBatteryWarning;
- (void)sessionHandheldDidReceiveEmptyBatteryStatus;
- (void)sessionHandheldDidReceiveChargingStatus;
- (void)sessionHandheldDidReceiveError:(NSString *)error;
- (void)sessionHandheldDidReceiveBootloaderReady:(NSString *)hardwareVersion;
- (void)sessionHandheldDidReceiveBootloaderPageWritten:(NSUInteger)address length:(NSUInteger)length;
- (void)sessionHandheldDidReceiveBootloaderUpdateCompleted;

#pragma mark user interaction

/**
 Callback when the user presses the button on the handheld.
 */
- (void)sessionUIDidPushButton;

/**
 Callback when the user shakes the handheld.
 */
- (void)sessionUIDidShake;

#pragma mark rfid

- (void)sessionRFIDDidReceiveSupportedRegions:(NHRFIDRegion)regionFlags;

/**
	An RFID tag has been read.
	@param epcObservation the observation in which the tag was read
 */
- (void)sessionRFIDDidReadTag:(NHEPCObservation *)epcObservation;

/**
	The current read cycle is ended.
 */
- (void)sessionRFIDDidFinishReadCycle;

/**
	Data has been read from a RFID tag.
	@param data the data that has been read
	@param available boolean to indicate whether more tags than one were available for reading
 */
- (void)sessionRFIDDidReadData:(NSData *)data moreTagsAvailable:(BOOL)available;

/**
 Data was written to a RFID tag.
 @param available boolean to indicate whether more tags than one were available for writing
 */
- (void)sessionRFIDDidWriteDataMoreTagsAvailable:(BOOL)available;

/**
	Indication that a specific action (read data, write, lock, kill) to a RFID tag has failed.
	@param available boolean that indicates whether tags were available
 */
- (void)sessionRFIDDidFailButTagsAvailable:(BOOL)available;

- (void)sessionRFIDWriteEPCResultNoTagsInField;
- (void)sessionRFIDWriteEPCResultTooMuchTagsInField;
- (void)sessionRFIDWriteEPCResultWriteToTagFailed;
- (void)sessionRFIDWriteEPCResultWriteSuccessful;

#pragma mark barcode

/**
	A barcode was read.
	@param barcode barcode type
	@param type contents of the barcode
 */
- (void)sessionBarcodeDidReadABarcode:(NSString *)barcode barcodeType:(NHBarcodeType)type;

/**
	No barcode was read during the specified time-out period.
 */
- (void)sessionBarcodeDidNotReadABarcode;

#pragma mark nfc

/**
 Callback for if the Near Field Card reader did read a card.
 @param cardNumber card serial number
 */
- (void)sessionNFCDidReadCard:(NSUInteger)cardNumber;

/**
	Callback for if the Near Field Card reader did not found a card.
 */
- (void)sessionNFCDidNotFoundACard;

@end