//
//  BarcodeReadViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 10-10-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "BarcodeReadViewController.h"

#import "NedapHandheldAPI.h"
#import "EPCGlobalTDS.h"
#import "RegexKitLite.h"

@implementation BarcodeReadViewController

@synthesize barcodeTypeLabel, barcodeLabel, barcodeNotReadAlertView, compatibleWithStoreID;

#pragma mark -
#pragma mark View lifecycle

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];

    self.title = NSLocalizedString(@"Read Barcode",nil);
    barcodeTypeLabel.text = NSLocalizedString(@"barcode type",nil);
    barcodeLabel.text = NSLocalizedString(@"barcode",nil);

    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadABarcode:) name:@"NHPresentationBarcodeDidReadABarcodeNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didNotReadABarcode:) name:@"NHPresentationBarcodeDidNotReadABarcodeNotification" object:nil];
	
	[self startReadingBarcodes];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];

	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

#pragma mark -
#pragma mark NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	[self startReadingBarcodes];
}

- (NSString *) translateBarcodeType:(NSNumber *)type
{
    NSString *result = @"UNKNOWN";
    switch ([type unsignedIntValue]) {
        case 0:
            result = @"All barcode types"; break;
        case 1:
            result = @"EAN8"; break;
        case 2:
            result = @"EAN13"; break;
        case 3:
            result = @"UPCA"; break;
        case 4:
            result = @"UPCE"; break;
        case 5:
            result = @"EAN128"; break;
        case 6:
            result = @"Codabar"; break;
        case 7:
            result = @"Code11"; break;
        case 8:
            result = @"Code39"; break;
        case 9:
            result = @"Code93"; break;
        case 10:
            result = @"EAN Bookland"; break;
        case 11:
            result = @"Discrete 2 of 5"; break;
        case 12:
            result = @"Interleaved 2 of 5"; break;
        case 13:
            result = @"Code 128"; break;
        case 14:
            result = @"MSI"; break;
        case 15:
            result = @"GS1 Databar"; break;
    }
    return result;
}

- (void)didReadABarcode:(NSNotification *)notification {
	if (barcodeNotReadAlertView != nil)
	{
		[barcodeNotReadAlertView dismissWithClickedButtonIndex:[barcodeNotReadAlertView cancelButtonIndex] animated:YES];
		[barcodeNotReadAlertView release];
		barcodeNotReadAlertView = nil;
	}
	
	NSDictionary *dictionary = [notification userInfo];
	NSString *barcode = [dictionary objectForKey:@"barcode"];
	barcodeLabel.text = barcode;
	barcodeTypeLabel.text = [NSString stringWithFormat:@"type: %@", [self translateBarcodeType:[dictionary objectForKey:@"type"]]];
	
	[self startReadingBarcodes];
    
    bool compatibleBarcode = NO;
    
    if ([[dictionary objectForKey:@"type"] unsignedIntValue] == 2) {
        compatibleBarcode = YES;
    }
    else if (![barcode isMatchedByRegex:@"[^a-zA-Z0-9]"]
             && [barcode length] < 13) {
        compatibleBarcode = YES;
    }
    
    if (compatibleBarcode) {
        [compatibleWithStoreID setImage:[UIImage imageNamed:@"storeid"]];
    }
    else {
        [compatibleWithStoreID setImage:nil];
    }
}

- (void)didNotReadABarcode:(NSNotification *)notification
{
	barcodeNotReadAlertView = [[UIAlertView alloc]
					  initWithTitle:@"No barcode read"
					  message:@"No barcode was read during read phase."
					  delegate:nil
					  cancelButtonTitle:@"OK"
					  otherButtonTitles:nil];
	
	[barcodeNotReadAlertView show];
    
    [compatibleWithStoreID setImage:nil];
}

#pragma mark -
#pragma mark Local functions

- (void)startReadingBarcodes
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    bool readAllBarcodes = [defaults boolForKey:@"readAllBarcodes"];
    NSArray *barcodeTypes = nil;

    // read all barcodes? 
    if (readAllBarcodes)
    {
        barcodeTypes = [[NSArray alloc] initWithObjects:[NSNumber numberWithUnsignedInt:NHBarcodeTypeEAN8],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeEAN13],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeUPCA],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeUPCE],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeEAN128],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeCodabar],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeCode11],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeCode39],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeCode93],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeEANBookland],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeDiscrete2of5],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeInterleaved2of5],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeCode128],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeMSI],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeGS1Databar],
                        nil];
	}
    // if not reading all barcodes then read only EAN8, EAN13, and EAN128 types
    else
    {
        barcodeTypes = [[NSArray alloc] initWithObjects:[NSNumber numberWithUnsignedInt:NHBarcodeTypeEAN8],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeEAN13],
                        [NSNumber numberWithUnsignedInt:NHBarcodeTypeEAN128],
                        nil];
    }
    [NHPresentation sharedPresentation].barcodeReadDuration = 4;
    [NHPresentation sharedPresentation].barcodeTypes = barcodeTypes;
//    [NHPresentation sharedPresentation].barcodeValidationBlock = ^(id input) {
//        NSDictionary *dictionary = (NSDictionary *)input;
//        NSString *barcode = [dictionary valueForKey:@"barcode"];
//        if ([barcode isEqualToString:@"8976690020012"])
//        {
//            NSLog(@"barcodeValidationBlock: this barcode (%@) does not work!", barcode);
//            return NO;
//        }
//        else
//        {
//            NSLog(@"barcodeValidationBlock: and all others do!");
//            return YES;
//        }
//    };
    
    [NHPresentation barcodeStartReading];
    
	[barcodeTypes release];
}

#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)dealloc {
    [super dealloc];
}

@end