//
//  BatteryViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 29-12-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "BatteryViewController.h"

#import "NedapHandheldAPI.h"

@implementation BatteryViewController

@synthesize percentageLabel;

// The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
/*
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization.
    }
    return self;
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Battery",nil);
    
    // set refresh button
    UIBarButtonItem *refreshButton = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Refresh",nil)
																	style:UIBarButtonItemStyleBordered
																   target:self 
																   action:@selector(refresh:)]; 
    self.navigationItem.rightBarButtonItem = refreshButton;
    self.navigationItem.rightBarButtonItem.enabled = YES;
    [refreshButton release];
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(batteryPercentage:) name:@"NHPresentationHandheldDidReceiveBatteryPercentageNotification" object:nil];

	[NHPresentation handheldGetBatteryPercentage];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


#pragma mark -
#pragma mark NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	[NHPresentation handheldGetBatteryPercentage];
}

- (void)batteryPercentage:(NSNotification *)notification
{
	NSDictionary *dictionary = [notification userInfo];
	
	percentageLabel.text = [NSString stringWithFormat:@"%@ %%", [dictionary objectForKey:@"percentage"]];
}

- (IBAction)refresh:(id)sender
{
	[NHPresentation handheldGetBatteryPercentage];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end
