//
//  DetailedTagInformation.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 06-01-11.
//  Copyright 2011 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "DetailedTagInformation.h"

#import "NedapHandheldAPI.h"

@implementation DetailedTagInformation

@synthesize mdidLabel, mdidStringLabel, modelNumberLabel, xtidLabel, modelNameLabel;

// The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
/*
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization.
    }
    return self;
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Tag Details",nil);
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	// new tags were found
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadRFIDTagInformation:) name:@"NHPresentationRFIDDidReadDetailedTagInformationNotification" object:nil];
	
	// prepare for single read
	[self startReadDetailedInformation];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark -
#pragma mark NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	[self startReadDetailedInformation];
}

- (void)didReadRFIDTagInformation:(NSNotification *)notification
{
	NSNumber *tagMDID = [[notification userInfo] valueForKey:@"tagMDID"];
	NSNumber *tagModelNumber = [[notification userInfo] valueForKey:@"tagModelNumber"];
	NSNumber *tagXTID = [[notification userInfo] valueForKey:@"tagXTID"];

	mdidLabel.text = [NSString stringWithFormat:@"%02X", [tagMDID intValue]];
	modelNumberLabel.text = [NSString stringWithFormat:@"%02X", [tagModelNumber intValue]];
	xtidLabel.text = [NSString stringWithFormat:@"%02X", [tagXTID intValue]];

    NSString *modelNumberName = @"unknown";
    
    switch([modelNumberLabel.text intValue]) {
        case 3:	modelNumberName = @"G2XM"; break;
        case 4:	modelNumberName = @"G2XL"; break;
        case 71:modelNumberName = @"Monza 2"; break;
        case 93:modelNumberName = @"Monza 3"; break;
        case 105:modelNumberName = @"Monza 4QT"; break;
//            case 10C	Monza 4E
        case 100:modelNumberName = @"Monza 4D"; break;
        case 130:modelNumberName = @"Monza 5"; break;
        case 411:modelNumberName = @"Higgs 2"; break;
        case 412:modelNumberName = @"Higgs 3"; break;
        case 805:modelNumberName = @"G2iL"; break;
        case 806:modelNumberName = @"G2iM"; break;
        default: break;
    }
    modelNameLabel.text = modelNumberName;
    
    switch ([mdidLabel.text intValue]) {
		case NHRFIDMDIDImpinj:             mdidStringLabel.text = @"Impinj"; break;
		case NHRFIDMDIDTexasInstruments:   mdidStringLabel.text = @"Texas Instruments"; break;
		case NHRFIDMDIDAlienTechnology:    mdidStringLabel.text = @"Alien Technology"; break;
		case NHRFIDMDIDIntelleflex:        mdidStringLabel.text = @"Intelleflex"; break;
		case NHRFIDMDIDAtmel:              mdidStringLabel.text = @"Atmel"; break;
		case NHRFIDMDIDNXP:                mdidStringLabel.text = @"NXP"; break;
		case NHRFIDMDIDSTMicroelectronics: mdidStringLabel.text = @"ST Mircoelectronics"; break;
		case NHRFIDMDIDEPMicroelectronics: mdidStringLabel.text = @"EP Mircoelectronics"; break;
		case NHRFIDMDIDMotorola:           mdidStringLabel.text = @"Motorola"; break;
		case NHRFIDMDIDSentechSndBhd:      mdidStringLabel.text = @"Sentech Snd Bhd"; break;
		case NHRFIDMDIDEMMicroelectronics: mdidStringLabel.text = @"EM Mircoelectronics"; break;
		case NHRFIDMDIDRenesasTechnology:  mdidStringLabel.text = @"Renesas Technology Corp."; break;
		case NHRFIDMDIDMstar:              mdidStringLabel.text = @"Mstar"; break;
		case NHRFIDMDIDTycoInternational:  mdidStringLabel.text = @"Tyco International"; break;
		case NHRFIDMDIDQuanrayElectronics: mdidStringLabel.text = @"Quanray Electronics"; break;
		case NHRFIDMDIDFujitsu:            mdidStringLabel.text = @"Fujitsu"; break;
		case NHRFIDMDIDLSIS:               mdidStringLabel.text = @"LSIS"; break;
		case NHRFIDMDIDCaenRFID:           mdidStringLabel.text = @"CAEN RFID srl"; break;
		case NHRFIDMDIDPEG:                mdidStringLabel.text = @"PEG"; break;
		default:                           mdidStringLabel.text = @"unknown"; break;
	}
	
	[self startReadDetailedInformation];
}

- (void)startReadDetailedInformation
{
	[NHPresentation rfidReadDetailedTagInformation];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations.
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end
