//
//  InventorySingleShotViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Wim Fikkert on 9/15/11.
//  Copyright 2011 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "InventorySingleShotViewController.h"
#import "NedapHandheldAPI.h"
#import "InventoryViewController.h"


#pragma mark - private class methods

@interface InventorySingleShotViewController ()
- (void) stopReading:(NSTimer *)theTimer;

- (void)didReadRFIDTagsFiltered:(NSNotification *)notification;
- (void)didConnect:(NSNotification *)notification;
- (void)didContinue:(NSNotification *)notification;
- (void)didPause:(NSNotification *)notification;
- (void)didClearInventory:(NSNotification *)notification;
@end


#pragma mark - Class implementation

@implementation InventorySingleShotViewController


#pragma mark - Synthesize variables

@synthesize epcCodes;
@synthesize inventoryRunning;
@synthesize tableViewInstance;
@synthesize activityIndicator;
@synthesize finishedReadTimestamp;
@synthesize finishedWriting;

#pragma mark - Memory management

- (void)dealloc
{
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}


#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
	// create an empty mutable tag array, used to show tags in the table view
	epcCodes = [[NSMutableArray alloc] initWithCapacity:10000];
	inventoryRunning = NO;
    finishedWriting = YES;
    self.title = NSLocalizedString(@"Single Shot",nil);

    // set clear button
    UIBarButtonItem *clearButton = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Clear",nil)
																	style:UIBarButtonItemStyleBordered
																   target:self 
																   action:@selector(clearButtonPush)]; 
    self.navigationItem.rightBarButtonItem = clearButton;
    self.navigationItem.rightBarButtonItem.enabled = NO;
    [clearButton release];
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
	// add activity indicator to the navigation bar
    //	activityIndicator = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0, 0, 20, 20)];
    //	activityIndicator.hidesWhenStopped = YES;
    //	UIBarButtonItem *barButton = [[UIBarButtonItem alloc] initWithCustomView:activityIndicator];
    //	self.navigationItem.rightBarButtonItem = barButton;
    //	[barButton release];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
	
	// connect with five notifications
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	// user presses button on device during inventory
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didPause:) name:@"NHPresentationHandheldDidPauseNotification" object:nil];
	
	// same, but for continue inventory
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didContinue:) name:@"NHPresentationHandheldDidContinueNotification" object:nil];
	
	// a new tag was found, filtering already took place
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadRFIDTagsFiltered:) name:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil];
	
	// clear the filter list, clear list here after this notification, to make sure both lists are consistent
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didClearInventory:) name:@"NHPresentationRFIDDidResetInventoryNotification" object:nil];
	
	// prepare for inventory, real inventory starts when user presses button on handheld
	[NHPresentation rfidInventoryHighVolumeStartInBuffer:@"singleShot"];
//	[NHPresentation rfidInventoryStart];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with the registered notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidPauseNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidContinueNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationRFIDDidResetInventoryNotification" object:nil];

    // do not forget to clear the Handheld's read buffer, otherwise it will remember previously seen tags
    [NHPresentation rfidInventoryReset];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [NSString stringWithFormat:@"%@ %i", NSLocalizedString(@"Read EPCs:",nil), [epcCodes count]];
}

- (CGFloat) tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
	return 40;
}

- (UIView *) tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
	NSString *sectionTitle = [self tableView:tableView titleForHeaderInSection:section];
    if (sectionTitle == nil) {
        return nil;
    }
	
    // create the parent view
	UIView * headerView = [[UIView alloc] initWithFrame:CGRectMake(0.0f, -5.0f, self.tableView.frame.size.width, [self tableView:tableView heightForHeaderInSection:section])];
	headerView.backgroundColor = [UIColor darkGrayColor];
	headerView.alpha = .85f;
	
	// create the label
	UILabel * headerLabel = [[UILabel alloc] initWithFrame:CGRectMake(20, 10, headerView.frame.size.width-40, 20)];
	headerLabel.backgroundColor = [UIColor clearColor];
	headerLabel.opaque = NO;
	headerLabel.textColor = [UIColor whiteColor];
	headerLabel.font = [UIFont fontWithName:@"Verdana-Bold" size:15];
	headerLabel.text = sectionTitle; 
    headerLabel.textAlignment = UITextAlignmentCenter;
	headerLabel.minimumFontSize = 11.0f;
	
	[headerView addSubview:headerLabel];
	[headerLabel release];
	return [headerView autorelease];
}


// Customize the number of sections in the table view.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [epcCodes count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
	// Configure the cell.
	NSDictionary *epcDictionary = [epcCodes objectAtIndex:indexPath.row];
    EPCCode *code = [epcDictionary valueForKey:@"code"];
	cell.textLabel.text = code.hexString;
	cell.textLabel.font = [UIFont fontWithName:@"Verdana" size:15];
 
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    /*
     <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
     [self.navigationController pushViewController:detailViewController animated:YES];
     [detailViewController release];
     */
}


#pragma mark - Actions


- (void)clearButtonPush
{
    [NHPresentation rfidInventoryReset];
}

- (void) stopReading:(NSTimer *)theTimer
{
    // if the !D Hand is connected
    if ([NHPresentation handheldIsConnected])
    {
        // stop reading
        [NHPresentation handheldStop];
        
        // set the action to perform when the user clicks on the !D Hand button
        [NHPresentation rfidInventoryHighVolumeStartInBuffer:@"singleShot"];
//        [NHPresentation rfidInventoryStart];
    }

    // get timestamp for "now"
    // NOTE: retain this object for now. This ensures it will live untill we have saved the epcCodes
    finishedReadTimestamp = [[NSDate dateWithTimeIntervalSinceReferenceDate:[NSDate timeIntervalSinceReferenceDate]] retain];
    NSLog(@"stopped reading @ %@", finishedReadTimestamp);

	// update table view - table title now shows the datetime when we finished reading
	[self.tableView reloadData];
	
    // store the read EPCs to file that can be synced using iTunes
    // NOTE: wait some extra time to ensure that all EPCs have been processed 
    [InventoryViewController writeEPCs:epcCodes toFile:@"highvolume.read.complete"];
}

#pragma mark - Notification handlers

- (void)didConnect:(NSNotification *)notification
{
	// if the handheld reconnects, start inventory again
	[NHPresentation rfidInventoryHighVolumeStartInBuffer:@"singleShot"];
//	[NHPresentation rfidInventoryStart];
}

- (void)didPause:(NSNotification *)notification
{
	[activityIndicator stopAnimating];
}

- (void)didContinue:(NSNotification *)notification
{
    // When we are not yet finished writing our previous log, alert the user and do not start reading
    if (!finishedWriting)
    {
        NSLog(@"%@:didContinue: not yet finished writing previous log, alert user", [self class]);
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Logging" message:@"Please wait for EPC logging to finish." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        [alert release];      

        [self stopReading:nil];
        return;
    }
    finishedWriting = NO;
	[activityIndicator startAnimating];
    
    // get timestamp for "now", also format it to fit the current timezone
    NSDate *timestamp = [NSDate dateWithTimeIntervalSinceReferenceDate:[NSDate timeIntervalSinceReferenceDate]];
	NSDateFormatter *dateFormatter = [[[NSDateFormatter alloc] init] autorelease];
	[dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
	NSString *formattedDate = [dateFormatter stringFromDate:timestamp];

    NSLog(@"starting read @ %@", formattedDate);
    
    // read for 2 seconds, stop the handheld reader
    [NSTimer scheduledTimerWithTimeInterval:2.3 target:self selector:@selector(stopReading:) userInfo:NULL repeats:NO];

    // clear the inventory when starting a new read cycle
    [self clearButtonPush];
}

- (void)didReadRFIDTagsFiltered:(NSNotification *)notification
{  
	NSArray *epcObservations = [[notification userInfo] valueForKey:@"epcObservations"];
	
    // get timestamp for "now", also format it to fit the current timezone
    NSDate *timestamp = [NSDate dateWithTimeIntervalSinceReferenceDate:[NSDate timeIntervalSinceReferenceDate]];
	NSDateFormatter *dateFormatter = [[[NSDateFormatter alloc] init] autorelease];
	[dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
	NSString *formattedDate = [dateFormatter stringFromDate:timestamp];

    for (NHEPCObservation *epcObservation in epcObservations)
    {      
        EPCCode *epcCode = epcObservation.epcCode;
        NSString *rssi = [NSString stringWithFormat:@"%i",epcObservation.rssi];
        NSLog(@"read EPC %@ @ %@ with %@", epcCode.pureIdentityURI, formattedDate, rssi);
        
        NSDictionary *epcDictionary = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:epcCode,formattedDate,rssi,nil] 
                                                                  forKeys:[NSArray arrayWithObjects:@"code",@"timestamp",@"rssi",nil]];
        
        // add epcCode to list of read EPCs
        [epcCodes addObject:epcDictionary];
    }
    
	// update table view
	[self.tableView reloadData];
}

- (void)didClearInventory:(NSNotification *)notification
{
	// clear array
	[epcCodes removeAllObjects];

	// update table view
	[self.tableView reloadData];

	// There are 0 read EPCs, clearing makes no sense
    self.navigationItem.rightBarButtonItem.enabled  = NO;
}


@end
