//
//  RootViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 03-10-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "InventoryViewController.h"

#import "NedapHandheldAPI.h"
#import "EPCGlobalTDS.h"
#import "WriteRFIDViewController.h"
#import "PurpleAlertView.h"

//#define DEVELOPMENT_TOOLS
#ifdef DEVELOPMENT_TOOLS
#  warning [WARNING] DEVELOPMENT_TOOLS is enabled, ONLY do this for testing purposes! 
#endif
#define GENERATE_EPC_COUNT 50000

@interface InventoryViewController()
- (void) segmentedControlHasChangedValue:(NSNotification *)notification;
#ifdef DEVELOPMENT_TOOLS
- (void) loadScan;
#endif
@end

@implementation InventoryViewController

@synthesize epcCodes;
@synthesize inventoryRunning;
@synthesize tableViewInstance;
@synthesize activityIndicator;

#pragma mark -
#pragma mark View lifecycle

#ifdef DEVELOPMENT_TOOLS
- (void) updateUI:(NSArray *)epcObservations {
    NSDictionary *userInfo = [NSDictionary dictionaryWithObject:epcObservations forKey:@"epcObservations"];
    NSNotification *notify = [NSNotification notificationWithName:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil userInfo:userInfo];
    [self didReadRFIDTagsFiltered:notify];
}

- (void) loadScan {
    NSDate *now = [NSDate date];
	NSMutableArray *epcObservations = [NSMutableArray arrayWithCapacity:GENERATE_EPC_COUNT];
    NSUInteger hashZeroCount = 0;
    NSUInteger hashExisted = 0;
    NSLog(@"max NSUInteger value = %u", NSNotFound);
    
    for (int i = 0; i < GENERATE_EPC_COUNT; i++) {
        @autoreleasepool {
            NSString *gtin = [NSString stringWithFormat:@"%llu",(arc4random() % 10000000000000)];
            while ([gtin length] < 14) {
                gtin = [@"0" stringByAppendingString:gtin];
            }
            
            unsigned long long randomNumber = arc4random() & 0x1FFFFFFFFF;
            randomNumber |= (1ULL << 37);
            NSNumber *serial = [NSNumber numberWithUnsignedLongLong:randomNumber];
            
            EPCCode *epc = [EPCCode codeWithGTIN14:gtin // arbitrary EPC
                                      serialNumber:serial
                               companyPrefixLength:7];
            if ([epc hash] == 0) {
                hashZeroCount++;
            }
            NSIndexSet *hashes = [NHPresentation rfidInventoryBuffer];
            if ([hashes containsIndex:[epc hash]]) {
                hashExisted++;
                NSLog(@"hash %u|%@ existed:",[epc hash], [epc pureIdentityURI]);
                for (NSDictionary *epcDict in epcCodes) {
                    EPCCode *tmpEpc = (EPCCode *)[epcDict objectForKey:@"code"];
                    if ([tmpEpc hash] == [epc hash]) {
                        NSLog(@" -- existing hash %u|%@:",[tmpEpc hash], [tmpEpc pureIdentityURI]);
                    }
                }
            }

            NHEPCObservation *ob = [[NHEPCObservation alloc] initWithEPCCode:epc timeStamp:now rssi:(arc4random() % 100) readCount:1];
            [epcObservations addObject:ob];
            [NHPresentation addEPC:epc];
            
            if (i % 2500 == 0) {
                NSLog(@"generated %u EPCs: #%u = %u|%@",i,i,[epc hash],epc.pureIdentityURI);
                [self updateUI:[epcObservations copy]];
                [epcObservations removeAllObjects];
            }
        }
    }
    [self updateUI:[epcObservations copy]];

    NSIndexSet *hashes = [NHPresentation rfidInventoryBuffer];
    NSLog(@"added %u EPCs with %u hashes: %u zero, %u existed",[epcCodes count],[hashes count], hashZeroCount, hashExisted);

    [tableViewInstance reloadData];
}
#endif

- (void)viewDidLoad {
    [super viewDidLoad];

	// create an empty mutable tag array, used to show tags in the table view
	epcCodes = [[NSMutableArray alloc] initWithCapacity:10000];
	inventoryRunning = NO;
    self.title = NSLocalizedString(@"Normal",nil);
    
    // set toggle button
    segmentedControl = [[UISegmentedControl alloc] initWithItems:
                                            [NSArray arrayWithObjects:
                                             [NSString stringWithString:NSLocalizedString(@"HEX", @"")],
                                             [NSString stringWithString:NSLocalizedString(@"URI", @"")],
                                             nil]];
    segmentedControl.segmentedControlStyle = UISegmentedControlStyleBar;
    segmentedControl.tintColor = [UIColor  colorWithRed:.776 green:.086 blue:.552 alpha:1.0];
    [segmentedControl setSelectedSegmentIndex:0];
    [segmentedControl addTarget:self action:@selector(segmentedControlHasChangedValue:) forControlEvents:UIControlEventValueChanged];
    CGRect navBounds = [self.navigationController.navigationBar bounds];
    [segmentedControl setFrame:CGRectMake(navBounds.size.width-85.0f, 5.0f, 80.0f, navBounds.size.height-10.0f)];
    [self.navigationController.navigationBar addSubview:segmentedControl];
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;

	// add activity indicator to the navigation bar
//	activityIndicator = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0, 0, 20, 20)];
//	activityIndicator.hidesWhenStopped = YES;
//	UIBarButtonItem *barButton = [[UIBarButtonItem alloc] initWithCustomView:activityIndicator];
//	self.navigationItem.rightBarButtonItem = barButton;
//	[barButton release];
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
	
	// connect with five notifications
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	// user presses button on device during inventory
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didPause:) name:@"NHPresentationHandheldDidPauseNotification" object:nil];
	
	// same, but for continue inventory
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didContinue:) name:@"NHPresentationHandheldDidContinueNotification" object:nil];
	
	// a new tag was found, filtering already took place
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadRFIDTagsFiltered:) name:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil];
	
	// clear the filter list, clear list here after this notification, to make sure both lists are consistent
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didClearInventory:) name:@"NHPresentationRFIDDidResetInventoryNotification" object:nil];
	
	// prepare for inventory, real inventory starts when user presses button on handheld
	[NHPresentation rfidInventoryStart];

#ifdef DEVELOPMENT_TOOLS
    [self loadScan];
#endif
    
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with the registered notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidPauseNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidContinueNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationRFIDDidResetInventoryNotification" object:nil];

    // store the read EPCs to file that can be synced using iTunes
    // NOTE: wait some extra time to ensure that all EPCs have been processed 
    [InventoryViewController writeEPCs:epcCodes toFile:@"normal.read.complete"];

    // do not forget to clear the Handheld's read buffer, otherwise it will remember previously seen tags
    [NHPresentation rfidInventoryReset];
    [segmentedControl removeFromSuperview];
}


#pragma mark - NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	// if the handheld reconnects, start inventory again
	[NHPresentation rfidInventoryStart];
}

- (void)didPause:(NSNotification *)notification
{
	[activityIndicator stopAnimating];

    // store the read EPCs to file that can be synced using iTunes
    // NOTE: wait some extra time to ensure that all EPCs have been processed 
//    [NSTimer scheduledTimerWithTimeInterval:3 target:self selector:@selector(writeEPCs) userInfo:NULL repeats:NO];
}

- (void)didContinue:(NSNotification *)notification
{
	[activityIndicator startAnimating];
}

- (void)didReadRFIDTagsFiltered:(NSNotification *)notification
{
    @autoreleasepool {
        NSArray *epcObservations = [[notification userInfo] objectForKey:@"epcObservations"];
        
        // get timestamp for "now", also format it to fit the current timezone
        NSDate *timestamp = [NSDate dateWithTimeIntervalSinceReferenceDate:[NSDate timeIntervalSinceReferenceDate]];
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
        NSString *formattedDate = [dateFormatter stringFromDate:timestamp];
        [dateFormatter release];
        
        for (NHEPCObservation *epcObservation in epcObservations) {
            @autoreleasepool {
                EPCCode *epcCode = epcObservation.epcCode;
                NSDictionary *epcDictionary = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:epcCode,formattedDate,[NSNumber numberWithUnsignedInteger:epcObservation.rssi],nil]
                                                                          forKeys:[NSArray arrayWithObjects:@"code",@"timestamp",@"rssi",nil]];
                
                // add epcCode to list of read EPCs
                [epcCodes addObject:epcDictionary];
            }
        }
        
        if ([epcCodes count] > 0)
            self.navigationItem.rightBarButtonItem.enabled  = YES;

        // update table view
        [tableViewInstance reloadData];
    }
}

- (void)didClearInventory:(NSNotification *)notification
{
	// clear array
	[epcCodes removeAllObjects];
	
	// update table view
	[tableViewInstance reloadData];
	
    self.navigationItem.rightBarButtonItem.enabled  = NO;
}

#pragma mark -
#pragma mark Actions

- (void) writeEPCs {
    [InventoryViewController writeEPCs:epcCodes toFile:@"normal.read.pause"];
}


+ (void) writeEPCs:(NSArray *)epcs toFile:(NSString *)description {
    if ([epcs count] == 0)
    {
        NSLog(@"%@:writeEPCs: no EPCs to write, aborting write",[self class]);
        return;
    }
    
    NSArray *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [path objectAtIndex:0];
    
    NSDate *timestamp = [NSDate dateWithTimeIntervalSinceReferenceDate:[NSDate timeIntervalSinceReferenceDate]];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	[dateFormatter setDateFormat:@"yyyyMMdd.HHmmss"];
	NSString *formattedDate = [dateFormatter stringFromDate:timestamp];
	[dateFormatter release];
    
    NSString *fileName = [NSString stringWithFormat:@"NEDAP.%@.%@.csv",description,formattedDate];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
    
    [[NSFileManager defaultManager] createFileAtPath:filePath contents:nil attributes:nil];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath:filePath];
    
    __block PurpleAlertView *progressAlert;
    dispatch_async(dispatch_get_main_queue(), ^{
        progressAlert = [[PurpleAlertView alloc] initWithTitle:NSLocalizedString(@"Writing",nil)
                                                       message:NSLocalizedString(@"Please wait",nil)
                                                      delegate:self
                                             cancelButtonTitle:nil
                                                         image:BUSY_ALERT_IMAGE
                                             otherButtonTitles:nil];
        [progressAlert show];
    });
    
    //    if (fileHandle)
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, NULL), ^{
        [fileHandle writeData:[@"SGTIN,hexString,timestamp,RSSI\n" dataUsingEncoding:NSUTF8StringEncoding]];
        __block NSString *epcString = nil;
        [epcs enumerateObjectsUsingBlock:^(id epcDict, NSUInteger index, BOOL *stop){
            NSDictionary *epcData = (NSDictionary *)epcDict;
            @autoreleasepool {
                EPCCode *epcCode = [epcData objectForKey:@"code"];
                NSString *epcTimestamp = [epcData valueForKey:@"timestamp"];
                NSNumber *rssi = [epcData objectForKey:@"rssi"];
                epcString = [NSString stringWithFormat:@"%@,%@,%@,%u\n",epcCode.pureIdentityURI,epcCode.hexString,epcTimestamp,[rssi unsignedIntegerValue]];
                epcTimestamp = nil;
                rssi = nil;
                
                NSData *stringData = [epcString dataUsingEncoding:NSUTF8StringEncoding];
                epcString = nil;

                [fileHandle writeData:stringData];
                stringData = nil;
            }
            if (index % 25 == 0) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    if (progressAlert != nil) {
                        double progress = ceil(100.0*(double)index/(double)[epcs count]);
                        [progressAlert setMessage:[NSString stringWithFormat:@"%@ %g%%",NSLocalizedString(@"written", nil),progress]];
                    }
                });
            }
        }];
        dispatch_async(dispatch_get_main_queue(), ^{
            [progressAlert dismissWithClickedButtonIndex:0 animated:NO];
        });
        [fileHandle closeFile];
    });
}

/*
- (void)pause
{
	if (inventoryRunning)
	{
		inventoryRunning = NO;
		[NHPresentation rfidInventoryPause];
	}
	else 
	{
		inventoryRunning = YES;
		[NHPresentation rfidInventoryContinue];
	}
}
 */

- (void) segmentedControlHasChangedValue:(NSNotification *)notification {
	[tableViewInstance reloadData];
}

- (void)clearButtonPush
{
   [NHPresentation rfidInventoryReset];
}

#pragma mark -
#pragma mark Table view data source

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [NSString stringWithFormat:@"%@ %i", NSLocalizedString(@"Read EPCs:",nil),[epcCodes count]];
}

- (CGFloat) tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
	return 40;
}

- (UIView *) tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
	NSString *sectionTitle = [self tableView:tableView titleForHeaderInSection:section];
    if (sectionTitle == nil) {
        return nil;
    }
	
    // create the parent view
	UIView * headerView = [[UIView alloc] initWithFrame:CGRectMake(0.0f, -5.0f, tableViewInstance.frame.size.width, [self tableView:tableViewInstance heightForHeaderInSection:section])];
	headerView.backgroundColor = [UIColor darkGrayColor];
	headerView.alpha = .85f;
	
	// create the label
	UILabel * headerLabel = [[UILabel alloc] initWithFrame:CGRectMake(20, 10, headerView.frame.size.width-40, 20)];
	headerLabel.backgroundColor = [UIColor clearColor];
	headerLabel.opaque = NO;
	headerLabel.textColor = [UIColor whiteColor];
	headerLabel.font = [UIFont fontWithName:@"Verdana-Bold" size:15];
	headerLabel.text = sectionTitle; 
    headerLabel.textAlignment = UITextAlignmentCenter;
	headerLabel.minimumFontSize = 11.0f;
	
	[headerView addSubview:headerLabel];
	[headerLabel release];
	return [headerView autorelease];
}

// Customize the number of sections in the table view.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [epcCodes count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        cell.textLabel.font = [UIFont fontWithName:@"Verdana" size:15];
        [cell.textLabel setAdjustsFontSizeToFitWidth:YES];
    }
    
	// Configure the cell.
	NSDictionary *epcDictionary = [epcCodes objectAtIndex:indexPath.row];
    EPCCode *code = [epcDictionary objectForKey:@"code"];
    
    if (segmentedControl.selectedSegmentIndex == 0) {
        cell.textLabel.text = code.hexString;
    } 
    else {
        cell.textLabel.text = code.pureIdentityURI;
    }
	
    return cell;
}


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
//    NSLog(@"selected %@, pushing with %@", indexPath, self.navigationController);
    WriteRFIDViewController *writeVC = [[WriteRFIDViewController alloc] initWithNibName:@"WriteRFIDViewController" bundle:nil];
    
    NSDictionary *epcDictionary = [epcCodes objectAtIndex:indexPath.row];
    
    writeVC.epcCode = [epcDictionary objectForKey:@"code"];
    [self.navigationController pushViewController:writeVC animated:YES];
    [writeVC release];
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end

