//
//  InventoryVolumeViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 03-10-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "InventoryVolumeViewController.h"

#import "NedapHandheldAPI.h"
#import "EPCGlobalTDS.h"
#import "WriteRFIDViewController.h"
#import "InventoryViewController.h"

@interface InventoryVolumeViewController()
- (void) segmentedControlHasChangedValue:(NSNotification *)notification;
@end


@implementation InventoryVolumeViewController

@synthesize epcCodes;
@synthesize inventoryRunning;
@synthesize tableViewInstance;
@synthesize activityIndicator;

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];

	// create an empty mutable tag array, used to show tags in the table view
	epcCodes = [[NSMutableArray alloc] initWithCapacity:10000];
	inventoryRunning = NO;

	// add clear button to the navigation bar
    self.title = NSLocalizedString(@"High Volume",nil);

    // set toggle button
    segmentedControl = [[UISegmentedControl alloc] initWithItems:
                        [NSArray arrayWithObjects:
                         [NSString stringWithString:NSLocalizedString(@"HEX", @"")],
                         [NSString stringWithString:NSLocalizedString(@"URI", @"")],
                         nil]];
    segmentedControl.segmentedControlStyle = UISegmentedControlStyleBar;
    segmentedControl.tintColor = [UIColor  colorWithRed:.776 green:.086 blue:.552 alpha:1.0];
    [segmentedControl setSelectedSegmentIndex:0];
    [segmentedControl addTarget:self action:@selector(segmentedControlHasChangedValue:) forControlEvents:UIControlEventValueChanged];
    CGRect navBounds = [self.navigationController.navigationBar bounds];
    [segmentedControl setFrame:CGRectMake(navBounds.size.width-85.0f, 5.0f, 80.0f, navBounds.size.height-10.0f)];
    [self.navigationController.navigationBar addSubview:segmentedControl];

	// add activity indicator to the navigation bar
//	activityIndicator = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0, 0, 20, 20)];
//	activityIndicator.hidesWhenStopped = YES;
//	UIBarButtonItem *barButton = [[UIBarButtonItem alloc] initWithCustomView:activityIndicator];
//	navItem.rightBarButtonItem = barButton;
//	[barButton release];
	
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

/*
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}
*/

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
	
	// connect with five notifications
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	// user presses button on device during inventory
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didPause:) name:@"NHPresentationHandheldDidPauseNotification" object:nil];
	
	// same, but for continue inventory
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didContinue:) name:@"NHPresentationHandheldDidContinueNotification" object:nil];
	
	// a new tag was found, filtering already took place
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadRFIDTagsFiltered:) name:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil];
	
	// clear the filter list, clear list here after this notification, to make sure both lists are consistent
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didClearInventory:) name:@"NHPresentationRFIDDidResetInventoryNotification" object:nil];
	
	// prepare for inventory, real inventory starts when user presses button on handheld
	[NHPresentation rfidInventoryHighVolumeStartInBuffer:@"highVolume"];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with the registered notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidPauseNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationHandheldDidContinueNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationRFIDDidReadTagsFilteredNotification" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"NHPresentationRFIDDidResetInventoryNotification" object:nil];

    // store the read EPCs to file that can be synced using iTunes
    // NOTE: wait some extra time to ensure that all EPCs have been processed 
    [InventoryViewController writeEPCs:epcCodes toFile:@"highvolume.read.complete"];

    [segmentedControl removeFromSuperview];

    // do not forget to clear the Handheld's read buffer, otherwise it will remember previously seen tags
    [NHPresentation rfidInventoryReset];
}


/*
- (void)viewDidDisappear:(BOOL)animated {
	[super viewDidDisappear:animated];
}
*/

/*
 // Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations.
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
 */

#pragma mark -
#pragma mark NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	// if the handheld reconnects, start inventory again
	[NHPresentation rfidInventoryHighVolumeStartInBuffer:@"highVolume"];
}

- (void)didPause:(NSNotification *)notification
{
	[activityIndicator stopAnimating];
}

- (void)didContinue:(NSNotification *)notification
{
	[activityIndicator startAnimating];
}

- (void)didReadRFIDTagsFiltered:(NSNotification *)notification
{
	NSArray *epcObservations = [[notification userInfo] valueForKey:@"epcObservations"];
	
    // get timestamp for "now", also format it to fit the current timezone
    NSDate *timestamp = [NSDate dateWithTimeIntervalSinceReferenceDate:[NSDate timeIntervalSinceReferenceDate]];
	NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
	[dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
	NSString *formattedDate = [dateFormatter stringFromDate:timestamp];
	[dateFormatter release];

    for (NHEPCObservation *epcObservation in epcObservations)
    {
        EPCCode *epcCode = epcObservation.epcCode;
        NSDictionary *epcDictionary = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:epcCode,formattedDate,[NSNumber numberWithUnsignedInteger:epcObservation.rssi],nil]
                                                                  forKeys:[NSArray arrayWithObjects:@"code",@"timestamp",@"rssi",nil]];
        
        // add epcCode to list of read EPCs
        [epcCodes addObject:epcDictionary];
    }
    
    if ([epcCodes count] > 0)
        self.navigationItem.rightBarButtonItem.enabled  = YES;
    
	// update table view
	[tableViewInstance reloadData];
}


- (void)didClearInventory:(NSNotification *)notification
{
	// clear array
	[epcCodes removeAllObjects];
	
	// update table view
	[tableViewInstance reloadData];

    self.navigationItem.rightBarButtonItem.enabled  = NO;
}

- (void) segmentedControlHasChangedValue:(NSNotification *)notification {
    [self.tableViewInstance reloadData];
}

#pragma mark -
#pragma mark Actions

/*
- (void)pause
{
	if (inventoryRunning)
	{
		inventoryRunning = NO;
		[NHPresentation rfidInventoryPause];
	}
	else 
	{
		inventoryRunning = YES;
		[NHPresentation rfidInventoryContinue];
	}
}
*/

- (void)clearButtonPush
{
	[NHPresentation rfidInventoryReset];
}

#pragma mark -
#pragma mark Table view data source

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    return [NSString stringWithFormat:@"%@ %i", NSLocalizedString(@"Read EPCs:",nil), [epcCodes count]];
}

- (CGFloat) tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
	return 40;
}

- (UIView *) tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {
	NSString *sectionTitle = [self tableView:tableView titleForHeaderInSection:section];
    if (sectionTitle == nil) {
        return nil;
    }
	
    // create the parent view
	UIView * headerView = [[UIView alloc] initWithFrame:CGRectMake(0.0f, -5.0f, tableViewInstance.frame.size.width, [self tableView:tableViewInstance heightForHeaderInSection:section])];
	headerView.backgroundColor = [UIColor darkGrayColor];
	headerView.alpha = .85f;
	
	// create the label
	UILabel * headerLabel = [[UILabel alloc] initWithFrame:CGRectMake(20, 10, headerView.frame.size.width-40, 20)];
	headerLabel.backgroundColor = [UIColor clearColor];
	headerLabel.opaque = NO;
	headerLabel.textColor = [UIColor whiteColor];
	headerLabel.font = [UIFont fontWithName:@"Verdana-Bold" size:15];
	headerLabel.text = sectionTitle; 
    headerLabel.textAlignment = UITextAlignmentCenter;
	headerLabel.minimumFontSize = 11.0f;
	
	[headerView addSubview:headerLabel];
	[headerLabel release];
	return [headerView autorelease];
}

// Customize the number of sections in the table view.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [epcCodes count];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        cell.textLabel.font = [UIFont fontWithName:@"Verdana" size:15];
        [cell.textLabel setAdjustsFontSizeToFitWidth:YES];
    }
    
	// Configure the cell.
	NSDictionary *epcDictionary = [epcCodes objectAtIndex:indexPath.row];
    EPCCode *code = [epcDictionary valueForKey:@"code"];
    if (segmentedControl.selectedSegmentIndex == 0) {
        cell.textLabel.text = code.hexString;
    } 
    else {
        cell.textLabel.text = code.pureIdentityURI;
    }
	
    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/


/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source.
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}
*/


/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/


/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    //    NSLog(@"selected %@, pushing with %@", indexPath, self.navigationController);
    WriteRFIDViewController *writeVC = [[WriteRFIDViewController alloc] initWithNibName:@"WriteRFIDViewController" bundle:nil];
    
    NSDictionary *epcDictionary = [epcCodes objectAtIndex:indexPath.row];
    
    writeVC.epcCode = [epcDictionary valueForKey:@"code"];
    [self.navigationController pushViewController:writeVC animated:YES];
    [writeVC release];
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end

