//
//  MonitorViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 10-10-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "MonitorViewController.h"

#import "NedapHandheldAPI.h"
#import "EPCGlobalTDS.h"
#import "MonitorCell.h"


@interface MonitorViewController (PrivateMethods)
- (void) performSearch:(UISearchBar *)searchBar;
@end

@implementation MonitorViewController

@synthesize tags;
@synthesize monitorRunning;
@synthesize tableViewInstance;
@synthesize searchBarInstance;
@synthesize searchPredicate;

bool shouldBeginEditing;


#pragma mark -
#pragma mark View lifecycle
- (void)viewDidLoad {
    [super viewDidLoad];
	
	// create an empty mutable tag array, used to show tags in the table view
	tags = [[NSMutableArray alloc] initWithCapacity:100];
	monitorRunning = NO;
    shouldBeginEditing = YES;
    searchPredicate = nil;
    
    searchBarInstance  = [[UISearchBar alloc] initWithFrame:CGRectMake(0,0,320,30)];
    searchBarInstance.delegate = self;
    searchBarInstance.autocorrectionType = UITextAutocorrectionTypeNo;
    [self.view addSubview:searchBarInstance];

    self.title = NSLocalizedString(@"Monitor",nil);
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;

    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

/*
 - (void)viewWillAppear:(BOOL)animated {
 [super viewWillAppear:animated];
 }
 */

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
	
	// connect with five notifications
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];

	// a new tag was found, filtering already took place
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadRFIDTags:) name:@"NHPresentationRFIDDidReadTagsNotification" object:nil];

	// prepare for inventory, real inventory starts when user presses button on handheld
	[NHPresentation rfidMonitorStart];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


/*
 - (void)viewDidDisappear:(BOOL)animated {
 [super viewDidDisappear:animated];
 }
 */

/*
 // Override to allow orientations other than the default portrait orientation.
 - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
 // Return YES for supported orientations.
 return (interfaceOrientation == UIInterfaceOrientationPortrait);
 }
 */

#pragma mark -
#pragma mark NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	// if the handheld reconnects, start inventory again
	[NHPresentation rfidMonitorStart];
}

- (void)didReadRFIDTags:(NSNotification *)notification
{
	[tags removeAllObjects];
	
	NSArray *epcObservations = [[notification userInfo] valueForKey:@"epcObservations"];
	[epcObservations retain];

	for (NHEPCObservation *epcObservation in epcObservations)
	{
		[tags addObject:epcObservation];
        
        if (searchPredicate != nil)
        {
            if([epcObservation.epcCode.hexString caseInsensitiveCompare:self.searchBarInstance.text] == NSOrderedSame)
            {
                [NHPresentation uiPlayTune:NHUITuneShort];
                [NHPresentation uiVibrateTune:NHUITuneShort];
            }
        }
	}
    
    if (searchPredicate != nil)
    {
        [tags filterUsingPredicate:searchPredicate];
    }

    if ([epcObservations count] > 0) {
        if ([NHPresentation handheldIsConnected]) {
            [NHPresentation actionUITune:NHUITuneShort];
        }
    }

	[epcObservations release];
	
	[tableViewInstance reloadData];
}

#pragma mark -
#pragma mark Table view data source

// Customize the number of sections in the table view.
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


// Customize the number of rows in the table view.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [tags count]+1;
}


- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section == 0 
        && indexPath.row == 0)
    {
        return 35.0f;
    }
    return 39.0f;
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"MonitorCellIdentifier";
    
    MonitorCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        NSArray *nibItems = [[NSBundle mainBundle] loadNibNamed:@"MonitorCell" owner:nil options:nil];
        for (id currObj in nibItems) {
            if ([currObj isKindOfClass:[UITableViewCell class]]) {
                cell = (MonitorCell *) currObj;
                break;
            }
        }
    }

    if (indexPath.section == 0
        && indexPath.row == 0) {
        cell.rssi.text = @"";
        cell.hex.text = @"";
        cell.sgtin.text = @"";
    }
    else {
        // Configure the cell.
        NHEPCObservation *epcObservation = [tags objectAtIndex:indexPath.row-1];
        EPCCode *tag = epcObservation.epcCode;
        cell.rssi.text = [NSString stringWithFormat:@"%i", epcObservation.rssi];
        cell.hex.text = tag.hexString;
        if (![tag.pureIdentityURI isEqualToString:tag.hexString]) {
            cell.sgtin.text = tag.pureIdentityURI;
        } else {
            cell.sgtin.text = @"";
        }
	}

    return cell;
}

/*
 // Override to support conditional editing of the table view.
 - (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the specified item to be editable.
 return YES;
 }
 */


/*
 // Override to support editing the table view.
 - (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
 
 if (editingStyle == UITableViewCellEditingStyleDelete) {
 // Delete the row from the data source.
 [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
 }   
 else if (editingStyle == UITableViewCellEditingStyleInsert) {
 // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
 }   
 }
 */


/*
 // Override to support rearranging the table view.
 - (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
 }
 */


/*
 // Override to support conditional rearranging of the table view.
 - (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
 // Return NO if you do not want the item to be re-orderable.
 return YES;
 }
 */


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
	/*
	 <#DetailViewController#> *detailViewController = [[<#DetailViewController#> alloc] initWithNibName:@"<#Nib name#>" bundle:nil];
     // ...
     // Pass the selected object to the new view controller.
	 [self.navigationController pushViewController:detailViewController animated:YES];
	 [detailViewController release];
	 */
}


#pragma mark -
#pragma mark UISearchBarDelegate

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{
#ifdef DEBUG
    NSLog(@"%@:textDidChange: changed searchText to \'%@\'", [self class],searchText);
#endif
    [self performSearch:searchBar];
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)bar {
    // reset the shouldBeginEditing BOOL ivar to YES, but first take its value and use it to return it from the method call
    BOOL boolToReturn = shouldBeginEditing;
    shouldBeginEditing = YES;
    return boolToReturn;
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar
{
    // only show the status bar’s cancel button while in edit mode
    searchBarInstance.showsCancelButton = YES;
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)searchBar
{
    searchBarInstance.showsCancelButton = NO;
}

- (void) searchBarCancelButtonClicked:(UISearchBar *)searchBar {
    [self performSearch:searchBar];
    [searchBarInstance resignFirstResponder]; 
}

- (void) performSearch:(UISearchBar *)searchBar {
    if (searchBar.text != nil
        && ![searchBar.text isEqualToString:@""])
    {
        self.searchPredicate = [NSPredicate predicateWithFormat:@"epcCode.hexString contains[cd] %@", searchBar.text];
    }
    else
    {
        self.searchPredicate = nil;
    }

#ifdef DEBUG
    NSLog(@"%@:performSearch: setting searchPredicate to %@",[self class], searchPredicate);
#endif
    
    [tableViewInstance reloadData];
}

- (void) searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    [self performSearch:searchBar];
    
    [searchBarInstance resignFirstResponder];   
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}

@end