//
//  NFCCardViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 29-12-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "NFCCardViewController.h"

#import "NedapHandheldAPI.h"
#import "EPCGlobalTDS.h"

@interface NFCCardViewController ()
- (void)clearButtonPush;
@end

@implementation NFCCardViewController

@synthesize cardSerialNumberLabel, clearButton;

// The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
/*
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization.
    }
    return self;
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    
    // set clear button
    self.clearButton = [[UIBarButtonItem alloc] initWithTitle:NSLocalizedString(@"Clear",nil)
                                                        style:UIBarButtonItemStyleBordered
                                                       target:self 
                                                       action:@selector(clearButtonPush)]; 
    self.navigationItem.rightBarButtonItem = self.clearButton;
    self.navigationItem.rightBarButtonItem.enabled = NO;
    
    self.title = NSLocalizedString(@"NFC Card",nil);
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;   
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	// attach notification handler to the read card notification
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(nfcCardRead:) name:@"NHPresentationNFCDidReadCardNotification" object:nil];
	
	[NHPresentation nfcReadStart];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];

	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations.
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

#pragma mark -
#pragma mark NotificationHandlers

- (void)clearButtonPush
{
    if ([NHPresentation handheldIsConnected])
    {
        [NHPresentation handheldStop];
        [NHPresentation nfcReadStart];
    }
    cardSerialNumberLabel.text = @"...";
    self.navigationItem.rightBarButtonItem.enabled = NO;
}


- (void)didConnect:(NSNotification *)notification
{
	[NHPresentation nfcReadStart];
}

- (void)nfcCardRead:(NSNotification *)notification {
	NSNumber *cardNumber = [[notification userInfo] valueForKey:@"cardNumber"];
	
	cardSerialNumberLabel.text = [NSString stringWithFormat:@"%@", cardNumber];
    self.navigationItem.rightBarButtonItem.enabled = YES;
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end
