//
//  NedapHandheldAPIDemoAppDelegate.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 03-10-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "NedapHandheldAPIDemoAppDelegate.h"
#import "InventoryViewController.h"

#import "NedapHandheldAPI.h"

@implementation NedapHandheldAPIDemoAppDelegate

@synthesize window;
@synthesize tabBarController;

@synthesize connectedAlertView;
@synthesize batteryAlertView;

#pragma mark -
#pragma mark Application lifecycle

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {    
    // reactivate the status bar
	[application setStatusBarHidden:NO];
	[application setStatusBarStyle:UIStatusBarStyleBlackOpaque animated:YES];

	// Init handheld reader interface
	[NHPresentation self];
    //[NHPresentation rfidSetRegion:NHRFIDRegionEurope];
	
    if ([NHPresentation handheldIsConnected])
    {
        NSLog(@"Handheld is connected!");
    }
    else
    {
        NSLog(@"Handheld is NOT connected!");
    }
    
    // Add the tab bar controller's view to the window and display.
    [window addSubview:tabBarController.view];
    [window makeKeyAndVisible];
	
	[[UIApplication sharedApplication] setIdleTimerDisabled:YES];
	
	// receive notifications for connects and disconnects of handheld reader
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didDisconnect:) name:@"NHPresentationHandheldDidDisconnectNotification" object:nil];
	
	// receive notifications for low and empty battery notifications
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(lowBatteryWarning:) name:@"NHPresentationHandheldDidReceiveLowBatteryNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(emptyBattery:) name:@"NHPresentationHandheldDidReceiveEmptyBatteryNotification" object:nil];
		
    // set barcode read types to all on app start
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setValue:[NSNumber numberWithBool:YES] forKey:@"readAllBarcodes"];
    
	return YES;
}

- (void)didConnect:(NSNotification *)notification
{
    /*
	// if handheld connects again, dissmiss alert
	[connectedAlertView dismissWithClickedButtonIndex:[connectedAlertView cancelButtonIndex] animated:YES];
	*/
	// log serial number
	NSString *serialNumber = [[notification userInfo] valueForKey:@"serialNumber"];
	NSLog(@"Serial number: %@", serialNumber);
}

- (void)didDisconnect:(NSNotification *)notification
{
    /*
	// if the handheld disconnects, show alert
	connectedAlertView = [[UIAlertView alloc] initWithTitle:@"Disconnected" message:@"The !d hand disconnected." delegate:self cancelButtonTitle:@"Dismiss" otherButtonTitles:nil];
	[connectedAlertView show];
     */
}

- (void)lowBatteryWarning:(NSNotification *)notification
{
	// if the battery is low, show alert
	batteryAlertView = [[[UIAlertView alloc] initWithTitle:@"Low battery" message:@"The !d hand battery has 10% remaining." delegate:self cancelButtonTitle:@"Dismiss" otherButtonTitles:nil] autorelease];
	[batteryAlertView show];
}

- (void)emptyBattery:(NSNotification *)notification
{
	// if the battery is empty, show alert
	batteryAlertView = [[[UIAlertView alloc] initWithTitle:@"Empty battery" message:@"The !d hand battery is empty." delegate:self cancelButtonTitle:@"Dismiss" otherButtonTitles:nil] autorelease];
	[batteryAlertView show];
}

- (void)applicationWillResignActive:(UIApplication *)application {
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
}


- (void)applicationDidEnterBackground:(UIApplication *)application {
    /*
     Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
     If your application supports background execution, called instead of applicationWillTerminate: when the user quits.
     */
}


- (void)applicationWillEnterForeground:(UIApplication *)application {
    /*
     Called as part of  transition from the background to the inactive state: here you can undo many of the changes made on entering the background.
     */
}


- (void)applicationDidBecomeActive:(UIApplication *)application {
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
}


- (void)applicationWillTerminate:(UIApplication *)application {
	/*
     Called when the application is about to terminate.
     See also applicationDidEnterBackground:.
     */
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


#pragma mark -
#pragma mark Memory management

- (void)applicationDidReceiveMemoryWarning:(UIApplication *)application {
    /*
     Free up as much memory as possible by purging cached data objects that can be recreated (or reloaded from disk) later.
     */
}


- (void)dealloc {
	[tabBarController release];
	[window release];
	[super dealloc];
}


@end

