//
//  SelectInventoryMethod.m
//  NedapHandheldAPIDemo
//
//  Created by Wim Fikkert on 9/15/11.
//  Copyright 2011 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "SelectInventoryMethod.h"
#import "InventorySingleShotViewController.h"
#import "InventoryViewController.h"
#import "InventoryVolumeViewController.h"


#pragma mark - Class private methods

@interface SelectInventoryMethod()
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath;
- (NSString*) buildHTMLemailBody:(MFMailComposeViewController*)picker;
- (void) attachTxtDescriptions: (MFMailComposeViewController *)picker;
@end


#pragma mark - Class implementation

@implementation SelectInventoryMethod


#pragma mark - synthesize class variables

@synthesize menuOptions;
@synthesize inventoryFileName;


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}


#pragma mark - View lifecycle

- (void) initialize
{
    menuOptions = [[NSMutableArray alloc] init];
//   [menuOptions addObject:[NSDictionary dictionaryWithObjectsAndKeys:
//                            NSLocalizedString(@"Single shot (2 sec)",nil),@"name",
//                            @"inventory_methods-01",@"icon",nil]];
    [menuOptions addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                            NSLocalizedString(@"Normal",nil),@"name",
                            NSLocalizedString(@"Session 1",nil),@"details",
                            @"inventory_methods-02",@"icon",nil]];
    [menuOptions addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                            NSLocalizedString(@"High volume",nil),@"name",
                            NSLocalizedString(@"Session 2 and 3",nil),@"details",
                            @"inventory_methods-03",@"icon",nil]];
}

- (NSString *) latestInventoryFilePath {
    
    NSArray *path = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [path objectAtIndex:0];
    NSFileManager * fileManager = [NSFileManager new];
    NSArray * subpaths = [fileManager subpathsAtPath:documentsDirectory];

    NSLog(@"directory %@",documentsDirectory);
    NSString *fileName = nil;
    
    // create date formatter
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyyMMdd.HHmmss"];
    NSTimeZone *gmt = [NSTimeZone timeZoneWithAbbreviation:@"GMT"];
    [dateFormatter setTimeZone:gmt];
    NSDate *lastDate = nil;
    
    for (NSString *subpath in subpaths)
    {
        // if invalid length, stop now
        if ([subpath length] < 19)
        {
            NSLog(@"latestInventoryFilePath: [ERROR] path BREAK: %@", subpath);
            break;
        }
        
        // Example: "NEDAP.normal.read.complete.20120606.124406.csv" > "20120606.124406"
        NSString *fileDateDescr = [subpath substringWithRange:NSMakeRange([subpath length]-19, 15)];
        NSDate *formattedDate = [dateFormatter dateFromString:fileDateDescr];
#ifdef DEBUG
        NSLog(@"fileName %@ with date-part %@ has date %@", subpath, fileDateDescr, formattedDate);
#endif
        // if lastDate not yet initialized OR when the lastDate occured before the current date
        if (lastDate == nil
            || [lastDate compare:formattedDate] == NSOrderedAscending)
        {
#ifdef DEBUG
            NSLog(@"new lastDate %@ < %@", lastDate, formattedDate);
#endif
            lastDate = formattedDate;
            fileName = subpath;
        }
        
    }
    [dateFormatter release];

    NSString *filePath = nil;
    if (fileName != nil)
    {
        filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
#ifdef DEBUG
        NSLog(@"filePath %@", filePath);
#endif
    }
    return filePath;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

	[self initialize];

	[self.tableView setScrollEnabled:YES];
    self.title = NSLocalizedString(@"Inventory",nil);

    // initialize with latest file handle
    self.inventoryFileName = [self latestInventoryFilePath];
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    // update with latest file handle
    self.inventoryFileName = [self latestInventoryFilePath];
    
    [self.tableView reloadData];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


#pragma mark - Table view data source

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    if (section == 0)
        return NSLocalizedString(@"select method",nil);
    
    return nil;
}

- (CGFloat) tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
	return 40;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    if (self.inventoryFileName != nil)
        return 2;
    
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (section == 0)
    {
        // Return the number of rows in the section.
        return [menuOptions count];
    }
    else {
        return 1;
    }
}

/**
 * Configures the cell for each location, shows lcoation name and number of items there
 */ 
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath {
	
    if (indexPath.section == 0)
    {
        // get option to choose from
        NSDictionary *myMenuOption = [menuOptions objectAtIndex:indexPath.row];
        
        UIImage *tmpImg = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[myMenuOption valueForKey:@"icon"] ofType:@"png"]];;
        
        cell.imageView.image = tmpImg;
        cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
        
        cell.textLabel.text = [myMenuOption valueForKey:@"name"];
        cell.textLabel.textColor = [UIColor darkTextColor];

        cell.detailTextLabel.text = [myMenuOption valueForKey:@"details"];
        
        cell.backgroundColor = [UIColor whiteColor];
    }
    else {
        UIImage *tmpImg = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"icon_send_mail-07white" ofType:@"png"]];;
        cell.imageView.image = tmpImg;

        cell.textLabel.text = NSLocalizedString(@"E-mail Inventory",nil);
        cell.textLabel.textColor = [UIColor whiteColor];

        cell.backgroundColor = UICOLOR_PURPLE;
    }
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
        cell.textLabel.font = [UIFont fontWithName:@"Verdana" size:15];
        cell.detailTextLabel.font = [UIFont fontWithName:@"Verdana" size:11];
    }
    
    // Configure the cell...
    [self configureCell:cell atIndexPath:indexPath];
    
    return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return NO;
}

// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    return NO;
}


- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
    return UITableViewCellEditingStyleDelete;
}


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    // do nothing
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UIViewController *nextVC = nil;

//    // single shot read (for 2 seconds)
//    if (indexPath.row == 0)
//    {
//        nextVC = [[InventorySingleShotViewController alloc] init];
//    }

    if (indexPath.section == 0)
    {
        // normal inventory
        if (indexPath.row == 0)
        {
            nextVC = [[InventoryViewController alloc] init];
        }
        // high volume inventory
        else if (indexPath.row == 1)
        {
            nextVC = [[InventoryVolumeViewController alloc] init];
        }
        else 
        {
            NSLog(@"%@:didSelectRowAtIndexPath: %@ has no handler",[self class],indexPath);
            return;
        }

#ifdef DEBUG
        assert (nextVC != nil);
#endif
        
        // push the view controller onto the display
        [self.navigationController pushViewController:nextVC animated:YES];
        [nextVC release];
    }    
    else 
    {
        // send E-mail
        NSLog(@"sending E-mail");
        
        [self displayComposerSheet];
        
        // reload to remove row-selected status
        [self.tableView reloadData];
    }
}


/**
 * Displays an email composition interface inside the application. Populates all the Mail fields. 
 */
-(void) displayComposerSheet 
{
	MFMailComposeViewController *picker = [[MFMailComposeViewController alloc] init];
	
	// Check if client has E-mail configured
	if([MFMailComposeViewController canSendMail]) 
    {
		picker.mailComposeDelegate = self;
		
		picker.navigationBar.tintColor = UICOLOR_PURPLE;
		
        // get date from file name
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateFormat:@"yyyyMMdd.HHmmss"];
        NSTimeZone *gmt = [NSTimeZone timeZoneWithAbbreviation:@"GMT"];
        [dateFormatter setTimeZone:gmt];
        NSString *fileDateDescr = [self.inventoryFileName substringWithRange:NSMakeRange([self.inventoryFileName length]-19, 15)];
        NSDate *formattedDate = [dateFormatter dateFromString:fileDateDescr];
        [dateFormatter release];

        // format date to print in subject & mail body
        dateFormatter = [[NSDateFormatter alloc] init];
        [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
        gmt = [NSTimeZone timeZoneWithAbbreviation:@"GMT"];
        [dateFormatter setTimeZone:gmt];
        NSString *formattedTimestamp = [dateFormatter stringFromDate:formattedDate];
        [dateFormatter release];

		[picker setSubject:[NSString stringWithFormat:@"[NEDAP !D Hand] %@", formattedTimestamp]];
		
		// Set up recipients
        //		NSArray *toRecipients = [NSArray arrayWithObject:@"someone@somewhere.domain"]; 
		//	NSArray *ccRecipients = [NSArray arrayWithObject:@""]; 
		//	NSArray *bccRecipients = [NSArray arrayWithObject:@"wimfikkert@gmail.com"]; 
		
        //		[picker setToRecipients:toRecipients];
		//	[picker setCcRecipients:ccRecipients];	
		//	[picker setBccRecipients:bccRecipients];
		
		// Fill out the email body text
		NSString *emailBody = [NSString stringWithFormat:@"Inventory at %@\n\n%@",
							   formattedTimestamp, 
                               [self buildHTMLemailBody:picker]];
		[picker setMessageBody:emailBody isHTML:YES];
#ifdef DEBUG
		NSLog(@"%@:displayComposerSheet: emailbody set", [self class]);
		NSLog(@"%@:displayComposerSheet: picker attach: %@", [self class],picker);
#endif

		// attach CSV
		[self attachTxtDescriptions:picker];
		
#ifdef DEBUG
		NSLog(@"%@:displayComposerSheet: picker present: %@", [self class],picker);
#endif
		[self presentModalViewController:picker animated:YES];
		
#ifdef DEBUG
		NSLog(@"%@:displayComposerSheet: attachments attached", [self class]);
#endif		
	}
    [picker release];
}

/** 
 * Dismisses the email composition interface when users tap Cancel or Send. Proceeds to update the message field with the result of the operation.
 */ 
- (void)mailComposeController:(MFMailComposeViewController*)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError*)error 
{	
	// Notifies users about errors associated with the interface
#if DEBUG
	switch (result)
	{
		case MFMailComposeResultCancelled:
			NSLog(@"%@:mailComposeController: Result: canceled", [self class]);
			break;
		case MFMailComposeResultSaved:
			NSLog(@"%@:mailComposeController: Result: saved", [self class]);
			break;
		case MFMailComposeResultSent:
			NSLog(@"%@:mailComposeController: Result: sent", [self class]);
            break;
		case MFMailComposeResultFailed:
			NSLog(@"%@:mailComposeController: Result: failed", [self class]);
			break;
		default:
			NSLog(@"%@:mailComposeController: Result: not sent", [self class]);
			break;
	}
#endif
	[self dismissModalViewControllerAnimated:YES];
}

- (NSString*) buildHTMLemailBody:(MFMailComposeViewController*)picker
{
    NSString *emailBody = @"<h1>NEDAP !D Hand API demo</h1>";

//    emailBody = [emailBody stringByAppendingFormat:@"<table border=0 cellpadding=0 cellspacing=0 width=500>"];
    emailBody = [emailBody stringByAppendingFormat:@"<img width=500 src=\"http://blog.storeid.net/storage/store_id_mail_header-04.png\">"];
    
    return emailBody;
}

- (void) attachTxtDescriptions: (MFMailComposeViewController *)picker
{
    NSLog(@"attaching %@", self.inventoryFileName);
    NSArray *subs = [self.inventoryFileName componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"/"]];

    for (NSString *s in subs)
        NSLog(@"  %@",s);
    
    // get file contents
    NSData *dataToWrite = [NSData dataWithContentsOfFile:self.inventoryFileName];  
    
    // attach file's contents with same file name
    [picker addAttachmentData:dataToWrite mimeType:@"text/csv" fileName:[subs lastObject]];
}

@end
