//
//  TagReadViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 10-10-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "TagReadViewController.h"

#import "NedapHandheldAPI.h"
#import "EPCGlobalTDS.h"

@implementation TagReadViewController

@synthesize epcLabel;

/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Single Read",nil);
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
	
	// handheld connects, used to start operation again
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
	
	// new tags were found
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didReadRFIDTags:) name:@"NHPresentationRFIDDidReadTagsNotification" object:nil];

	// prepare for single read
	[self startSingleRead];
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

#pragma mark -
#pragma mark NotificationHandlers

- (void)didConnect:(NSNotification *)notification
{
	[self startSingleRead];
}

- (void)didReadRFIDTags:(NSNotification *)notification
{
	NSArray *epcObservations = [[notification userInfo] valueForKey:@"epcObservations"];
	[epcObservations retain];
	
	epcLabel.text = [NSString stringWithFormat:@"%i", [epcObservations count]];
	
	for (NHEPCObservation *epcObservation in epcObservations)
	{
		NSLog(@"epc: %@", epcObservation.epcCode.pureIdentityURI);
	}
	
	[epcObservations release];

	// prepare for another single read
	[self startSingleRead];
}

- (void)startSingleRead
{
	[NHPresentation rfidReadSingle];
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end
