//
//  UIAlertView+White.m
//  Cube
//
//  Created by Wim Fikkert on 11/4/11.
//  Copyright (c) 2011 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "PurpleAlertView.h"

@interface PurpleAlertView () {
    UIImageView *statusImageView;
}
- (void) drawRoundedRect:(CGRect)rect inContext:(CGContextRef)context withRadius:(CGFloat)radius;
@end

@implementation PurpleAlertView

static UIColor *fillColor = nil;
static UIColor *borderColor = nil;

#pragma mark - View lifecycle

- (void)setFillColor:(UIColor *)color {
    if (fillColor != nil) {
        fillColor = nil;
    }

    fillColor = color;
}


- (void)setBorderColor:(UIColor *)color {
    if (borderColor != nil) {
        borderColor = nil;
    }

    borderColor = color;
}


- (void)setBackgroundColor:(UIColor *)background withStrokeColor:(UIColor *)stroke {
    [self setFillColor:background];
    [self setBorderColor:stroke];
}


- (id)initWithFrame:(CGRect)frame {
    if ((self = [super initWithFrame:CGRectMake(frame.origin.x, frame.origin.y - 50.0f, frame.size.width, frame.size.height + 100.0f)])) {
        [self setBackgroundColor:[UIColor whiteColor] withStrokeColor:UICOLOR_PURPLE];
    }

    return self;
}


- (void)layoutSubviews {
    UILabel *title = [self valueForKey:@"_titleLabel"];
    title.font = [UIFont fontWithName:@"Verdana" size:16.0f];
    title.textColor = [UIColor whiteColor];
    title.shadowColor = [UIColor clearColor];
    title.shadowOffset = CGSizeMake(0, 0);

    for (UIView *sub in[self subviews]) {
        if ([sub class] == [UIImageView class] && sub.tag == 0) {
            [sub removeFromSuperview];
        }

        // Wow!
        NSString *name = NSStringFromClass ([sub class]);
        if ([name isEqualToString:@"UIAlertButton"]) {
            UIButton *isThisAButton = (UIButton *)sub;
            isThisAButton.titleLabel.font = [UIFont fontWithName:@"White" size:14.0f];
            [isThisAButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
            isThisAButton.titleLabel.shadowColor = [UIColor clearColor];
            isThisAButton.titleLabel.shadowOffset = CGSizeMake(0, 0);
        }
    }
}


- (void)drawRect:(CGRect)rect {
    CGContextRef context = UIGraphicsGetCurrentContext();

    CGContextClearRect(context, rect);
    CGContextSetAllowsAntialiasing(context, true);
    CGContextSetLineWidth(context, 0.0);
    CGContextSetAlpha(context, 0.8);
    CGContextSetLineWidth(context, 2.0);
    CGContextSetStrokeColorWithColor(context, [[UIColor whiteColor] CGColor]);
    CGContextSetFillColorWithColor(context, [UICOLOR_PURPLE CGColor]);

    // Draw background
    CGFloat backOffset = 2;
    CGRect backRect = CGRectMake(rect.origin.x + backOffset,
                                 rect.origin.y + backOffset,
                                 rect.size.width - backOffset * 2,
                                 rect.size.height - backOffset * 2);

    [self drawRoundedRect:backRect inContext:context withRadius:8];
    CGContextDrawPath(context, kCGPathFillStroke);

    // Clip Context
    CGRect clipRect = CGRectMake(backRect.origin.x + backOffset - 1,
                                 backRect.origin.y + backOffset - 1,
                                 backRect.size.width - (backOffset - 1) * 2,
                                 backRect.size.height - (backOffset - 1) * 2);

    [self drawRoundedRect:clipRect inContext:context withRadius:8];
    CGContextClip(context);

    //Draw highlight
    CGGradientRef glossGradient;
    CGColorSpaceRef rgbColorspace;
    size_t num_locations = 2;
    CGFloat locations[2] = { 0.0, 1.0 };
    CGFloat components[8] = { 1.0, 1.0, 1.0, 0.35, 1.0, 1.0, 1.0, 0.06 };
    rgbColorspace = CGColorSpaceCreateDeviceRGB();
    glossGradient = CGGradientCreateWithColorComponents(rgbColorspace,
                                                        components, locations, num_locations);

    CGRect ovalRect = CGRectMake(-130, -115, (rect.size.width * 2),
                                 rect.size.width / 2);

    CGPoint start = CGPointMake(rect.origin.x, rect.origin.y);
    CGPoint end = CGPointMake(rect.origin.x, rect.size.height / 5);

    CGContextSetAlpha(context, 1.0);
    CGContextAddEllipseInRect(context, ovalRect);
    CGContextClip(context);

    CGContextDrawLinearGradient(context, glossGradient, start, end, 0);

    CGGradientRelease(glossGradient);
    CGColorSpaceRelease(rgbColorspace);
}


- (void) drawRoundedRect:(CGRect)rrect inContext:(CGContextRef)context withRadius:(CGFloat)radius {
    CGContextBeginPath(context);

    CGFloat minx = CGRectGetMinX(rrect), midx = CGRectGetMidX(rrect),
            maxx = CGRectGetMaxX(rrect);

    CGFloat miny = CGRectGetMinY(rrect), midy = CGRectGetMidY(rrect),
            maxy = CGRectGetMaxY(rrect);

    CGContextMoveToPoint(context, minx, midy);
    CGContextAddArcToPoint(context, minx, miny, midx, miny, radius);
    CGContextAddArcToPoint(context, maxx, miny, maxx, midy, radius);
    CGContextAddArcToPoint(context, maxx, maxy, midx, maxy, radius);
    CGContextAddArcToPoint(context, minx, maxy, minx, midy, radius);
    CGContextClosePath(context);
}


- (id)initWithTitle:(NSString *)title message:(NSString *)message delegate:(id)delegate cancelButtonTitle:(NSString *)cancelButtonTitle image:(NSString *)statusImageName otherButtonTitles:(NSString *)otherButtonTitles, ...{
    NSString *titleWithExtraHeight = title;
    bool noButtons = cancelButtonTitle == nil && otherButtonTitles == nil;
    if (statusImageName != nil && !noButtons) { // do this here to create larger alertview
        if ([statusImageName isEqualToString:BUSY_ALERT_IMAGE]) {
            titleWithExtraHeight = [NSString stringWithFormat:@"\n\n\n%@", title];
        }
        else {
            titleWithExtraHeight = [NSString stringWithFormat:@"\n\n\n\n%@", title];
        }
    }

    if ([self initWithTitle:titleWithExtraHeight message:message delegate:delegate cancelButtonTitle:cancelButtonTitle otherButtonTitles:otherButtonTitles, nil]) {
        if (statusImageName != nil) {
            float yPos = (noButtons ? self.frame.size.height - 65.0f : 10.0f);

            CGSize labelSize = CGSizeMake(0.0f, 0.0f);
            if ([super.subviews count] > 2) {
                id subView = [super.subviews objectAtIndex:1];
                if (subView != nil
                    && [subView isKindOfClass:[UILabel class]])
                {
                    UILabel *messageLabel = (UILabel *)subView;
                    labelSize = [message sizeWithFont:[UIFont fontWithName:@"Verdana" size:16.0f]
                                    constrainedToSize:[messageLabel frame].size
                                        lineBreakMode:NSLineBreakByWordWrapping];
                    [messageLabel setFont:[UIFont fontWithName:@"Verdana" size:16.0f]];
                    [messageLabel setTextColor:[UIColor whiteColor]];
                    [messageLabel setShadowColor:[UIColor clearColor]];
                    [messageLabel setShadowOffset:CGSizeMake(0.f, 0.f)];
                }
            }

            if (message != nil) {
                yPos += 30.0f - labelSize.height;
            }

            statusImageView = [[UIImageView alloc] initWithFrame:CGRectMake(([[UIScreen mainScreen] bounds].size.width - 75) / 2 - 20,
                                                                            yPos,
                                                                            75, 75)]; // put in center

            if ([statusImageName isEqualToString:BUSY_ALERT_IMAGE]) { // create animated busy indicator
                UIActivityIndicatorView *indicator = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
                [indicator setFrame:statusImageView.bounds];
                indicator.center = CGPointMake(CGRectGetMidX(statusImageView.bounds), CGRectGetMidY(statusImageView.bounds));
                [indicator startAnimating];
                indicator.tag = 456;
                [statusImageView addSubview:indicator];
            }
            else {
                UIImage *statusImage = [UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:statusImageName ofType:@"png"]];
                [statusImageView setImage:statusImage];
                statusImage = nil;
            }

            statusImageView.tag = 123; // random tag - to avoid it being cleared in layoutSubviews
            [statusImageView setContentMode:UIViewContentModeScaleAspectFill];
            [self addSubview:statusImageView];
        }
    }
    [self setBackgroundColor:[UIColor whiteColor] withStrokeColor:UICOLOR_PURPLE];

    return self;
}


- (void)dealloc {
    @try {
        if (statusImageView != nil) {
            if ([statusImageView superview] != nil) {
                [statusImageView removeFromSuperview];
            }

            statusImageView = nil;
        }
    } @catch (NSException *e) {
        NSLog(@"Could not clear statusImageView: %@", e);
    } @finally {
        [super dealloc];
    }
}

@end