//
//  WriteRFIDViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 01-06-10.
//  Copyright 2010 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "WriteRFIDViewController.h"

#import "NedapHandheldAPI.h"

#define VALID_HEX @"0123456789ABCDEF"

@implementation WriteRFIDViewController

@synthesize rfidLabelValue;
@synthesize epcCode;


/*
 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if ((self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil])) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Write EPC",nil);
    
    // Make navigation bar buttons in nicer color
    self.navigationController.navigationBar.tintColor = UICOLOR_PURPLE;
    
    // Set nicer background
	self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"background_stripes" ofType:@"png"]]];
}

- (void)viewDidAppear:(BOOL)animated {
	[super viewDidAppear:animated];

	// connects all notifications that can arise when writing an EPC value
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeSucceeded:) name:@"NHPresentationRFIDDidSucceedWriteEPCNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeFailed:) name:@"NHPresentationRFIDDidFailWriteEPCZeroReadNotification" object:nil];	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeFailed:) name:@"NHPresentationRFIDDidFailWriteEPCVerificationFailedNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeFailed:) name:@"NHPresentationRFIDDidFailWriteEPCWriteFailedNotification" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(writeFailed:) name:@"NHPresentationRFIDDidFailWriteEPCMultipleReadNotification" object:nil];
    
    if (epcCode != nil)
    {
        rfidLabelValue.text = epcCode.hexString;
		[NHPresentation rfidWriteEPC:epcCode];	
    }
}

- (void)viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	
	[NHPresentation handheldStop];
	
	// disconnect with all notifications as soon as another screen is shown
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
	if (interfaceOrientation == UIInterfaceOrientationPortrait)
		return YES;
	
	if (interfaceOrientation == UIInterfaceOrientationLandscapeRight)
		return YES;
	
	return NO;
}

- (void)writeSucceeded:(NSNotification *)notification
{
	UIAlertView *writeAlertView = [[[UIAlertView alloc] initWithTitle:@"Write succeeded" message:@"Writing tag succeeded." delegate:self cancelButtonTitle:@"Dismiss" otherButtonTitles:nil] autorelease];
	[writeAlertView show];
	
    if (epcCode == nil)
    {
        NSString *epcString = self.rfidLabelValue.text;
        EPCCode *epc = [[[EPCCode alloc] initWithString:epcString] autorelease];
        [NHPresentation rfidWriteEPC:epc];	
    }
    else
    {
        [NHPresentation rfidWriteEPC:epcCode];
    }
}

- (void)writeFailed:(NSNotification *)notification
{
	NSString *message = [NSString stringWithFormat:@"Error: %@", [notification name]];
	UIAlertView *writeAlertView = [[[UIAlertView alloc] initWithTitle:@"Write failed" message:message delegate:self cancelButtonTitle:@"Dismiss" otherButtonTitles:nil] autorelease];
	[writeAlertView show];
}

- (BOOL)isValidHex:(NSString *)stringInput
{
	// check length
	if ([stringInput length] == 0)
	{
		UIAlertView* alertView = [[UIAlertView alloc] initWithTitle:@"Input error" message:@"Input cannot have a zero length." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
		[alertView show];
		[alertView release];
		return NO;
	}
	
	if ([stringInput length] % 4 > 0)
	{
		UIAlertView* alertView = [[UIAlertView alloc] initWithTitle:@"Input error" message:@"Input length should be a multiple of four." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
		[alertView show];
		[alertView release];
		return NO;		
	}
	
	// check for hex
	NSString *regex = @"[A-Fa-f0-9]+";
	NSPredicate *regextest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", regex];
	
	if ([regextest evaluateWithObject:stringInput] == NO) {
		UIAlertView* alertView = [[UIAlertView alloc] initWithTitle:@"Input error" message:@"Input is not hexadecimal." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
		[alertView show];
		[alertView release];
		return NO;	
	}
	
	return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
	if ([self isValidHex:textField.text])
	{
		[textField resignFirstResponder];
		return YES;
	}
	return NO;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
	if (textField == self.rfidLabelValue)
	{
		NSString *epcString = self.rfidLabelValue.text;
		
		EPCCode *epc = [[[EPCCode alloc] initWithString:epcString] autorelease];
		
		[NHPresentation rfidWriteEPC:epc];	
	}
}

- (void)didReceiveMemoryWarning 
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload 
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc 
{
    [super dealloc];
}


@end
