//
//  BootloaderViewController.m
//  NedapHandheldAPIDemo
//
//  Created by Danny Haak on 12-06-11.
//  Copyright 2011 N.V. Nederlandsche Apparatenfabriek "Nedap". All rights reserved.
//

#import "BootloaderViewController.h"

#import "NedapHandheldAPI.h"

@implementation BootloaderViewController

@synthesize availableVersionLabel, currentVersionLabel, progressBar, hardwareRevisionLabel, updateFirmwareButton;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (IBAction)reboot:(id)sender
{
    progressBar.hidden = NO;
    
    [NHPresentation handheldUpdateFirmware];
}

- (void)didProgressFirmware:(NSNotification *)notification
{
    NSNumber *value = [[notification userInfo] valueForKey:@"percentage"];
    progressBar.progress = [value floatValue];
}

- (void)didComplete:(NSNotification *)notification
{
    progressBar.progress = 0.0f;
    progressBar.hidden = YES;
}

- (void)updateView
{
    availableVersionLabel.text = [NSString stringWithFormat:@"Available version: %@", [NHPresentation handheldUpdateFirmwareAvailableVersion]];
    
    if ([NHPresentation handheldIsConnected])
    {
        if ([NHPresentation handheldUpdateFirmwareAvailable])
        {
            hardwareRevisionLabel.hidden = YES;
            updateFirmwareButton.enabled = YES;
        }
        else
        {
            hardwareRevisionLabel.hidden = NO;
            updateFirmwareButton.enabled = NO;
        }
        
        NSDictionary *dictionary = [NHPresentation handheldGetInformation];
        NSString *firmwareVersionString = [dictionary valueForKey:@"firmwareRevision"];
        currentVersionLabel.text = [NSString stringWithFormat:@"Current version: %@", firmwareVersionString];
        
        progressBar.progress = 0.0f;
        progressBar.hidden = YES;
    }
    else
    {
        currentVersionLabel.text = [NSString stringWithFormat:@"Current version: n/a"];
        
        progressBar.progress = 0.0f;
        progressBar.hidden = YES;
    }
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didProgressFirmware:) name:@"NHPresentationHandheldDidReceiveFirmwareUpdateProgressNotification" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didComplete:) name:@"NHPresentationHandheldDidReceiveFirmwareUpdateCompletedNotification" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didConnect:) name:@"NHPresentationHandheldDidConnectNotification" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didDisconnect:) name:@"NHPresentationHandheldDidDisconnectNotification" object:nil];
    
    [self updateView];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)didConnect:(NSNotification *)notification
{
	[self updateView];
}

- (void)didDisconnect:(NSNotification *)notification
{
	[self updateView];
}

@end
